package org.zkoss.stateless.sul;

import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.primitives.Booleans;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;
import org.zkoss.stateless.immutable.StatelessOnly;
import org.zkoss.stateless.util.ActionHandler;
import org.zkoss.zk.ui.sys.EventListenerMap;
import org.zkoss.zul.Auxheader;

/**
 * Immutable implementation of {@link IAuxheader}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IAuxheader.Builder()}.
 */
@Generated(from = "IAuxheader", generator = "Immutables")
@SuppressWarnings({"all"})
@ParametersAreNonnullByDefault
@javax.annotation.processing.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableIAuxheader<I extends IAnyGroup>
    implements IAuxheader<I> {
  private final String id;
  private final @Nullable ActionHandler action;
  private final @Nullable ImmutableList<ActionHandler> actions;
  private final boolean visible;
  private final String mold;
  private final @Nullable EventListenerMap eventListenerMap;
  private final @Nullable ImmutableMap<String, String> widgetListeners;
  private final @Nullable ImmutableMap<String, String> widgetOverrides;
  private final @Nullable ImmutableMap<String, String> clientAttributes;
  private final @Nullable String left;
  private final @Nullable String top;
  private final int zIndex;
  private final @Nullable String height;
  private final @Nullable String width;
  private final @Nullable String tooltiptext;
  private final @Nullable String zclass;
  private final @Nullable String sclass;
  private final @Nullable String style;
  private final String draggable;
  private final boolean focus;
  private final String droppable;
  private final @Nullable String vflex;
  private final @Nullable String hflex;
  private final int renderdefer;
  private final @Nullable String clientAction;
  private final @Nullable Integer tabindex;
  private final @Nullable String ctrlKeys;
  private final @Nullable String context;
  private final @Nullable String popup;
  private final @Nullable String tooltip;
  private final String label;
  private final @Nullable String iconSclass;
  private final @Nullable String[] iconSclasses;
  private final @Nullable String iconTooltip;
  private final @Nullable String[] iconTooltips;
  private final @Nullable String image;
  private final @Nullable String hoverImage;
  private final boolean preloadImage;
  private final @Nullable String valign;
  private final @Nullable String align;
  private final ImmutableList<I> children;
  private final String widgetClass;
  private final int colspan;
  private final int rowspan;

  private ImmutableIAuxheader(ImmutableIAuxheader.Builder<I> builder) {
    this.action = builder.action;
    this.actions = builder.actions == null ? null : builder.actions.build();
    this.eventListenerMap = builder.eventListenerMap;
    this.widgetListeners = builder.widgetListeners == null ? null : builder.widgetListeners.build();
    this.widgetOverrides = builder.widgetOverrides == null ? null : builder.widgetOverrides.build();
    this.clientAttributes = builder.clientAttributes == null ? null : builder.clientAttributes.build();
    this.left = builder.left;
    this.top = builder.top;
    this.height = builder.height;
    this.width = builder.width;
    this.tooltiptext = builder.tooltiptext;
    this.zclass = builder.zclass;
    this.sclass = builder.sclass;
    this.style = builder.style;
    this.vflex = builder.vflex;
    this.hflex = builder.hflex;
    this.clientAction = builder.clientAction;
    this.tabindex = builder.tabindex;
    this.ctrlKeys = builder.ctrlKeys;
    this.context = builder.context;
    this.popup = builder.popup;
    this.tooltip = builder.tooltip;
    this.valign = builder.valign;
    this.align = builder.align;
    this.children = builder.children.build();
    if (builder.id != null) {
      initShim.setId(builder.id);
    }
    if (builder.visibleIsSet()) {
      initShim.setVisible(builder.visible);
    }
    if (builder.mold != null) {
      initShim.setMold(builder.mold);
    }
    if (builder.zIndexIsSet()) {
      initShim.setZIndex(builder.zIndex);
    }
    if (builder.draggable != null) {
      initShim.setDraggable(builder.draggable);
    }
    if (builder.focusIsSet()) {
      initShim.setFocus(builder.focus);
    }
    if (builder.droppable != null) {
      initShim.setDroppable(builder.droppable);
    }
    if (builder.renderdeferIsSet()) {
      initShim.setRenderdefer(builder.renderdefer);
    }
    if (builder.label != null) {
      initShim.setLabel(builder.label);
    }
    if (builder.iconSclassIsSet()) {
      initShim.setIconSclass(builder.iconSclass);
    }
    if (builder.iconSclassesIsSet()) {
      initShim.setIconSclasses(builder.iconSclasses);
    }
    if (builder.iconTooltipIsSet()) {
      initShim.setIconTooltip(builder.iconTooltip);
    }
    if (builder.iconTooltipsIsSet()) {
      initShim.setIconTooltips(builder.iconTooltips);
    }
    if (builder.imageIsSet()) {
      initShim.setImage(builder.image);
    }
    if (builder.hoverImageIsSet()) {
      initShim.setHoverImage(builder.hoverImage);
    }
    if (builder.preloadImageIsSet()) {
      initShim.setPreloadImage(builder.preloadImage);
    }
    if (builder.widgetClass != null) {
      initShim.setWidgetClass(builder.widgetClass);
    }
    if (builder.colspanIsSet()) {
      initShim.setColspan(builder.colspan);
    }
    if (builder.rowspanIsSet()) {
      initShim.setRowspan(builder.rowspan);
    }
    this.id = initShim.getId();
    this.visible = initShim.isVisible();
    this.mold = initShim.getMold();
    this.zIndex = initShim.getZIndex();
    this.draggable = initShim.getDraggable();
    this.focus = initShim.isFocus();
    this.droppable = initShim.getDroppable();
    this.renderdefer = initShim.getRenderdefer();
    this.label = initShim.getLabel();
    this.iconSclass = initShim.getIconSclass();
    this.iconSclasses = initShim.getIconSclasses();
    this.iconTooltip = initShim.getIconTooltip();
    this.iconTooltips = initShim.getIconTooltips();
    this.image = initShim.getImage();
    this.hoverImage = initShim.getHoverImage();
    this.preloadImage = initShim.isPreloadImage();
    this.widgetClass = initShim.getWidgetClass();
    this.colspan = initShim.getColspan();
    this.rowspan = initShim.getRowspan();
    this.initShim = null;
  }

  private ImmutableIAuxheader(
      String id,
      @Nullable ActionHandler action,
      @Nullable ImmutableList<ActionHandler> actions,
      boolean visible,
      String mold,
      @Nullable EventListenerMap eventListenerMap,
      @Nullable ImmutableMap<String, String> widgetListeners,
      @Nullable ImmutableMap<String, String> widgetOverrides,
      @Nullable ImmutableMap<String, String> clientAttributes,
      @Nullable String left,
      @Nullable String top,
      int zIndex,
      @Nullable String height,
      @Nullable String width,
      @Nullable String tooltiptext,
      @Nullable String zclass,
      @Nullable String sclass,
      @Nullable String style,
      String draggable,
      boolean focus,
      String droppable,
      @Nullable String vflex,
      @Nullable String hflex,
      int renderdefer,
      @Nullable String clientAction,
      @Nullable Integer tabindex,
      @Nullable String ctrlKeys,
      @Nullable String context,
      @Nullable String popup,
      @Nullable String tooltip,
      String label,
      @Nullable String iconSclass,
      @Nullable String[] iconSclasses,
      @Nullable String iconTooltip,
      @Nullable String[] iconTooltips,
      @Nullable String image,
      @Nullable String hoverImage,
      boolean preloadImage,
      @Nullable String valign,
      @Nullable String align,
      ImmutableList<I> children,
      String widgetClass,
      int colspan,
      int rowspan) {
    this.id = id;
    this.action = action;
    this.actions = actions;
    this.visible = visible;
    this.mold = mold;
    this.eventListenerMap = eventListenerMap;
    this.widgetListeners = widgetListeners;
    this.widgetOverrides = widgetOverrides;
    this.clientAttributes = clientAttributes;
    this.left = left;
    this.top = top;
    this.zIndex = zIndex;
    this.height = height;
    this.width = width;
    this.tooltiptext = tooltiptext;
    this.zclass = zclass;
    this.sclass = sclass;
    this.style = style;
    this.draggable = draggable;
    this.focus = focus;
    this.droppable = droppable;
    this.vflex = vflex;
    this.hflex = hflex;
    this.renderdefer = renderdefer;
    this.clientAction = clientAction;
    this.tabindex = tabindex;
    this.ctrlKeys = ctrlKeys;
    this.context = context;
    this.popup = popup;
    this.tooltip = tooltip;
    this.label = label;
    this.iconSclass = iconSclass;
    this.iconSclasses = iconSclasses;
    this.iconTooltip = iconTooltip;
    this.iconTooltips = iconTooltips;
    this.image = image;
    this.hoverImage = hoverImage;
    this.preloadImage = preloadImage;
    this.valign = valign;
    this.align = align;
    this.children = children;
    this.widgetClass = widgetClass;
    this.colspan = colspan;
    this.rowspan = rowspan;
    this.initShim = null;
  }

  private static final byte STAGE_INITIALIZING = -1;
  private static final byte STAGE_UNINITIALIZED = 0;
  private static final byte STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  @Generated(from = "IAuxheader", generator = "Immutables")
  private final class InitShim {
    private byte idBuildStage = STAGE_UNINITIALIZED;
    private String id;

    String getId() {
      if (idBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (idBuildStage == STAGE_UNINITIALIZED) {
        idBuildStage = STAGE_INITIALIZING;
        this.id = Objects.requireNonNull(getIdInitialize(), "id");
        idBuildStage = STAGE_INITIALIZED;
      }
      return this.id;
    }

    void setId(String id) {
      this.id = id;
      idBuildStage = STAGE_INITIALIZED;
    }

    private byte visibleBuildStage = STAGE_UNINITIALIZED;
    private boolean visible;

    boolean isVisible() {
      if (visibleBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (visibleBuildStage == STAGE_UNINITIALIZED) {
        visibleBuildStage = STAGE_INITIALIZING;
        this.visible = isVisibleInitialize();
        visibleBuildStage = STAGE_INITIALIZED;
      }
      return this.visible;
    }

    void setVisible(boolean visible) {
      this.visible = visible;
      visibleBuildStage = STAGE_INITIALIZED;
    }

    private byte moldBuildStage = STAGE_UNINITIALIZED;
    private String mold;

    String getMold() {
      if (moldBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (moldBuildStage == STAGE_UNINITIALIZED) {
        moldBuildStage = STAGE_INITIALIZING;
        this.mold = Objects.requireNonNull(getMoldInitialize(), "mold");
        moldBuildStage = STAGE_INITIALIZED;
      }
      return this.mold;
    }

    void setMold(String mold) {
      this.mold = mold;
      moldBuildStage = STAGE_INITIALIZED;
    }

    private byte zIndexBuildStage = STAGE_UNINITIALIZED;
    private int zIndex;

    int getZIndex() {
      if (zIndexBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (zIndexBuildStage == STAGE_UNINITIALIZED) {
        zIndexBuildStage = STAGE_INITIALIZING;
        this.zIndex = getZIndexInitialize();
        zIndexBuildStage = STAGE_INITIALIZED;
      }
      return this.zIndex;
    }

    void setZIndex(int zIndex) {
      this.zIndex = zIndex;
      zIndexBuildStage = STAGE_INITIALIZED;
    }

    private byte draggableBuildStage = STAGE_UNINITIALIZED;
    private String draggable;

    String getDraggable() {
      if (draggableBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (draggableBuildStage == STAGE_UNINITIALIZED) {
        draggableBuildStage = STAGE_INITIALIZING;
        this.draggable = Objects.requireNonNull(getDraggableInitialize(), "draggable");
        draggableBuildStage = STAGE_INITIALIZED;
      }
      return this.draggable;
    }

    void setDraggable(String draggable) {
      this.draggable = draggable;
      draggableBuildStage = STAGE_INITIALIZED;
    }

    private byte focusBuildStage = STAGE_UNINITIALIZED;
    private boolean focus;

    boolean isFocus() {
      if (focusBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (focusBuildStage == STAGE_UNINITIALIZED) {
        focusBuildStage = STAGE_INITIALIZING;
        this.focus = isFocusInitialize();
        focusBuildStage = STAGE_INITIALIZED;
      }
      return this.focus;
    }

    void setFocus(boolean focus) {
      this.focus = focus;
      focusBuildStage = STAGE_INITIALIZED;
    }

    private byte droppableBuildStage = STAGE_UNINITIALIZED;
    private String droppable;

    String getDroppable() {
      if (droppableBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (droppableBuildStage == STAGE_UNINITIALIZED) {
        droppableBuildStage = STAGE_INITIALIZING;
        this.droppable = Objects.requireNonNull(getDroppableInitialize(), "droppable");
        droppableBuildStage = STAGE_INITIALIZED;
      }
      return this.droppable;
    }

    void setDroppable(String droppable) {
      this.droppable = droppable;
      droppableBuildStage = STAGE_INITIALIZED;
    }

    private byte renderdeferBuildStage = STAGE_UNINITIALIZED;
    private int renderdefer;

    int getRenderdefer() {
      if (renderdeferBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (renderdeferBuildStage == STAGE_UNINITIALIZED) {
        renderdeferBuildStage = STAGE_INITIALIZING;
        this.renderdefer = getRenderdeferInitialize();
        renderdeferBuildStage = STAGE_INITIALIZED;
      }
      return this.renderdefer;
    }

    void setRenderdefer(int renderdefer) {
      this.renderdefer = renderdefer;
      renderdeferBuildStage = STAGE_INITIALIZED;
    }

    private byte labelBuildStage = STAGE_UNINITIALIZED;
    private String label;

    String getLabel() {
      if (labelBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (labelBuildStage == STAGE_UNINITIALIZED) {
        labelBuildStage = STAGE_INITIALIZING;
        this.label = Objects.requireNonNull(getLabelInitialize(), "label");
        labelBuildStage = STAGE_INITIALIZED;
      }
      return this.label;
    }

    void setLabel(String label) {
      this.label = label;
      labelBuildStage = STAGE_INITIALIZED;
    }

    private byte iconSclassBuildStage = STAGE_UNINITIALIZED;
    private String iconSclass;

    String getIconSclass() {
      if (iconSclassBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconSclassBuildStage == STAGE_UNINITIALIZED) {
        iconSclassBuildStage = STAGE_INITIALIZING;
        this.iconSclass = getIconSclassInitialize();
        iconSclassBuildStage = STAGE_INITIALIZED;
      }
      return this.iconSclass;
    }

    void setIconSclass(String iconSclass) {
      this.iconSclass = iconSclass;
      iconSclassBuildStage = STAGE_INITIALIZED;
    }

    private byte iconSclassesBuildStage = STAGE_UNINITIALIZED;
    private String[] iconSclasses;

    String[] getIconSclasses() {
      if (iconSclassesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconSclassesBuildStage == STAGE_UNINITIALIZED) {
        iconSclassesBuildStage = STAGE_INITIALIZING;
        this.iconSclasses = getIconSclassesInitialize();
        iconSclassesBuildStage = STAGE_INITIALIZED;
      }
      return this.iconSclasses;
    }

    void setIconSclasses(String[] iconSclasses) {
      this.iconSclasses = iconSclasses;
      iconSclassesBuildStage = STAGE_INITIALIZED;
    }

    private byte iconTooltipBuildStage = STAGE_UNINITIALIZED;
    private String iconTooltip;

    String getIconTooltip() {
      if (iconTooltipBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconTooltipBuildStage == STAGE_UNINITIALIZED) {
        iconTooltipBuildStage = STAGE_INITIALIZING;
        this.iconTooltip = getIconTooltipInitialize();
        iconTooltipBuildStage = STAGE_INITIALIZED;
      }
      return this.iconTooltip;
    }

    void setIconTooltip(String iconTooltip) {
      this.iconTooltip = iconTooltip;
      iconTooltipBuildStage = STAGE_INITIALIZED;
    }

    private byte iconTooltipsBuildStage = STAGE_UNINITIALIZED;
    private String[] iconTooltips;

    String[] getIconTooltips() {
      if (iconTooltipsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconTooltipsBuildStage == STAGE_UNINITIALIZED) {
        iconTooltipsBuildStage = STAGE_INITIALIZING;
        this.iconTooltips = getIconTooltipsInitialize();
        iconTooltipsBuildStage = STAGE_INITIALIZED;
      }
      return this.iconTooltips;
    }

    void setIconTooltips(String[] iconTooltips) {
      this.iconTooltips = iconTooltips;
      iconTooltipsBuildStage = STAGE_INITIALIZED;
    }

    private byte imageBuildStage = STAGE_UNINITIALIZED;
    private String image;

    String getImage() {
      if (imageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (imageBuildStage == STAGE_UNINITIALIZED) {
        imageBuildStage = STAGE_INITIALIZING;
        this.image = getImageInitialize();
        imageBuildStage = STAGE_INITIALIZED;
      }
      return this.image;
    }

    void setImage(String image) {
      this.image = image;
      imageBuildStage = STAGE_INITIALIZED;
    }

    private byte hoverImageBuildStage = STAGE_UNINITIALIZED;
    private String hoverImage;

    String getHoverImage() {
      if (hoverImageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (hoverImageBuildStage == STAGE_UNINITIALIZED) {
        hoverImageBuildStage = STAGE_INITIALIZING;
        this.hoverImage = getHoverImageInitialize();
        hoverImageBuildStage = STAGE_INITIALIZED;
      }
      return this.hoverImage;
    }

    void setHoverImage(String hoverImage) {
      this.hoverImage = hoverImage;
      hoverImageBuildStage = STAGE_INITIALIZED;
    }

    private byte preloadImageBuildStage = STAGE_UNINITIALIZED;
    private boolean preloadImage;

    boolean isPreloadImage() {
      if (preloadImageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (preloadImageBuildStage == STAGE_UNINITIALIZED) {
        preloadImageBuildStage = STAGE_INITIALIZING;
        this.preloadImage = isPreloadImageInitialize();
        preloadImageBuildStage = STAGE_INITIALIZED;
      }
      return this.preloadImage;
    }

    void setPreloadImage(boolean preloadImage) {
      this.preloadImage = preloadImage;
      preloadImageBuildStage = STAGE_INITIALIZED;
    }

    private byte widgetClassBuildStage = STAGE_UNINITIALIZED;
    private String widgetClass;

    String getWidgetClass() {
      if (widgetClassBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (widgetClassBuildStage == STAGE_UNINITIALIZED) {
        widgetClassBuildStage = STAGE_INITIALIZING;
        this.widgetClass = Objects.requireNonNull(getWidgetClassInitialize(), "widgetClass");
        widgetClassBuildStage = STAGE_INITIALIZED;
      }
      return this.widgetClass;
    }

    void setWidgetClass(String widgetClass) {
      this.widgetClass = widgetClass;
      widgetClassBuildStage = STAGE_INITIALIZED;
    }

    private byte colspanBuildStage = STAGE_UNINITIALIZED;
    private int colspan;

    int getColspan() {
      if (colspanBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (colspanBuildStage == STAGE_UNINITIALIZED) {
        colspanBuildStage = STAGE_INITIALIZING;
        this.colspan = getColspanInitialize();
        colspanBuildStage = STAGE_INITIALIZED;
      }
      return this.colspan;
    }

    void setColspan(int colspan) {
      this.colspan = colspan;
      colspanBuildStage = STAGE_INITIALIZED;
    }

    private byte rowspanBuildStage = STAGE_UNINITIALIZED;
    private int rowspan;

    int getRowspan() {
      if (rowspanBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (rowspanBuildStage == STAGE_UNINITIALIZED) {
        rowspanBuildStage = STAGE_INITIALIZING;
        this.rowspan = getRowspanInitialize();
        rowspanBuildStage = STAGE_INITIALIZED;
      }
      return this.rowspan;
    }

    void setRowspan(int rowspan) {
      this.rowspan = rowspan;
      rowspanBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      List<String> attributes = new ArrayList<>();
      if (idBuildStage == STAGE_INITIALIZING) attributes.add("id");
      if (visibleBuildStage == STAGE_INITIALIZING) attributes.add("visible");
      if (moldBuildStage == STAGE_INITIALIZING) attributes.add("mold");
      if (zIndexBuildStage == STAGE_INITIALIZING) attributes.add("zIndex");
      if (draggableBuildStage == STAGE_INITIALIZING) attributes.add("draggable");
      if (focusBuildStage == STAGE_INITIALIZING) attributes.add("focus");
      if (droppableBuildStage == STAGE_INITIALIZING) attributes.add("droppable");
      if (renderdeferBuildStage == STAGE_INITIALIZING) attributes.add("renderdefer");
      if (labelBuildStage == STAGE_INITIALIZING) attributes.add("label");
      if (iconSclassBuildStage == STAGE_INITIALIZING) attributes.add("iconSclass");
      if (iconSclassesBuildStage == STAGE_INITIALIZING) attributes.add("iconSclasses");
      if (iconTooltipBuildStage == STAGE_INITIALIZING) attributes.add("iconTooltip");
      if (iconTooltipsBuildStage == STAGE_INITIALIZING) attributes.add("iconTooltips");
      if (imageBuildStage == STAGE_INITIALIZING) attributes.add("image");
      if (hoverImageBuildStage == STAGE_INITIALIZING) attributes.add("hoverImage");
      if (preloadImageBuildStage == STAGE_INITIALIZING) attributes.add("preloadImage");
      if (widgetClassBuildStage == STAGE_INITIALIZING) attributes.add("widgetClass");
      if (colspanBuildStage == STAGE_INITIALIZING) attributes.add("colspan");
      if (rowspanBuildStage == STAGE_INITIALIZING) attributes.add("rowspan");
      return "Cannot build IAuxheader, attribute initializers form cycle " + attributes;
    }
  }

  private String getIdInitialize() {
    return IAuxheader.super.getId();
  }

  private boolean isVisibleInitialize() {
    return IAuxheader.super.isVisible();
  }

  private String getMoldInitialize() {
    return IAuxheader.super.getMold();
  }

  private int getZIndexInitialize() {
    return IAuxheader.super.getZIndex();
  }

  private String getDraggableInitialize() {
    return IAuxheader.super.getDraggable();
  }

  private boolean isFocusInitialize() {
    return IAuxheader.super.isFocus();
  }

  private String getDroppableInitialize() {
    return IAuxheader.super.getDroppable();
  }

  private int getRenderdeferInitialize() {
    return IAuxheader.super.getRenderdefer();
  }

  private String getLabelInitialize() {
    return IAuxheader.super.getLabel();
  }

  private @Nullable String getIconSclassInitialize() {
    return IAuxheader.super.getIconSclass();
  }

  private @Nullable String[] getIconSclassesInitialize() {
    return IAuxheader.super.getIconSclasses();
  }

  private @Nullable String getIconTooltipInitialize() {
    return IAuxheader.super.getIconTooltip();
  }

  private @Nullable String[] getIconTooltipsInitialize() {
    return IAuxheader.super.getIconTooltips();
  }

  private @Nullable String getImageInitialize() {
    return IAuxheader.super.getImage();
  }

  private @Nullable String getHoverImageInitialize() {
    return IAuxheader.super.getHoverImage();
  }

  private boolean isPreloadImageInitialize() {
    return IAuxheader.super.isPreloadImage();
  }

  private String getWidgetClassInitialize() {
    return IAuxheader.super.getWidgetClass();
  }

  private int getColspanInitialize() {
    return IAuxheader.super.getColspan();
  }

  private int getRowspanInitialize() {
    return IAuxheader.super.getRowspan();
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @Override
  public String getId() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getId()
        : this.id;
  }

  /**
   * @return The value of the {@code action} attribute
   */
  @StatelessOnly
  @Override
  public @Nullable ActionHandler getAction() {
    return action;
  }

  /**
   * @return The value of the {@code actions} attribute
   */
  @StatelessOnly
  @Override
  public @Nullable ImmutableList<ActionHandler> getActions() {
    return actions;
  }

  /**
   * @return The value of the {@code visible} attribute
   */
  @Override
  public boolean isVisible() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isVisible()
        : this.visible;
  }

  /**
   * @return The value of the {@code mold} attribute
   */
  @Override
  public String getMold() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getMold()
        : this.mold;
  }

  /**
   * @return The value of the {@code eventListenerMap} attribute
   */
  @Override
  public @Nullable EventListenerMap getEventListenerMap() {
    return eventListenerMap;
  }

  /**
   * @return The value of the {@code widgetListeners} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getWidgetListeners() {
    return widgetListeners;
  }

  /**
   * @return The value of the {@code widgetOverrides} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getWidgetOverrides() {
    return widgetOverrides;
  }

  /**
   * @return The value of the {@code clientAttributes} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getClientAttributes() {
    return clientAttributes;
  }

  /**
   * @return The value of the {@code left} attribute
   */
  @Override
  public @Nullable String getLeft() {
    return left;
  }

  /**
   * @return The value of the {@code top} attribute
   */
  @Override
  public @Nullable String getTop() {
    return top;
  }

  /**
   * @return The value of the {@code zIndex} attribute
   */
  @Override
  public int getZIndex() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getZIndex()
        : this.zIndex;
  }

  /**
   * @return The value of the {@code height} attribute
   */
  @Override
  public @Nullable String getHeight() {
    return height;
  }

  /**
   * @return The value of the {@code width} attribute
   */
  @Override
  public @Nullable String getWidth() {
    return width;
  }

  /**
   * @return The value of the {@code tooltiptext} attribute
   */
  @Override
  public @Nullable String getTooltiptext() {
    return tooltiptext;
  }

  /**
   * @return The value of the {@code zclass} attribute
   */
  @Override
  public @Nullable String getZclass() {
    return zclass;
  }

  /**
   * @return The value of the {@code sclass} attribute
   */
  @Override
  public @Nullable String getSclass() {
    return sclass;
  }

  /**
   * @return The value of the {@code style} attribute
   */
  @Override
  public @Nullable String getStyle() {
    return style;
  }

  /**
   * @return The value of the {@code draggable} attribute
   */
  @Override
  public String getDraggable() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDraggable()
        : this.draggable;
  }

  /**
   * @return The value of the {@code focus} attribute
   */
  @Override
  public boolean isFocus() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isFocus()
        : this.focus;
  }

  /**
   * @return The value of the {@code droppable} attribute
   */
  @Override
  public String getDroppable() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDroppable()
        : this.droppable;
  }

  /**
   * @return The value of the {@code vflex} attribute
   */
  @Override
  public @Nullable String getVflex() {
    return vflex;
  }

  /**
   * @return The value of the {@code hflex} attribute
   */
  @Override
  public @Nullable String getHflex() {
    return hflex;
  }

  /**
   * @return The value of the {@code renderdefer} attribute
   */
  @Override
  public int getRenderdefer() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getRenderdefer()
        : this.renderdefer;
  }

  /**
   * @return The value of the {@code clientAction} attribute
   */
  @Override
  public @Nullable String getClientAction() {
    return clientAction;
  }

  /**
   * @return The value of the {@code tabindex} attribute
   */
  @Override
  public @Nullable Integer getTabindex() {
    return tabindex;
  }

  /**
   * @return The value of the {@code ctrlKeys} attribute
   */
  @Override
  public @Nullable String getCtrlKeys() {
    return ctrlKeys;
  }

  /**
   * @return The value of the {@code context} attribute
   */
  @Override
  public @Nullable String getContext() {
    return context;
  }

  /**
   * @return The value of the {@code popup} attribute
   */
  @Override
  public @Nullable String getPopup() {
    return popup;
  }

  /**
   * @return The value of the {@code tooltip} attribute
   */
  @Override
  public @Nullable String getTooltip() {
    return tooltip;
  }

  /**
   * @return The value of the {@code label} attribute
   */
  @Override
  public String getLabel() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getLabel()
        : this.label;
  }

  /**
   * @return The value of the {@code iconSclass} attribute
   */
  @Override
  public @Nullable String getIconSclass() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconSclass()
        : this.iconSclass;
  }

  /**
   * @return A cloned {@code iconSclasses} array
   */
  @Override
  public @Nullable String[] getIconSclasses() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconSclasses()
        : this.iconSclasses;
  }

  /**
   * @return The value of the {@code iconTooltip} attribute
   */
  @Override
  public @Nullable String getIconTooltip() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconTooltip()
        : this.iconTooltip;
  }

  /**
   * @return A cloned {@code iconTooltips} array
   */
  @Override
  public @Nullable String[] getIconTooltips() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconTooltips()
        : this.iconTooltips;
  }

  /**
   * @return The value of the {@code image} attribute
   */
  @Override
  public @Nullable String getImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getImage()
        : this.image;
  }

  /**
   * @return The value of the {@code hoverImage} attribute
   */
  @Override
  public @Nullable String getHoverImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getHoverImage()
        : this.hoverImage;
  }

  /**
   * @return The value of the {@code preloadImage} attribute
   */
  @StatelessOnly
  @Override
  public boolean isPreloadImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isPreloadImage()
        : this.preloadImage;
  }

  /**
   * @return The value of the {@code valign} attribute
   */
  @Override
  public @Nullable String getValign() {
    return valign;
  }

  /**
   * @return The value of the {@code align} attribute
   */
  @Override
  public @Nullable String getAlign() {
    return align;
  }

  /**
   * @return The value of the {@code children} attribute
   */
  @StatelessOnly
  @Override
  public ImmutableList<I> getChildren() {
    return children;
  }

  /**
   * @return The value of the {@code widgetClass} attribute
   */
  @Override
  public String getWidgetClass() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getWidgetClass()
        : this.widgetClass;
  }

  /**
   * @return The value of the {@code colspan} attribute
   */
  @Override
  public int getColspan() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getColspan()
        : this.colspan;
  }

  /**
   * @return The value of the {@code rowspan} attribute
   */
  @Override
  public int getRowspan() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getRowspan()
        : this.rowspan;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getId() id} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for id
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (this.id.equals(newValue)) return this;
    return validate(new ImmutableIAuxheader<>(
        newValue,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getAction() action} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for action (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withAction(@Nullable ActionHandler value) {
    if (this.action == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        value,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IAuxheader#getActions() actions}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIAuxheader<I> withActions(@Nullable ActionHandler... elements) {
    if (elements == null) {
      return validate(new ImmutableIAuxheader<>(
          this.id,
          this.action,
          null,
          this.visible,
          this.mold,
          this.eventListenerMap,
          this.widgetListeners,
          this.widgetOverrides,
          this.clientAttributes,
          this.left,
          this.top,
          this.zIndex,
          this.height,
          this.width,
          this.tooltiptext,
          this.zclass,
          this.sclass,
          this.style,
          this.draggable,
          this.focus,
          this.droppable,
          this.vflex,
          this.hflex,
          this.renderdefer,
          this.clientAction,
          this.tabindex,
          this.ctrlKeys,
          this.context,
          this.popup,
          this.tooltip,
          this.label,
          this.iconSclass,
          this.iconSclasses,
          this.iconTooltip,
          this.iconTooltips,
          this.image,
          this.hoverImage,
          this.preloadImage,
          this.valign,
          this.align,
          this.children,
          this.widgetClass,
          this.colspan,
          this.rowspan));
    }
    @Nullable ImmutableList<ActionHandler> newValue = elements == null ? null : ImmutableList.copyOf(elements);
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        newValue,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IAuxheader#getActions() actions}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of actions elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIAuxheader<I> withActions(@Nullable Iterable<? extends ActionHandler> elements) {
    if (this.actions == elements) return this;
    @Nullable ImmutableList<ActionHandler> newValue = elements == null ? null : ImmutableList.copyOf(elements);
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        newValue,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#isVisible() visible} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for visible
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withVisible(boolean value) {
    if (this.visible == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        value,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getMold() mold} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for mold
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withMold(String value) {
    String newValue = Objects.requireNonNull(value, "mold");
    if (this.mold.equals(newValue)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        newValue,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getEventListenerMap() eventListenerMap} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for eventListenerMap (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withEventListenerMap(@Nullable EventListenerMap value) {
    if (this.eventListenerMap == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        value,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by replacing the {@link IAuxheader#getWidgetListeners() widgetListeners} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the widgetListeners map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIAuxheader<I> withWidgetListeners(@Nullable Map<String, ? extends String> entries) {
    if (this.widgetListeners == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        newValue,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by replacing the {@link IAuxheader#getWidgetOverrides() widgetOverrides} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the widgetOverrides map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIAuxheader<I> withWidgetOverrides(@Nullable Map<String, ? extends String> entries) {
    if (this.widgetOverrides == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        newValue,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by replacing the {@link IAuxheader#getClientAttributes() clientAttributes} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the clientAttributes map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIAuxheader<I> withClientAttributes(@Nullable Map<String, ? extends String> entries) {
    if (this.clientAttributes == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        newValue,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getLeft() left} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for left (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withLeft(@Nullable String value) {
    if (Objects.equals(this.left, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        value,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getTop() top} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for top (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withTop(@Nullable String value) {
    if (Objects.equals(this.top, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        value,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getZIndex() zIndex} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for zIndex
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withZIndex(int value) {
    if (this.zIndex == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        value,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getHeight() height} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for height (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withHeight(@Nullable String value) {
    if (Objects.equals(this.height, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        value,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getWidth() width} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for width (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withWidth(@Nullable String value) {
    if (Objects.equals(this.width, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        value,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getTooltiptext() tooltiptext} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tooltiptext (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withTooltiptext(@Nullable String value) {
    if (Objects.equals(this.tooltiptext, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        value,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getZclass() zclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for zclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withZclass(@Nullable String value) {
    if (Objects.equals(this.zclass, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        value,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getSclass() sclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withSclass(@Nullable String value) {
    if (Objects.equals(this.sclass, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        value,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getStyle() style} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for style (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withStyle(@Nullable String value) {
    if (Objects.equals(this.style, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        value,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getDraggable() draggable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for draggable
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withDraggable(String value) {
    String newValue = Objects.requireNonNull(value, "draggable");
    if (this.draggable.equals(newValue)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        newValue,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#isFocus() focus} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for focus
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withFocus(boolean value) {
    if (this.focus == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        value,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getDroppable() droppable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for droppable
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withDroppable(String value) {
    String newValue = Objects.requireNonNull(value, "droppable");
    if (this.droppable.equals(newValue)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        newValue,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getVflex() vflex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for vflex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withVflex(@Nullable String value) {
    if (Objects.equals(this.vflex, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        value,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getHflex() hflex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for hflex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withHflex(@Nullable String value) {
    if (Objects.equals(this.hflex, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        value,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getRenderdefer() renderdefer} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for renderdefer
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withRenderdefer(int value) {
    if (this.renderdefer == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        value,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getClientAction() clientAction} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for clientAction (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withClientAction(@Nullable String value) {
    if (Objects.equals(this.clientAction, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        value,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getTabindex() tabindex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tabindex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withTabindex(@Nullable Integer value) {
    if (Objects.equals(this.tabindex, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        value,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getCtrlKeys() ctrlKeys} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for ctrlKeys (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withCtrlKeys(@Nullable String value) {
    if (Objects.equals(this.ctrlKeys, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        value,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getContext() context} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for context (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withContext(@Nullable String value) {
    if (Objects.equals(this.context, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        value,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getPopup() popup} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for popup (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withPopup(@Nullable String value) {
    if (Objects.equals(this.popup, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        value,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getTooltip() tooltip} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tooltip (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withTooltip(@Nullable String value) {
    if (Objects.equals(this.tooltip, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        value,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getLabel() label} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for label
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withLabel(String value) {
    String newValue = Objects.requireNonNull(value, "label");
    if (this.label.equals(newValue)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        newValue,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getIconSclass() iconSclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for iconSclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withIconSclass(@Nullable String value) {
    if (Objects.equals(this.iconSclass, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        value,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IAuxheader#getIconSclasses() iconSclasses}.
   * The array is cloned before being saved as attribute values.
   * @param elements The non-null elements for iconSclasses
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIAuxheader<I> withIconSclasses(@Nullable String... elements) {
    @Nullable String[] newValue = elements == null ? null : elements.clone();
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        newValue,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getIconTooltip() iconTooltip} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for iconTooltip (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withIconTooltip(@Nullable String value) {
    if (Objects.equals(this.iconTooltip, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        value,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IAuxheader#getIconTooltips() iconTooltips}.
   * The array is cloned before being saved as attribute values.
   * @param elements The non-null elements for iconTooltips
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIAuxheader<I> withIconTooltips(@Nullable String... elements) {
    @Nullable String[] newValue = elements == null ? null : elements.clone();
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        newValue,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getImage() image} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for image (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withImage(@Nullable String value) {
    if (Objects.equals(this.image, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        value,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getHoverImage() hoverImage} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for hoverImage (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withHoverImage(@Nullable String value) {
    if (Objects.equals(this.hoverImage, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        value,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#isPreloadImage() preloadImage} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for preloadImage
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withPreloadImage(boolean value) {
    if (this.preloadImage == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        value,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getValign() valign} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for valign (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withValign(@Nullable String value) {
    if (Objects.equals(this.valign, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        value,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getAlign() align} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for align (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withAlign(@Nullable String value) {
    if (Objects.equals(this.align, value)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        value,
        this.children,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IAuxheader#getChildren() children}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  @SafeVarargs @SuppressWarnings("varargs")
  public final ImmutableIAuxheader<I> withChildren(I... elements) {
    ImmutableList<I> newValue = ImmutableList.copyOf(elements);
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        newValue,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IAuxheader#getChildren() children}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of children elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIAuxheader<I> withChildren(Iterable<? extends I> elements) {
    if (this.children == elements) return this;
    ImmutableList<I> newValue = ImmutableList.copyOf(elements);
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        newValue,
        this.widgetClass,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getWidgetClass() widgetClass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for widgetClass
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withWidgetClass(String value) {
    String newValue = Objects.requireNonNull(value, "widgetClass");
    if (this.widgetClass.equals(newValue)) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        newValue,
        this.colspan,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getColspan() colspan} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for colspan
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withColspan(int value) {
    if (this.colspan == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        value,
        this.rowspan));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IAuxheader#getRowspan() rowspan} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for rowspan
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIAuxheader<I> withRowspan(int value) {
    if (this.rowspan == value) return this;
    return validate(new ImmutableIAuxheader<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.colspan,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIAuxheader} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIAuxheader<?>
        && equalTo((ImmutableIAuxheader<?>) another);
  }

  private boolean equalTo(ImmutableIAuxheader<?> another) {
    return id.equals(another.id)
        && Objects.equals(action, another.action)
        && Objects.equals(actions, another.actions)
        && visible == another.visible
        && mold.equals(another.mold)
        && Objects.equals(eventListenerMap, another.eventListenerMap)
        && Objects.equals(widgetListeners, another.widgetListeners)
        && Objects.equals(widgetOverrides, another.widgetOverrides)
        && Objects.equals(clientAttributes, another.clientAttributes)
        && Objects.equals(left, another.left)
        && Objects.equals(top, another.top)
        && zIndex == another.zIndex
        && Objects.equals(height, another.height)
        && Objects.equals(width, another.width)
        && Objects.equals(tooltiptext, another.tooltiptext)
        && Objects.equals(zclass, another.zclass)
        && Objects.equals(sclass, another.sclass)
        && Objects.equals(style, another.style)
        && draggable.equals(another.draggable)
        && focus == another.focus
        && droppable.equals(another.droppable)
        && Objects.equals(vflex, another.vflex)
        && Objects.equals(hflex, another.hflex)
        && renderdefer == another.renderdefer
        && Objects.equals(clientAction, another.clientAction)
        && Objects.equals(tabindex, another.tabindex)
        && Objects.equals(ctrlKeys, another.ctrlKeys)
        && Objects.equals(context, another.context)
        && Objects.equals(popup, another.popup)
        && Objects.equals(tooltip, another.tooltip)
        && label.equals(another.label)
        && Objects.equals(iconSclass, another.iconSclass)
        && Arrays.equals(iconSclasses, another.iconSclasses)
        && Objects.equals(iconTooltip, another.iconTooltip)
        && Arrays.equals(iconTooltips, another.iconTooltips)
        && Objects.equals(image, another.image)
        && Objects.equals(hoverImage, another.hoverImage)
        && preloadImage == another.preloadImage
        && Objects.equals(valign, another.valign)
        && Objects.equals(align, another.align)
        && children.equals(another.children)
        && widgetClass.equals(another.widgetClass)
        && colspan == another.colspan
        && rowspan == another.rowspan;
  }

  /**
   * Computes a hash code from attributes: {@code id}, {@code action}, {@code actions}, {@code visible}, {@code mold}, {@code eventListenerMap}, {@code widgetListeners}, {@code widgetOverrides}, {@code clientAttributes}, {@code left}, {@code top}, {@code zIndex}, {@code height}, {@code width}, {@code tooltiptext}, {@code zclass}, {@code sclass}, {@code style}, {@code draggable}, {@code focus}, {@code droppable}, {@code vflex}, {@code hflex}, {@code renderdefer}, {@code clientAction}, {@code tabindex}, {@code ctrlKeys}, {@code context}, {@code popup}, {@code tooltip}, {@code label}, {@code iconSclass}, {@code iconSclasses}, {@code iconTooltip}, {@code iconTooltips}, {@code image}, {@code hoverImage}, {@code preloadImage}, {@code valign}, {@code align}, {@code children}, {@code widgetClass}, {@code colspan}, {@code rowspan}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + id.hashCode();
    h += (h << 5) + Objects.hashCode(action);
    h += (h << 5) + Objects.hashCode(actions);
    h += (h << 5) + Booleans.hashCode(visible);
    h += (h << 5) + mold.hashCode();
    h += (h << 5) + Objects.hashCode(eventListenerMap);
    h += (h << 5) + Objects.hashCode(widgetListeners);
    h += (h << 5) + Objects.hashCode(widgetOverrides);
    h += (h << 5) + Objects.hashCode(clientAttributes);
    h += (h << 5) + Objects.hashCode(left);
    h += (h << 5) + Objects.hashCode(top);
    h += (h << 5) + zIndex;
    h += (h << 5) + Objects.hashCode(height);
    h += (h << 5) + Objects.hashCode(width);
    h += (h << 5) + Objects.hashCode(tooltiptext);
    h += (h << 5) + Objects.hashCode(zclass);
    h += (h << 5) + Objects.hashCode(sclass);
    h += (h << 5) + Objects.hashCode(style);
    h += (h << 5) + draggable.hashCode();
    h += (h << 5) + Booleans.hashCode(focus);
    h += (h << 5) + droppable.hashCode();
    h += (h << 5) + Objects.hashCode(vflex);
    h += (h << 5) + Objects.hashCode(hflex);
    h += (h << 5) + renderdefer;
    h += (h << 5) + Objects.hashCode(clientAction);
    h += (h << 5) + Objects.hashCode(tabindex);
    h += (h << 5) + Objects.hashCode(ctrlKeys);
    h += (h << 5) + Objects.hashCode(context);
    h += (h << 5) + Objects.hashCode(popup);
    h += (h << 5) + Objects.hashCode(tooltip);
    h += (h << 5) + label.hashCode();
    h += (h << 5) + Objects.hashCode(iconSclass);
    h += (h << 5) + Arrays.hashCode(iconSclasses);
    h += (h << 5) + Objects.hashCode(iconTooltip);
    h += (h << 5) + Arrays.hashCode(iconTooltips);
    h += (h << 5) + Objects.hashCode(image);
    h += (h << 5) + Objects.hashCode(hoverImage);
    h += (h << 5) + Booleans.hashCode(preloadImage);
    h += (h << 5) + Objects.hashCode(valign);
    h += (h << 5) + Objects.hashCode(align);
    h += (h << 5) + children.hashCode();
    h += (h << 5) + widgetClass.hashCode();
    h += (h << 5) + colspan;
    h += (h << 5) + rowspan;
    return h;
  }

  /**
   * Prints the immutable value {@code IAuxheader} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("IAuxheader")
        .omitNullValues()
        .add("id", id)
        .add("action", action)
        .add("actions", actions)
        .add("visible", visible)
        .add("mold", mold)
        .add("eventListenerMap", eventListenerMap)
        .add("widgetListeners", widgetListeners)
        .add("widgetOverrides", widgetOverrides)
        .add("clientAttributes", clientAttributes)
        .add("left", left)
        .add("top", top)
        .add("zIndex", zIndex)
        .add("height", height)
        .add("width", width)
        .add("tooltiptext", tooltiptext)
        .add("zclass", zclass)
        .add("sclass", sclass)
        .add("style", style)
        .add("draggable", draggable)
        .add("focus", focus)
        .add("droppable", droppable)
        .add("vflex", vflex)
        .add("hflex", hflex)
        .add("renderdefer", renderdefer)
        .add("clientAction", clientAction)
        .add("tabindex", tabindex)
        .add("ctrlKeys", ctrlKeys)
        .add("context", context)
        .add("popup", popup)
        .add("tooltip", tooltip)
        .add("label", label)
        .add("iconSclass", iconSclass)
        .add("iconSclasses", Arrays.toString(iconSclasses))
        .add("iconTooltip", iconTooltip)
        .add("iconTooltips", Arrays.toString(iconTooltips))
        .add("image", image)
        .add("hoverImage", hoverImage)
        .add("preloadImage", preloadImage)
        .add("valign", valign)
        .add("align", align)
        .add("children", children)
        .add("widgetClass", widgetClass)
        .add("colspan", colspan)
        .add("rowspan", rowspan)
        .toString();
  }

  private transient volatile long lazyInitBitmap;

  private static final long Z_K_TYPE_LAZY_INIT_BIT = 0x1L;

  private transient Class<Auxheader> zKType;

  /**
   * {@inheritDoc}
   * <p>
   * Returns a lazily initialized value of the {@link IAuxheader#getZKType() zKType} attribute.
   * Initialized once and only once and stored for subsequent access with proper synchronization.
   * In case of any exception or error thrown by the lazy value initializer,
   * the result will not be memoised (i.e. remembered) and on next call computation
   * will be attempted again.
   * @return A lazily initialized value of the {@code zKType} attribute
   */
  @Override
  public Class<Auxheader> getZKType() {
    if ((lazyInitBitmap & Z_K_TYPE_LAZY_INIT_BIT) == 0) {
      synchronized (this) {
        if ((lazyInitBitmap & Z_K_TYPE_LAZY_INIT_BIT) == 0) {
          this.zKType = Objects.requireNonNull(IAuxheader.super.getZKType(), "zKType");
          lazyInitBitmap |= Z_K_TYPE_LAZY_INIT_BIT;
        }
      }
    }
    return zKType;
  }

  private static <I extends IAnyGroup> ImmutableIAuxheader<I> validate(ImmutableIAuxheader<I> instance) {
    instance.checkFlex();
    instance.checkWidth();
    instance.checkHeight();
    instance.checkColspanAndRowSpan();
    instance.checkHflexAndWidth();
    instance.checkVflexAndHeight();
    instance = (ImmutableIAuxheader<I>) instance.checkDroppable();
    instance = (ImmutableIAuxheader<I>) instance.checkDraggable();
    instance.checkActions();
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link IAuxheader} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param <I> generic parameter I
   * @param instance The instance to copy
   * @return A copied immutable IAuxheader instance
   */
  public static <I extends IAnyGroup> IAuxheader<I> copyOf(IAuxheader<I> instance) {
    if (instance instanceof ImmutableIAuxheader<?>) {
      return (ImmutableIAuxheader<I>) instance;
    }
    return new IAuxheader.Builder<I>()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link IAuxheader IAuxheader}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "IAuxheader", generator = "Immutables")
  @NotThreadSafe
  public static class Builder<I extends IAnyGroup> {
    private static final long OPT_BIT_VISIBLE = 0x1L;
    private static final long OPT_BIT_Z_INDEX = 0x2L;
    private static final long OPT_BIT_FOCUS = 0x4L;
    private static final long OPT_BIT_RENDERDEFER = 0x8L;
    private static final long OPT_BIT_ICON_SCLASS = 0x10L;
    private static final long OPT_BIT_ICON_SCLASSES = 0x20L;
    private static final long OPT_BIT_ICON_TOOLTIP = 0x40L;
    private static final long OPT_BIT_ICON_TOOLTIPS = 0x80L;
    private static final long OPT_BIT_IMAGE = 0x100L;
    private static final long OPT_BIT_HOVER_IMAGE = 0x200L;
    private static final long OPT_BIT_PRELOAD_IMAGE = 0x400L;
    private static final long OPT_BIT_COLSPAN = 0x800L;
    private static final long OPT_BIT_ROWSPAN = 0x1000L;
    private long optBits;

    private @Nullable String id;
    private @Nullable ActionHandler action;
    private ImmutableList.Builder<ActionHandler> actions = null;
    private boolean visible;
    private @Nullable String mold;
    private @Nullable EventListenerMap eventListenerMap;
    private ImmutableMap.Builder<String, String> widgetListeners = null;
    private ImmutableMap.Builder<String, String> widgetOverrides = null;
    private ImmutableMap.Builder<String, String> clientAttributes = null;
    private @Nullable String left;
    private @Nullable String top;
    private int zIndex;
    private @Nullable String height;
    private @Nullable String width;
    private @Nullable String tooltiptext;
    private @Nullable String zclass;
    private @Nullable String sclass;
    private @Nullable String style;
    private @Nullable String draggable;
    private boolean focus;
    private @Nullable String droppable;
    private @Nullable String vflex;
    private @Nullable String hflex;
    private int renderdefer;
    private @Nullable String clientAction;
    private @Nullable Integer tabindex;
    private @Nullable String ctrlKeys;
    private @Nullable String context;
    private @Nullable String popup;
    private @Nullable String tooltip;
    private @Nullable String label;
    private @Nullable String iconSclass;
    private @Nullable String[] iconSclasses;
    private @Nullable String iconTooltip;
    private @Nullable String[] iconTooltips;
    private @Nullable String image;
    private @Nullable String hoverImage;
    private boolean preloadImage;
    private @Nullable String valign;
    private @Nullable String align;
    private ImmutableList.Builder<I> children = ImmutableList.builder();
    private @Nullable String widgetClass;
    private int colspan;
    private int rowspan;

    /**
     * Creates a builder for {@link IAuxheader IAuxheader} instances.
     * <pre>
     * new IAuxheader.Builder&amp;lt;I&amp;gt;()
     *    .setId(String) // optional {@link IAuxheader#getId() id}
     *    .setAction(org.zkoss.stateless.util.ActionHandler | null) // nullable {@link IAuxheader#getAction() action}
     *    .setActions(List&amp;lt;org.zkoss.stateless.util.ActionHandler&amp;gt; | null) // nullable {@link IAuxheader#getActions() actions}
     *    .setVisible(boolean) // optional {@link IAuxheader#isVisible() visible}
     *    .setMold(String) // optional {@link IAuxheader#getMold() mold}
     *    .setEventListenerMap(org.zkoss.zk.ui.sys.EventListenerMap | null) // nullable {@link IAuxheader#getEventListenerMap() eventListenerMap}
     *    .setWidgetListeners(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link IAuxheader#getWidgetListeners() widgetListeners}
     *    .setWidgetOverrides(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link IAuxheader#getWidgetOverrides() widgetOverrides}
     *    .setClientAttributes(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link IAuxheader#getClientAttributes() clientAttributes}
     *    .setLeft(String | null) // nullable {@link IAuxheader#getLeft() left}
     *    .setTop(String | null) // nullable {@link IAuxheader#getTop() top}
     *    .setZIndex(int) // optional {@link IAuxheader#getZIndex() zIndex}
     *    .setHeight(String | null) // nullable {@link IAuxheader#getHeight() height}
     *    .setWidth(String | null) // nullable {@link IAuxheader#getWidth() width}
     *    .setTooltiptext(String | null) // nullable {@link IAuxheader#getTooltiptext() tooltiptext}
     *    .setZclass(String | null) // nullable {@link IAuxheader#getZclass() zclass}
     *    .setSclass(String | null) // nullable {@link IAuxheader#getSclass() sclass}
     *    .setStyle(String | null) // nullable {@link IAuxheader#getStyle() style}
     *    .setDraggable(String) // optional {@link IAuxheader#getDraggable() draggable}
     *    .setFocus(boolean) // optional {@link IAuxheader#isFocus() focus}
     *    .setDroppable(String) // optional {@link IAuxheader#getDroppable() droppable}
     *    .setVflex(String | null) // nullable {@link IAuxheader#getVflex() vflex}
     *    .setHflex(String | null) // nullable {@link IAuxheader#getHflex() hflex}
     *    .setRenderdefer(int) // optional {@link IAuxheader#getRenderdefer() renderdefer}
     *    .setClientAction(String | null) // nullable {@link IAuxheader#getClientAction() clientAction}
     *    .setTabindex(Integer | null) // nullable {@link IAuxheader#getTabindex() tabindex}
     *    .setCtrlKeys(String | null) // nullable {@link IAuxheader#getCtrlKeys() ctrlKeys}
     *    .setContext(String | null) // nullable {@link IAuxheader#getContext() context}
     *    .setPopup(String | null) // nullable {@link IAuxheader#getPopup() popup}
     *    .setTooltip(String | null) // nullable {@link IAuxheader#getTooltip() tooltip}
     *    .setLabel(String) // optional {@link IAuxheader#getLabel() label}
     *    .setIconSclass(String | null) // nullable {@link IAuxheader#getIconSclass() iconSclass}
     *    .setIconSclasses(String[] | null) // nullable {@link IAuxheader#getIconSclasses() iconSclasses}
     *    .setIconTooltip(String | null) // nullable {@link IAuxheader#getIconTooltip() iconTooltip}
     *    .setIconTooltips(String[] | null) // nullable {@link IAuxheader#getIconTooltips() iconTooltips}
     *    .setImage(String | null) // nullable {@link IAuxheader#getImage() image}
     *    .setHoverImage(String | null) // nullable {@link IAuxheader#getHoverImage() hoverImage}
     *    .setPreloadImage(boolean) // optional {@link IAuxheader#isPreloadImage() preloadImage}
     *    .setValign(String | null) // nullable {@link IAuxheader#getValign() valign}
     *    .setAlign(String | null) // nullable {@link IAuxheader#getAlign() align}
     *    .addChildren|addAllChildren(I) // {@link IAuxheader#getChildren() children} elements
     *    .setWidgetClass(String) // optional {@link IAuxheader#getWidgetClass() widgetClass}
     *    .setColspan(int) // optional {@link IAuxheader#getColspan() colspan}
     *    .setRowspan(int) // optional {@link IAuxheader#getRowspan() rowspan}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof IAuxheader.Builder<?>)) {
        throw new UnsupportedOperationException("Use: new IAuxheader.Builder<I>()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code IAuxheader} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> from(IAuxheader<I> instance) {
      Objects.requireNonNull(instance, "instance");
      setId(instance.getId());
      @Nullable ActionHandler actionValue = instance.getAction();
      if (actionValue != null) {
        setAction(actionValue);
      }
      @Nullable List<ActionHandler> actionsValue = instance.getActions();
      if (actionsValue != null) {
        addAllActions(actionsValue);
      }
      setVisible(instance.isVisible());
      setMold(instance.getMold());
      @Nullable EventListenerMap eventListenerMapValue = instance.getEventListenerMap();
      if (eventListenerMapValue != null) {
        setEventListenerMap(eventListenerMapValue);
      }
      @Nullable Map<String, String> widgetListenersValue = instance.getWidgetListeners();
      if (widgetListenersValue != null) {
        putAllWidgetListeners(widgetListenersValue);
      }
      @Nullable Map<String, String> widgetOverridesValue = instance.getWidgetOverrides();
      if (widgetOverridesValue != null) {
        putAllWidgetOverrides(widgetOverridesValue);
      }
      @Nullable Map<String, String> clientAttributesValue = instance.getClientAttributes();
      if (clientAttributesValue != null) {
        putAllClientAttributes(clientAttributesValue);
      }
      @Nullable String leftValue = instance.getLeft();
      if (leftValue != null) {
        setLeft(leftValue);
      }
      @Nullable String topValue = instance.getTop();
      if (topValue != null) {
        setTop(topValue);
      }
      setZIndex(instance.getZIndex());
      @Nullable String heightValue = instance.getHeight();
      if (heightValue != null) {
        setHeight(heightValue);
      }
      @Nullable String widthValue = instance.getWidth();
      if (widthValue != null) {
        setWidth(widthValue);
      }
      @Nullable String tooltiptextValue = instance.getTooltiptext();
      if (tooltiptextValue != null) {
        setTooltiptext(tooltiptextValue);
      }
      @Nullable String zclassValue = instance.getZclass();
      if (zclassValue != null) {
        setZclass(zclassValue);
      }
      @Nullable String sclassValue = instance.getSclass();
      if (sclassValue != null) {
        setSclass(sclassValue);
      }
      @Nullable String styleValue = instance.getStyle();
      if (styleValue != null) {
        setStyle(styleValue);
      }
      setDraggable(instance.getDraggable());
      setFocus(instance.isFocus());
      setDroppable(instance.getDroppable());
      @Nullable String vflexValue = instance.getVflex();
      if (vflexValue != null) {
        setVflex(vflexValue);
      }
      @Nullable String hflexValue = instance.getHflex();
      if (hflexValue != null) {
        setHflex(hflexValue);
      }
      setRenderdefer(instance.getRenderdefer());
      @Nullable String clientActionValue = instance.getClientAction();
      if (clientActionValue != null) {
        setClientAction(clientActionValue);
      }
      @Nullable Integer tabindexValue = instance.getTabindex();
      if (tabindexValue != null) {
        setTabindex(tabindexValue);
      }
      @Nullable String ctrlKeysValue = instance.getCtrlKeys();
      if (ctrlKeysValue != null) {
        setCtrlKeys(ctrlKeysValue);
      }
      @Nullable String contextValue = instance.getContext();
      if (contextValue != null) {
        setContext(contextValue);
      }
      @Nullable String popupValue = instance.getPopup();
      if (popupValue != null) {
        setPopup(popupValue);
      }
      @Nullable String tooltipValue = instance.getTooltip();
      if (tooltipValue != null) {
        setTooltip(tooltipValue);
      }
      setLabel(instance.getLabel());
      @Nullable String iconSclassValue = instance.getIconSclass();
      if (iconSclassValue != null) {
        setIconSclass(iconSclassValue);
      }
      @Nullable String[] iconSclassesValue = instance.getIconSclasses();
      if (iconSclassesValue != null) {
        setIconSclasses(iconSclassesValue);
      }
      @Nullable String iconTooltipValue = instance.getIconTooltip();
      if (iconTooltipValue != null) {
        setIconTooltip(iconTooltipValue);
      }
      @Nullable String[] iconTooltipsValue = instance.getIconTooltips();
      if (iconTooltipsValue != null) {
        setIconTooltips(iconTooltipsValue);
      }
      @Nullable String imageValue = instance.getImage();
      if (imageValue != null) {
        setImage(imageValue);
      }
      @Nullable String hoverImageValue = instance.getHoverImage();
      if (hoverImageValue != null) {
        setHoverImage(hoverImageValue);
      }
      setPreloadImage(instance.isPreloadImage());
      @Nullable String valignValue = instance.getValign();
      if (valignValue != null) {
        setValign(valignValue);
      }
      @Nullable String alignValue = instance.getAlign();
      if (alignValue != null) {
        setAlign(alignValue);
      }
      addAllChildren(instance.getChildren());
      setWidgetClass(instance.getWidgetClass());
      setColspan(instance.getColspan());
      setRowspan(instance.getRowspan());
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getId() id} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getId() id}.</em>
     * @param id The value for id 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setId(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getAction() action} attribute.
     * @param action The value for action (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setAction(@Nullable ActionHandler action) {
      this.action = action;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Adds one element to {@link IAuxheader#getActions() actions} list.
     * @param element A actions element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> addActions(ActionHandler element) {
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.add(element);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Adds elements to {@link IAuxheader#getActions() actions} list.
     * @param elements An array of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> addActions(ActionHandler... elements) {
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.add(elements);
      return (IAuxheader.Builder<I>) this;
    }


    /**
     * Sets or replaces all elements for {@link IAuxheader#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setActions(@Nullable Iterable<? extends ActionHandler> elements) {
      if (elements == null) {
        this.actions = null;
        return (IAuxheader.Builder<I>) this;
      }
      this.actions = ImmutableList.builder();
      return addAllActions(elements);
    }

    /**
     * Adds elements to {@link IAuxheader#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> addAllActions(Iterable<? extends ActionHandler> elements) {
      Objects.requireNonNull(elements, "actions element");
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.addAll(elements);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#isVisible() visible} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#isVisible() visible}.</em>
     * @param visible The value for visible 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setVisible(boolean visible) {
      this.visible = visible;
      optBits |= OPT_BIT_VISIBLE;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getMold() mold} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getMold() mold}.</em>
     * @param mold The value for mold 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setMold(String mold) {
      this.mold = Objects.requireNonNull(mold, "mold");
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getEventListenerMap() eventListenerMap} attribute.
     * @param eventListenerMap The value for eventListenerMap (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setEventListenerMap(@Nullable EventListenerMap eventListenerMap) {
      this.eventListenerMap = eventListenerMap;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link IAuxheader#getWidgetListeners() widgetListeners} map.
     * @param key The key in the widgetListeners map
     * @param value The associated value in the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putWidgetListeners(String key, String value) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.put(key, value);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link IAuxheader#getWidgetListeners() widgetListeners} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putWidgetListeners(Map.Entry<String, ? extends String> entry) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.put(entry);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link IAuxheader#getWidgetListeners() widgetListeners} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setWidgetListeners(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.widgetListeners = null;
        return (IAuxheader.Builder<I>) this;
      }
      this.widgetListeners = ImmutableMap.builder();
      return putAllWidgetListeners(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link IAuxheader#getWidgetListeners() widgetListeners} map. Nulls are not permitted
     * @param entries The entries that will be added to the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putAllWidgetListeners(Map<String, ? extends String> entries) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.putAll(entries);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link IAuxheader#getWidgetOverrides() widgetOverrides} map.
     * @param key The key in the widgetOverrides map
     * @param value The associated value in the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putWidgetOverrides(String key, String value) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.put(key, value);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link IAuxheader#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putWidgetOverrides(Map.Entry<String, ? extends String> entry) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.put(entry);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link IAuxheader#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setWidgetOverrides(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.widgetOverrides = null;
        return (IAuxheader.Builder<I>) this;
      }
      this.widgetOverrides = ImmutableMap.builder();
      return putAllWidgetOverrides(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link IAuxheader#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted
     * @param entries The entries that will be added to the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putAllWidgetOverrides(Map<String, ? extends String> entries) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.putAll(entries);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link IAuxheader#getClientAttributes() clientAttributes} map.
     * @param key The key in the clientAttributes map
     * @param value The associated value in the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putClientAttributes(String key, String value) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.put(key, value);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link IAuxheader#getClientAttributes() clientAttributes} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putClientAttributes(Map.Entry<String, ? extends String> entry) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.put(entry);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link IAuxheader#getClientAttributes() clientAttributes} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setClientAttributes(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.clientAttributes = null;
        return (IAuxheader.Builder<I>) this;
      }
      this.clientAttributes = ImmutableMap.builder();
      return putAllClientAttributes(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link IAuxheader#getClientAttributes() clientAttributes} map. Nulls are not permitted
     * @param entries The entries that will be added to the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> putAllClientAttributes(Map<String, ? extends String> entries) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.putAll(entries);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getLeft() left} attribute.
     * @param left The value for left (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setLeft(@Nullable String left) {
      this.left = left;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getTop() top} attribute.
     * @param top The value for top (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setTop(@Nullable String top) {
      this.top = top;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getZIndex() zIndex} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getZIndex() zIndex}.</em>
     * @param zIndex The value for zIndex 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setZIndex(int zIndex) {
      this.zIndex = zIndex;
      optBits |= OPT_BIT_Z_INDEX;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getHeight() height} attribute.
     * @param height The value for height (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setHeight(@Nullable String height) {
      this.height = height;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getWidth() width} attribute.
     * @param width The value for width (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setWidth(@Nullable String width) {
      this.width = width;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getTooltiptext() tooltiptext} attribute.
     * @param tooltiptext The value for tooltiptext (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setTooltiptext(@Nullable String tooltiptext) {
      this.tooltiptext = tooltiptext;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getZclass() zclass} attribute.
     * @param zclass The value for zclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setZclass(@Nullable String zclass) {
      this.zclass = zclass;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getSclass() sclass} attribute.
     * @param sclass The value for sclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setSclass(@Nullable String sclass) {
      this.sclass = sclass;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getStyle() style} attribute.
     * @param style The value for style (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setStyle(@Nullable String style) {
      this.style = style;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getDraggable() draggable} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getDraggable() draggable}.</em>
     * @param draggable The value for draggable 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setDraggable(String draggable) {
      this.draggable = Objects.requireNonNull(draggable, "draggable");
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#isFocus() focus} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#isFocus() focus}.</em>
     * @param focus The value for focus 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setFocus(boolean focus) {
      this.focus = focus;
      optBits |= OPT_BIT_FOCUS;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getDroppable() droppable} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getDroppable() droppable}.</em>
     * @param droppable The value for droppable 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setDroppable(String droppable) {
      this.droppable = Objects.requireNonNull(droppable, "droppable");
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getVflex() vflex} attribute.
     * @param vflex The value for vflex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setVflex(@Nullable String vflex) {
      this.vflex = vflex;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getHflex() hflex} attribute.
     * @param hflex The value for hflex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setHflex(@Nullable String hflex) {
      this.hflex = hflex;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getRenderdefer() renderdefer} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getRenderdefer() renderdefer}.</em>
     * @param renderdefer The value for renderdefer 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setRenderdefer(int renderdefer) {
      this.renderdefer = renderdefer;
      optBits |= OPT_BIT_RENDERDEFER;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getClientAction() clientAction} attribute.
     * @param clientAction The value for clientAction (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setClientAction(@Nullable String clientAction) {
      this.clientAction = clientAction;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getTabindex() tabindex} attribute.
     * @param tabindex The value for tabindex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setTabindex(@Nullable Integer tabindex) {
      this.tabindex = tabindex;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getCtrlKeys() ctrlKeys} attribute.
     * @param ctrlKeys The value for ctrlKeys (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setCtrlKeys(@Nullable String ctrlKeys) {
      this.ctrlKeys = ctrlKeys;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getContext() context} attribute.
     * @param context The value for context (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setContext(@Nullable String context) {
      this.context = context;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getPopup() popup} attribute.
     * @param popup The value for popup (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setPopup(@Nullable String popup) {
      this.popup = popup;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getTooltip() tooltip} attribute.
     * @param tooltip The value for tooltip (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setTooltip(@Nullable String tooltip) {
      this.tooltip = tooltip;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getLabel() label} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getLabel() label}.</em>
     * @param label The value for label 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setLabel(String label) {
      this.label = Objects.requireNonNull(label, "label");
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getIconSclass() iconSclass} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getIconSclass() iconSclass}.</em>
     * @param iconSclass The value for iconSclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setIconSclass(@Nullable String iconSclass) {
      this.iconSclass = iconSclass;
      optBits |= OPT_BIT_ICON_SCLASS;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getIconSclasses() iconSclasses} attribute.
     * <p><em>If not set, this attribute will have a default value as defined by {@link IAuxheader#getIconSclasses() iconSclasses}.</em>
     * @param iconSclasses The elements for iconSclasses
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setIconSclasses(String... iconSclasses) {
      this.iconSclasses = iconSclasses;
      optBits |= OPT_BIT_ICON_SCLASSES;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getIconTooltip() iconTooltip} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getIconTooltip() iconTooltip}.</em>
     * @param iconTooltip The value for iconTooltip (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setIconTooltip(@Nullable String iconTooltip) {
      this.iconTooltip = iconTooltip;
      optBits |= OPT_BIT_ICON_TOOLTIP;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getIconTooltips() iconTooltips} attribute.
     * <p><em>If not set, this attribute will have a default value as defined by {@link IAuxheader#getIconTooltips() iconTooltips}.</em>
     * @param iconTooltips The elements for iconTooltips
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setIconTooltips(String... iconTooltips) {
      this.iconTooltips = iconTooltips;
      optBits |= OPT_BIT_ICON_TOOLTIPS;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getImage() image} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getImage() image}.</em>
     * @param image The value for image (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setImage(@Nullable String image) {
      this.image = image;
      optBits |= OPT_BIT_IMAGE;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getHoverImage() hoverImage} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getHoverImage() hoverImage}.</em>
     * @param hoverImage The value for hoverImage (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setHoverImage(@Nullable String hoverImage) {
      this.hoverImage = hoverImage;
      optBits |= OPT_BIT_HOVER_IMAGE;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#isPreloadImage() preloadImage} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#isPreloadImage() preloadImage}.</em>
     * @param preloadImage The value for preloadImage 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setPreloadImage(boolean preloadImage) {
      this.preloadImage = preloadImage;
      optBits |= OPT_BIT_PRELOAD_IMAGE;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getValign() valign} attribute.
     * @param valign The value for valign (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setValign(@Nullable String valign) {
      this.valign = valign;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getAlign() align} attribute.
     * @param align The value for align (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setAlign(@Nullable String align) {
      this.align = align;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Adds one element to {@link IAuxheader#getChildren() children} list.
     * @param element A children element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> addChildren(I element) {
      this.children.add(element);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Adds elements to {@link IAuxheader#getChildren() children} list.
     * @param elements An array of children elements
     * @return {@code this} builder for use in a chained invocation
     */
    @SafeVarargs @SuppressWarnings("varargs")
    public final IAuxheader.Builder<I> addChildren(I... elements) {
      this.children.add(elements);
      return (IAuxheader.Builder<I>) this;
    }


    /**
     * Sets or replaces all elements for {@link IAuxheader#getChildren() children} list.
     * @param elements An iterable of children elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setChildren(Iterable<? extends I> elements) {
      this.children = ImmutableList.builder();
      return addAllChildren(elements);
    }

    /**
     * Adds elements to {@link IAuxheader#getChildren() children} list.
     * @param elements An iterable of children elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> addAllChildren(Iterable<? extends I> elements) {
      this.children.addAll(elements);
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getWidgetClass() widgetClass} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getWidgetClass() widgetClass}.</em>
     * @param widgetClass The value for widgetClass 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setWidgetClass(String widgetClass) {
      this.widgetClass = Objects.requireNonNull(widgetClass, "widgetClass");
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getColspan() colspan} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getColspan() colspan}.</em>
     * @param colspan The value for colspan 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setColspan(int colspan) {
      this.colspan = colspan;
      optBits |= OPT_BIT_COLSPAN;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link IAuxheader#getRowspan() rowspan} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IAuxheader#getRowspan() rowspan}.</em>
     * @param rowspan The value for rowspan 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IAuxheader.Builder<I> setRowspan(int rowspan) {
      this.rowspan = rowspan;
      optBits |= OPT_BIT_ROWSPAN;
      return (IAuxheader.Builder<I>) this;
    }

    /**
     * Builds a new {@link IAuxheader IAuxheader}.
     * @return An immutable instance of IAuxheader
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public IAuxheader<I> build() {
      return ImmutableIAuxheader.validate(new ImmutableIAuxheader<I>(this));
    }

    private boolean visibleIsSet() {
      return (optBits & OPT_BIT_VISIBLE) != 0;
    }

    private boolean zIndexIsSet() {
      return (optBits & OPT_BIT_Z_INDEX) != 0;
    }

    private boolean focusIsSet() {
      return (optBits & OPT_BIT_FOCUS) != 0;
    }

    private boolean renderdeferIsSet() {
      return (optBits & OPT_BIT_RENDERDEFER) != 0;
    }

    private boolean iconSclassIsSet() {
      return (optBits & OPT_BIT_ICON_SCLASS) != 0;
    }

    private boolean iconSclassesIsSet() {
      return (optBits & OPT_BIT_ICON_SCLASSES) != 0;
    }

    private boolean iconTooltipIsSet() {
      return (optBits & OPT_BIT_ICON_TOOLTIP) != 0;
    }

    private boolean iconTooltipsIsSet() {
      return (optBits & OPT_BIT_ICON_TOOLTIPS) != 0;
    }

    private boolean imageIsSet() {
      return (optBits & OPT_BIT_IMAGE) != 0;
    }

    private boolean hoverImageIsSet() {
      return (optBits & OPT_BIT_HOVER_IMAGE) != 0;
    }

    private boolean preloadImageIsSet() {
      return (optBits & OPT_BIT_PRELOAD_IMAGE) != 0;
    }

    private boolean colspanIsSet() {
      return (optBits & OPT_BIT_COLSPAN) != 0;
    }

    private boolean rowspanIsSet() {
      return (optBits & OPT_BIT_ROWSPAN) != 0;
    }
  }
}
