package org.zkoss.stateless.sul;

import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.primitives.Booleans;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;
import org.zkoss.stateless.immutable.StatelessOnly;
import org.zkoss.stateless.util.ActionHandler;
import org.zkoss.zk.ui.sys.EventListenerMap;
import org.zkoss.zul.Menuitem;

/**
 * Immutable implementation of {@link IMenuitem}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IMenuitem.Builder()}.
 */
@Generated(from = "IMenuitem", generator = "Immutables")
@SuppressWarnings({"all"})
@ParametersAreNonnullByDefault
@javax.annotation.processing.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableIMenuitem implements IMenuitem {
  private final String id;
  private final @Nullable ActionHandler action;
  private final @Nullable ImmutableList<ActionHandler> actions;
  private final boolean visible;
  private final String mold;
  private final @Nullable EventListenerMap eventListenerMap;
  private final @Nullable ImmutableMap<String, String> widgetListeners;
  private final @Nullable ImmutableMap<String, String> widgetOverrides;
  private final @Nullable ImmutableMap<String, String> clientAttributes;
  private final @Nullable String left;
  private final @Nullable String top;
  private final int zIndex;
  private final @Nullable String height;
  private final @Nullable String width;
  private final @Nullable String tooltiptext;
  private final @Nullable String zclass;
  private final @Nullable String sclass;
  private final @Nullable String style;
  private final String draggable;
  private final boolean focus;
  private final String droppable;
  private final @Nullable String vflex;
  private final @Nullable String hflex;
  private final int renderdefer;
  private final @Nullable String clientAction;
  private final @Nullable Integer tabindex;
  private final @Nullable String ctrlKeys;
  private final @Nullable String context;
  private final @Nullable String popup;
  private final @Nullable String tooltip;
  private final String label;
  private final @Nullable String iconSclass;
  private final @Nullable String[] iconSclasses;
  private final @Nullable String iconTooltip;
  private final @Nullable String[] iconTooltips;
  private final @Nullable String image;
  private final @Nullable String hoverImage;
  private final boolean preloadImage;
  private final boolean disabled;
  private final String widgetClass;
  private final @Nullable String autodisable;
  private final @Nullable String href;
  private final @Nullable String target;
  private final @Nullable String upload;
  private final boolean checkmark;
  private final boolean checked;
  private final boolean autocheck;

  private ImmutableIMenuitem(ImmutableIMenuitem.Builder builder) {
    this.action = builder.action;
    this.actions = builder.actions == null ? null : builder.actions.build();
    this.eventListenerMap = builder.eventListenerMap;
    this.widgetListeners = builder.widgetListeners == null ? null : builder.widgetListeners.build();
    this.widgetOverrides = builder.widgetOverrides == null ? null : builder.widgetOverrides.build();
    this.clientAttributes = builder.clientAttributes == null ? null : builder.clientAttributes.build();
    this.left = builder.left;
    this.top = builder.top;
    this.height = builder.height;
    this.width = builder.width;
    this.tooltiptext = builder.tooltiptext;
    this.zclass = builder.zclass;
    this.sclass = builder.sclass;
    this.style = builder.style;
    this.vflex = builder.vflex;
    this.hflex = builder.hflex;
    this.clientAction = builder.clientAction;
    this.tabindex = builder.tabindex;
    this.ctrlKeys = builder.ctrlKeys;
    this.context = builder.context;
    this.popup = builder.popup;
    this.tooltip = builder.tooltip;
    this.autodisable = builder.autodisable;
    this.href = builder.href;
    this.target = builder.target;
    this.upload = builder.upload;
    if (builder.id != null) {
      initShim.setId(builder.id);
    }
    if (builder.visibleIsSet()) {
      initShim.setVisible(builder.visible);
    }
    if (builder.mold != null) {
      initShim.setMold(builder.mold);
    }
    if (builder.zIndexIsSet()) {
      initShim.setZIndex(builder.zIndex);
    }
    if (builder.draggable != null) {
      initShim.setDraggable(builder.draggable);
    }
    if (builder.focusIsSet()) {
      initShim.setFocus(builder.focus);
    }
    if (builder.droppable != null) {
      initShim.setDroppable(builder.droppable);
    }
    if (builder.renderdeferIsSet()) {
      initShim.setRenderdefer(builder.renderdefer);
    }
    if (builder.label != null) {
      initShim.setLabel(builder.label);
    }
    if (builder.iconSclassIsSet()) {
      initShim.setIconSclass(builder.iconSclass);
    }
    if (builder.iconSclassesIsSet()) {
      initShim.setIconSclasses(builder.iconSclasses);
    }
    if (builder.iconTooltipIsSet()) {
      initShim.setIconTooltip(builder.iconTooltip);
    }
    if (builder.iconTooltipsIsSet()) {
      initShim.setIconTooltips(builder.iconTooltips);
    }
    if (builder.imageIsSet()) {
      initShim.setImage(builder.image);
    }
    if (builder.hoverImageIsSet()) {
      initShim.setHoverImage(builder.hoverImage);
    }
    if (builder.preloadImageIsSet()) {
      initShim.setPreloadImage(builder.preloadImage);
    }
    if (builder.disabledIsSet()) {
      initShim.setDisabled(builder.disabled);
    }
    if (builder.widgetClass != null) {
      initShim.setWidgetClass(builder.widgetClass);
    }
    if (builder.checkmarkIsSet()) {
      initShim.setCheckmark(builder.checkmark);
    }
    if (builder.checkedIsSet()) {
      initShim.setChecked(builder.checked);
    }
    if (builder.autocheckIsSet()) {
      initShim.setAutocheck(builder.autocheck);
    }
    this.id = initShim.getId();
    this.visible = initShim.isVisible();
    this.mold = initShim.getMold();
    this.zIndex = initShim.getZIndex();
    this.draggable = initShim.getDraggable();
    this.focus = initShim.isFocus();
    this.droppable = initShim.getDroppable();
    this.renderdefer = initShim.getRenderdefer();
    this.label = initShim.getLabel();
    this.iconSclass = initShim.getIconSclass();
    this.iconSclasses = initShim.getIconSclasses();
    this.iconTooltip = initShim.getIconTooltip();
    this.iconTooltips = initShim.getIconTooltips();
    this.image = initShim.getImage();
    this.hoverImage = initShim.getHoverImage();
    this.preloadImage = initShim.isPreloadImage();
    this.disabled = initShim.isDisabled();
    this.widgetClass = initShim.getWidgetClass();
    this.checkmark = initShim.isCheckmark();
    this.checked = initShim.isChecked();
    this.autocheck = initShim.isAutocheck();
    this.initShim = null;
  }

  private ImmutableIMenuitem(
      String id,
      @Nullable ActionHandler action,
      @Nullable ImmutableList<ActionHandler> actions,
      boolean visible,
      String mold,
      @Nullable EventListenerMap eventListenerMap,
      @Nullable ImmutableMap<String, String> widgetListeners,
      @Nullable ImmutableMap<String, String> widgetOverrides,
      @Nullable ImmutableMap<String, String> clientAttributes,
      @Nullable String left,
      @Nullable String top,
      int zIndex,
      @Nullable String height,
      @Nullable String width,
      @Nullable String tooltiptext,
      @Nullable String zclass,
      @Nullable String sclass,
      @Nullable String style,
      String draggable,
      boolean focus,
      String droppable,
      @Nullable String vflex,
      @Nullable String hflex,
      int renderdefer,
      @Nullable String clientAction,
      @Nullable Integer tabindex,
      @Nullable String ctrlKeys,
      @Nullable String context,
      @Nullable String popup,
      @Nullable String tooltip,
      String label,
      @Nullable String iconSclass,
      @Nullable String[] iconSclasses,
      @Nullable String iconTooltip,
      @Nullable String[] iconTooltips,
      @Nullable String image,
      @Nullable String hoverImage,
      boolean preloadImage,
      boolean disabled,
      String widgetClass,
      @Nullable String autodisable,
      @Nullable String href,
      @Nullable String target,
      @Nullable String upload,
      boolean checkmark,
      boolean checked,
      boolean autocheck) {
    this.id = id;
    this.action = action;
    this.actions = actions;
    this.visible = visible;
    this.mold = mold;
    this.eventListenerMap = eventListenerMap;
    this.widgetListeners = widgetListeners;
    this.widgetOverrides = widgetOverrides;
    this.clientAttributes = clientAttributes;
    this.left = left;
    this.top = top;
    this.zIndex = zIndex;
    this.height = height;
    this.width = width;
    this.tooltiptext = tooltiptext;
    this.zclass = zclass;
    this.sclass = sclass;
    this.style = style;
    this.draggable = draggable;
    this.focus = focus;
    this.droppable = droppable;
    this.vflex = vflex;
    this.hflex = hflex;
    this.renderdefer = renderdefer;
    this.clientAction = clientAction;
    this.tabindex = tabindex;
    this.ctrlKeys = ctrlKeys;
    this.context = context;
    this.popup = popup;
    this.tooltip = tooltip;
    this.label = label;
    this.iconSclass = iconSclass;
    this.iconSclasses = iconSclasses;
    this.iconTooltip = iconTooltip;
    this.iconTooltips = iconTooltips;
    this.image = image;
    this.hoverImage = hoverImage;
    this.preloadImage = preloadImage;
    this.disabled = disabled;
    this.widgetClass = widgetClass;
    this.autodisable = autodisable;
    this.href = href;
    this.target = target;
    this.upload = upload;
    this.checkmark = checkmark;
    this.checked = checked;
    this.autocheck = autocheck;
    this.initShim = null;
  }

  private static final byte STAGE_INITIALIZING = -1;
  private static final byte STAGE_UNINITIALIZED = 0;
  private static final byte STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  @Generated(from = "IMenuitem", generator = "Immutables")
  private final class InitShim {
    private byte idBuildStage = STAGE_UNINITIALIZED;
    private String id;

    String getId() {
      if (idBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (idBuildStage == STAGE_UNINITIALIZED) {
        idBuildStage = STAGE_INITIALIZING;
        this.id = Objects.requireNonNull(getIdInitialize(), "id");
        idBuildStage = STAGE_INITIALIZED;
      }
      return this.id;
    }

    void setId(String id) {
      this.id = id;
      idBuildStage = STAGE_INITIALIZED;
    }

    private byte visibleBuildStage = STAGE_UNINITIALIZED;
    private boolean visible;

    boolean isVisible() {
      if (visibleBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (visibleBuildStage == STAGE_UNINITIALIZED) {
        visibleBuildStage = STAGE_INITIALIZING;
        this.visible = isVisibleInitialize();
        visibleBuildStage = STAGE_INITIALIZED;
      }
      return this.visible;
    }

    void setVisible(boolean visible) {
      this.visible = visible;
      visibleBuildStage = STAGE_INITIALIZED;
    }

    private byte moldBuildStage = STAGE_UNINITIALIZED;
    private String mold;

    String getMold() {
      if (moldBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (moldBuildStage == STAGE_UNINITIALIZED) {
        moldBuildStage = STAGE_INITIALIZING;
        this.mold = Objects.requireNonNull(getMoldInitialize(), "mold");
        moldBuildStage = STAGE_INITIALIZED;
      }
      return this.mold;
    }

    void setMold(String mold) {
      this.mold = mold;
      moldBuildStage = STAGE_INITIALIZED;
    }

    private byte zIndexBuildStage = STAGE_UNINITIALIZED;
    private int zIndex;

    int getZIndex() {
      if (zIndexBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (zIndexBuildStage == STAGE_UNINITIALIZED) {
        zIndexBuildStage = STAGE_INITIALIZING;
        this.zIndex = getZIndexInitialize();
        zIndexBuildStage = STAGE_INITIALIZED;
      }
      return this.zIndex;
    }

    void setZIndex(int zIndex) {
      this.zIndex = zIndex;
      zIndexBuildStage = STAGE_INITIALIZED;
    }

    private byte draggableBuildStage = STAGE_UNINITIALIZED;
    private String draggable;

    String getDraggable() {
      if (draggableBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (draggableBuildStage == STAGE_UNINITIALIZED) {
        draggableBuildStage = STAGE_INITIALIZING;
        this.draggable = Objects.requireNonNull(getDraggableInitialize(), "draggable");
        draggableBuildStage = STAGE_INITIALIZED;
      }
      return this.draggable;
    }

    void setDraggable(String draggable) {
      this.draggable = draggable;
      draggableBuildStage = STAGE_INITIALIZED;
    }

    private byte focusBuildStage = STAGE_UNINITIALIZED;
    private boolean focus;

    boolean isFocus() {
      if (focusBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (focusBuildStage == STAGE_UNINITIALIZED) {
        focusBuildStage = STAGE_INITIALIZING;
        this.focus = isFocusInitialize();
        focusBuildStage = STAGE_INITIALIZED;
      }
      return this.focus;
    }

    void setFocus(boolean focus) {
      this.focus = focus;
      focusBuildStage = STAGE_INITIALIZED;
    }

    private byte droppableBuildStage = STAGE_UNINITIALIZED;
    private String droppable;

    String getDroppable() {
      if (droppableBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (droppableBuildStage == STAGE_UNINITIALIZED) {
        droppableBuildStage = STAGE_INITIALIZING;
        this.droppable = Objects.requireNonNull(getDroppableInitialize(), "droppable");
        droppableBuildStage = STAGE_INITIALIZED;
      }
      return this.droppable;
    }

    void setDroppable(String droppable) {
      this.droppable = droppable;
      droppableBuildStage = STAGE_INITIALIZED;
    }

    private byte renderdeferBuildStage = STAGE_UNINITIALIZED;
    private int renderdefer;

    int getRenderdefer() {
      if (renderdeferBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (renderdeferBuildStage == STAGE_UNINITIALIZED) {
        renderdeferBuildStage = STAGE_INITIALIZING;
        this.renderdefer = getRenderdeferInitialize();
        renderdeferBuildStage = STAGE_INITIALIZED;
      }
      return this.renderdefer;
    }

    void setRenderdefer(int renderdefer) {
      this.renderdefer = renderdefer;
      renderdeferBuildStage = STAGE_INITIALIZED;
    }

    private byte labelBuildStage = STAGE_UNINITIALIZED;
    private String label;

    String getLabel() {
      if (labelBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (labelBuildStage == STAGE_UNINITIALIZED) {
        labelBuildStage = STAGE_INITIALIZING;
        this.label = Objects.requireNonNull(getLabelInitialize(), "label");
        labelBuildStage = STAGE_INITIALIZED;
      }
      return this.label;
    }

    void setLabel(String label) {
      this.label = label;
      labelBuildStage = STAGE_INITIALIZED;
    }

    private byte iconSclassBuildStage = STAGE_UNINITIALIZED;
    private String iconSclass;

    String getIconSclass() {
      if (iconSclassBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconSclassBuildStage == STAGE_UNINITIALIZED) {
        iconSclassBuildStage = STAGE_INITIALIZING;
        this.iconSclass = getIconSclassInitialize();
        iconSclassBuildStage = STAGE_INITIALIZED;
      }
      return this.iconSclass;
    }

    void setIconSclass(String iconSclass) {
      this.iconSclass = iconSclass;
      iconSclassBuildStage = STAGE_INITIALIZED;
    }

    private byte iconSclassesBuildStage = STAGE_UNINITIALIZED;
    private String[] iconSclasses;

    String[] getIconSclasses() {
      if (iconSclassesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconSclassesBuildStage == STAGE_UNINITIALIZED) {
        iconSclassesBuildStage = STAGE_INITIALIZING;
        this.iconSclasses = getIconSclassesInitialize();
        iconSclassesBuildStage = STAGE_INITIALIZED;
      }
      return this.iconSclasses;
    }

    void setIconSclasses(String[] iconSclasses) {
      this.iconSclasses = iconSclasses;
      iconSclassesBuildStage = STAGE_INITIALIZED;
    }

    private byte iconTooltipBuildStage = STAGE_UNINITIALIZED;
    private String iconTooltip;

    String getIconTooltip() {
      if (iconTooltipBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconTooltipBuildStage == STAGE_UNINITIALIZED) {
        iconTooltipBuildStage = STAGE_INITIALIZING;
        this.iconTooltip = getIconTooltipInitialize();
        iconTooltipBuildStage = STAGE_INITIALIZED;
      }
      return this.iconTooltip;
    }

    void setIconTooltip(String iconTooltip) {
      this.iconTooltip = iconTooltip;
      iconTooltipBuildStage = STAGE_INITIALIZED;
    }

    private byte iconTooltipsBuildStage = STAGE_UNINITIALIZED;
    private String[] iconTooltips;

    String[] getIconTooltips() {
      if (iconTooltipsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconTooltipsBuildStage == STAGE_UNINITIALIZED) {
        iconTooltipsBuildStage = STAGE_INITIALIZING;
        this.iconTooltips = getIconTooltipsInitialize();
        iconTooltipsBuildStage = STAGE_INITIALIZED;
      }
      return this.iconTooltips;
    }

    void setIconTooltips(String[] iconTooltips) {
      this.iconTooltips = iconTooltips;
      iconTooltipsBuildStage = STAGE_INITIALIZED;
    }

    private byte imageBuildStage = STAGE_UNINITIALIZED;
    private String image;

    String getImage() {
      if (imageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (imageBuildStage == STAGE_UNINITIALIZED) {
        imageBuildStage = STAGE_INITIALIZING;
        this.image = getImageInitialize();
        imageBuildStage = STAGE_INITIALIZED;
      }
      return this.image;
    }

    void setImage(String image) {
      this.image = image;
      imageBuildStage = STAGE_INITIALIZED;
    }

    private byte hoverImageBuildStage = STAGE_UNINITIALIZED;
    private String hoverImage;

    String getHoverImage() {
      if (hoverImageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (hoverImageBuildStage == STAGE_UNINITIALIZED) {
        hoverImageBuildStage = STAGE_INITIALIZING;
        this.hoverImage = getHoverImageInitialize();
        hoverImageBuildStage = STAGE_INITIALIZED;
      }
      return this.hoverImage;
    }

    void setHoverImage(String hoverImage) {
      this.hoverImage = hoverImage;
      hoverImageBuildStage = STAGE_INITIALIZED;
    }

    private byte preloadImageBuildStage = STAGE_UNINITIALIZED;
    private boolean preloadImage;

    boolean isPreloadImage() {
      if (preloadImageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (preloadImageBuildStage == STAGE_UNINITIALIZED) {
        preloadImageBuildStage = STAGE_INITIALIZING;
        this.preloadImage = isPreloadImageInitialize();
        preloadImageBuildStage = STAGE_INITIALIZED;
      }
      return this.preloadImage;
    }

    void setPreloadImage(boolean preloadImage) {
      this.preloadImage = preloadImage;
      preloadImageBuildStage = STAGE_INITIALIZED;
    }

    private byte disabledBuildStage = STAGE_UNINITIALIZED;
    private boolean disabled;

    boolean isDisabled() {
      if (disabledBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (disabledBuildStage == STAGE_UNINITIALIZED) {
        disabledBuildStage = STAGE_INITIALIZING;
        this.disabled = isDisabledInitialize();
        disabledBuildStage = STAGE_INITIALIZED;
      }
      return this.disabled;
    }

    void setDisabled(boolean disabled) {
      this.disabled = disabled;
      disabledBuildStage = STAGE_INITIALIZED;
    }

    private byte widgetClassBuildStage = STAGE_UNINITIALIZED;
    private String widgetClass;

    String getWidgetClass() {
      if (widgetClassBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (widgetClassBuildStage == STAGE_UNINITIALIZED) {
        widgetClassBuildStage = STAGE_INITIALIZING;
        this.widgetClass = Objects.requireNonNull(getWidgetClassInitialize(), "widgetClass");
        widgetClassBuildStage = STAGE_INITIALIZED;
      }
      return this.widgetClass;
    }

    void setWidgetClass(String widgetClass) {
      this.widgetClass = widgetClass;
      widgetClassBuildStage = STAGE_INITIALIZED;
    }

    private byte checkmarkBuildStage = STAGE_UNINITIALIZED;
    private boolean checkmark;

    boolean isCheckmark() {
      if (checkmarkBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (checkmarkBuildStage == STAGE_UNINITIALIZED) {
        checkmarkBuildStage = STAGE_INITIALIZING;
        this.checkmark = isCheckmarkInitialize();
        checkmarkBuildStage = STAGE_INITIALIZED;
      }
      return this.checkmark;
    }

    void setCheckmark(boolean checkmark) {
      this.checkmark = checkmark;
      checkmarkBuildStage = STAGE_INITIALIZED;
    }

    private byte checkedBuildStage = STAGE_UNINITIALIZED;
    private boolean checked;

    boolean isChecked() {
      if (checkedBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (checkedBuildStage == STAGE_UNINITIALIZED) {
        checkedBuildStage = STAGE_INITIALIZING;
        this.checked = isCheckedInitialize();
        checkedBuildStage = STAGE_INITIALIZED;
      }
      return this.checked;
    }

    void setChecked(boolean checked) {
      this.checked = checked;
      checkedBuildStage = STAGE_INITIALIZED;
    }

    private byte autocheckBuildStage = STAGE_UNINITIALIZED;
    private boolean autocheck;

    boolean isAutocheck() {
      if (autocheckBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (autocheckBuildStage == STAGE_UNINITIALIZED) {
        autocheckBuildStage = STAGE_INITIALIZING;
        this.autocheck = isAutocheckInitialize();
        autocheckBuildStage = STAGE_INITIALIZED;
      }
      return this.autocheck;
    }

    void setAutocheck(boolean autocheck) {
      this.autocheck = autocheck;
      autocheckBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      List<String> attributes = new ArrayList<>();
      if (idBuildStage == STAGE_INITIALIZING) attributes.add("id");
      if (visibleBuildStage == STAGE_INITIALIZING) attributes.add("visible");
      if (moldBuildStage == STAGE_INITIALIZING) attributes.add("mold");
      if (zIndexBuildStage == STAGE_INITIALIZING) attributes.add("zIndex");
      if (draggableBuildStage == STAGE_INITIALIZING) attributes.add("draggable");
      if (focusBuildStage == STAGE_INITIALIZING) attributes.add("focus");
      if (droppableBuildStage == STAGE_INITIALIZING) attributes.add("droppable");
      if (renderdeferBuildStage == STAGE_INITIALIZING) attributes.add("renderdefer");
      if (labelBuildStage == STAGE_INITIALIZING) attributes.add("label");
      if (iconSclassBuildStage == STAGE_INITIALIZING) attributes.add("iconSclass");
      if (iconSclassesBuildStage == STAGE_INITIALIZING) attributes.add("iconSclasses");
      if (iconTooltipBuildStage == STAGE_INITIALIZING) attributes.add("iconTooltip");
      if (iconTooltipsBuildStage == STAGE_INITIALIZING) attributes.add("iconTooltips");
      if (imageBuildStage == STAGE_INITIALIZING) attributes.add("image");
      if (hoverImageBuildStage == STAGE_INITIALIZING) attributes.add("hoverImage");
      if (preloadImageBuildStage == STAGE_INITIALIZING) attributes.add("preloadImage");
      if (disabledBuildStage == STAGE_INITIALIZING) attributes.add("disabled");
      if (widgetClassBuildStage == STAGE_INITIALIZING) attributes.add("widgetClass");
      if (checkmarkBuildStage == STAGE_INITIALIZING) attributes.add("checkmark");
      if (checkedBuildStage == STAGE_INITIALIZING) attributes.add("checked");
      if (autocheckBuildStage == STAGE_INITIALIZING) attributes.add("autocheck");
      return "Cannot build IMenuitem, attribute initializers form cycle " + attributes;
    }
  }

  private String getIdInitialize() {
    return IMenuitem.super.getId();
  }

  private boolean isVisibleInitialize() {
    return IMenuitem.super.isVisible();
  }

  private String getMoldInitialize() {
    return IMenuitem.super.getMold();
  }

  private int getZIndexInitialize() {
    return IMenuitem.super.getZIndex();
  }

  private String getDraggableInitialize() {
    return IMenuitem.super.getDraggable();
  }

  private boolean isFocusInitialize() {
    return IMenuitem.super.isFocus();
  }

  private String getDroppableInitialize() {
    return IMenuitem.super.getDroppable();
  }

  private int getRenderdeferInitialize() {
    return IMenuitem.super.getRenderdefer();
  }

  private String getLabelInitialize() {
    return IMenuitem.super.getLabel();
  }

  private @Nullable String getIconSclassInitialize() {
    return IMenuitem.super.getIconSclass();
  }

  private @Nullable String[] getIconSclassesInitialize() {
    return IMenuitem.super.getIconSclasses();
  }

  private @Nullable String getIconTooltipInitialize() {
    return IMenuitem.super.getIconTooltip();
  }

  private @Nullable String[] getIconTooltipsInitialize() {
    return IMenuitem.super.getIconTooltips();
  }

  private @Nullable String getImageInitialize() {
    return IMenuitem.super.getImage();
  }

  private @Nullable String getHoverImageInitialize() {
    return IMenuitem.super.getHoverImage();
  }

  private boolean isPreloadImageInitialize() {
    return IMenuitem.super.isPreloadImage();
  }

  private boolean isDisabledInitialize() {
    return IMenuitem.super.isDisabled();
  }

  private String getWidgetClassInitialize() {
    return IMenuitem.super.getWidgetClass();
  }

  private boolean isCheckmarkInitialize() {
    return IMenuitem.super.isCheckmark();
  }

  private boolean isCheckedInitialize() {
    return IMenuitem.super.isChecked();
  }

  private boolean isAutocheckInitialize() {
    return IMenuitem.super.isAutocheck();
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @Override
  public String getId() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getId()
        : this.id;
  }

  /**
   * @return The value of the {@code action} attribute
   */
  @StatelessOnly
  @Override
  public @Nullable ActionHandler getAction() {
    return action;
  }

  /**
   * @return The value of the {@code actions} attribute
   */
  @StatelessOnly
  @Override
  public @Nullable ImmutableList<ActionHandler> getActions() {
    return actions;
  }

  /**
   * @return The value of the {@code visible} attribute
   */
  @Override
  public boolean isVisible() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isVisible()
        : this.visible;
  }

  /**
   * @return The value of the {@code mold} attribute
   */
  @Override
  public String getMold() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getMold()
        : this.mold;
  }

  /**
   * @return The value of the {@code eventListenerMap} attribute
   */
  @Override
  public @Nullable EventListenerMap getEventListenerMap() {
    return eventListenerMap;
  }

  /**
   * @return The value of the {@code widgetListeners} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getWidgetListeners() {
    return widgetListeners;
  }

  /**
   * @return The value of the {@code widgetOverrides} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getWidgetOverrides() {
    return widgetOverrides;
  }

  /**
   * @return The value of the {@code clientAttributes} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getClientAttributes() {
    return clientAttributes;
  }

  /**
   * @return The value of the {@code left} attribute
   */
  @Override
  public @Nullable String getLeft() {
    return left;
  }

  /**
   * @return The value of the {@code top} attribute
   */
  @Override
  public @Nullable String getTop() {
    return top;
  }

  /**
   * @return The value of the {@code zIndex} attribute
   */
  @Override
  public int getZIndex() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getZIndex()
        : this.zIndex;
  }

  /**
   * @return The value of the {@code height} attribute
   */
  @Override
  public @Nullable String getHeight() {
    return height;
  }

  /**
   * @return The value of the {@code width} attribute
   */
  @Override
  public @Nullable String getWidth() {
    return width;
  }

  /**
   * @return The value of the {@code tooltiptext} attribute
   */
  @Override
  public @Nullable String getTooltiptext() {
    return tooltiptext;
  }

  /**
   * @return The value of the {@code zclass} attribute
   */
  @Override
  public @Nullable String getZclass() {
    return zclass;
  }

  /**
   * @return The value of the {@code sclass} attribute
   */
  @Override
  public @Nullable String getSclass() {
    return sclass;
  }

  /**
   * @return The value of the {@code style} attribute
   */
  @Override
  public @Nullable String getStyle() {
    return style;
  }

  /**
   * @return The value of the {@code draggable} attribute
   */
  @Override
  public String getDraggable() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDraggable()
        : this.draggable;
  }

  /**
   * @return The value of the {@code focus} attribute
   */
  @Override
  public boolean isFocus() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isFocus()
        : this.focus;
  }

  /**
   * @return The value of the {@code droppable} attribute
   */
  @Override
  public String getDroppable() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDroppable()
        : this.droppable;
  }

  /**
   * @return The value of the {@code vflex} attribute
   */
  @Override
  public @Nullable String getVflex() {
    return vflex;
  }

  /**
   * @return The value of the {@code hflex} attribute
   */
  @Override
  public @Nullable String getHflex() {
    return hflex;
  }

  /**
   * @return The value of the {@code renderdefer} attribute
   */
  @Override
  public int getRenderdefer() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getRenderdefer()
        : this.renderdefer;
  }

  /**
   * @return The value of the {@code clientAction} attribute
   */
  @Override
  public @Nullable String getClientAction() {
    return clientAction;
  }

  /**
   * @return The value of the {@code tabindex} attribute
   */
  @Override
  public @Nullable Integer getTabindex() {
    return tabindex;
  }

  /**
   * @return The value of the {@code ctrlKeys} attribute
   */
  @Override
  public @Nullable String getCtrlKeys() {
    return ctrlKeys;
  }

  /**
   * @return The value of the {@code context} attribute
   */
  @Override
  public @Nullable String getContext() {
    return context;
  }

  /**
   * @return The value of the {@code popup} attribute
   */
  @Override
  public @Nullable String getPopup() {
    return popup;
  }

  /**
   * @return The value of the {@code tooltip} attribute
   */
  @Override
  public @Nullable String getTooltip() {
    return tooltip;
  }

  /**
   * @return The value of the {@code label} attribute
   */
  @Override
  public String getLabel() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getLabel()
        : this.label;
  }

  /**
   * @return The value of the {@code iconSclass} attribute
   */
  @Override
  public @Nullable String getIconSclass() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconSclass()
        : this.iconSclass;
  }

  /**
   * @return A cloned {@code iconSclasses} array
   */
  @Override
  public @Nullable String[] getIconSclasses() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconSclasses()
        : this.iconSclasses;
  }

  /**
   * @return The value of the {@code iconTooltip} attribute
   */
  @Override
  public @Nullable String getIconTooltip() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconTooltip()
        : this.iconTooltip;
  }

  /**
   * @return A cloned {@code iconTooltips} array
   */
  @Override
  public @Nullable String[] getIconTooltips() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconTooltips()
        : this.iconTooltips;
  }

  /**
   * @return The value of the {@code image} attribute
   */
  @Override
  public @Nullable String getImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getImage()
        : this.image;
  }

  /**
   * @return The value of the {@code hoverImage} attribute
   */
  @Override
  public @Nullable String getHoverImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getHoverImage()
        : this.hoverImage;
  }

  /**
   * @return The value of the {@code preloadImage} attribute
   */
  @StatelessOnly
  @Override
  public boolean isPreloadImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isPreloadImage()
        : this.preloadImage;
  }

  /**
   * @return The value of the {@code disabled} attribute
   */
  @Override
  public boolean isDisabled() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isDisabled()
        : this.disabled;
  }

  /**
   * @return The value of the {@code widgetClass} attribute
   */
  @Override
  public String getWidgetClass() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getWidgetClass()
        : this.widgetClass;
  }

  /**
   * @return The value of the {@code autodisable} attribute
   */
  @Override
  public @Nullable String getAutodisable() {
    return autodisable;
  }

  /**
   * @return The value of the {@code href} attribute
   */
  @Override
  public @Nullable String getHref() {
    return href;
  }

  /**
   * @return The value of the {@code target} attribute
   */
  @Override
  public @Nullable String getTarget() {
    return target;
  }

  /**
   * @return The value of the {@code upload} attribute
   */
  @Override
  public @Nullable String getUpload() {
    return upload;
  }

  /**
   * @return The value of the {@code checkmark} attribute
   */
  @Override
  public boolean isCheckmark() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isCheckmark()
        : this.checkmark;
  }

  /**
   * @return The value of the {@code checked} attribute
   */
  @Override
  public boolean isChecked() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isChecked()
        : this.checked;
  }

  /**
   * @return The value of the {@code autocheck} attribute
   */
  @Override
  public boolean isAutocheck() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isAutocheck()
        : this.autocheck;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getId() id} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for id
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (this.id.equals(newValue)) return this;
    return validate(new ImmutableIMenuitem(
        newValue,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getAction() action} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for action (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withAction(@Nullable ActionHandler value) {
    if (this.action == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        value,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IMenuitem#getActions() actions}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIMenuitem withActions(@Nullable ActionHandler... elements) {
    if (elements == null) {
      return validate(new ImmutableIMenuitem(
          this.id,
          this.action,
          null,
          this.visible,
          this.mold,
          this.eventListenerMap,
          this.widgetListeners,
          this.widgetOverrides,
          this.clientAttributes,
          this.left,
          this.top,
          this.zIndex,
          this.height,
          this.width,
          this.tooltiptext,
          this.zclass,
          this.sclass,
          this.style,
          this.draggable,
          this.focus,
          this.droppable,
          this.vflex,
          this.hflex,
          this.renderdefer,
          this.clientAction,
          this.tabindex,
          this.ctrlKeys,
          this.context,
          this.popup,
          this.tooltip,
          this.label,
          this.iconSclass,
          this.iconSclasses,
          this.iconTooltip,
          this.iconTooltips,
          this.image,
          this.hoverImage,
          this.preloadImage,
          this.disabled,
          this.widgetClass,
          this.autodisable,
          this.href,
          this.target,
          this.upload,
          this.checkmark,
          this.checked,
          this.autocheck));
    }
    @Nullable ImmutableList<ActionHandler> newValue = elements == null ? null : ImmutableList.copyOf(elements);
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        newValue,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IMenuitem#getActions() actions}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of actions elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIMenuitem withActions(@Nullable Iterable<? extends ActionHandler> elements) {
    if (this.actions == elements) return this;
    @Nullable ImmutableList<ActionHandler> newValue = elements == null ? null : ImmutableList.copyOf(elements);
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        newValue,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#isVisible() visible} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for visible
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withVisible(boolean value) {
    if (this.visible == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        value,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getMold() mold} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for mold
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withMold(String value) {
    String newValue = Objects.requireNonNull(value, "mold");
    if (this.mold.equals(newValue)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        newValue,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getEventListenerMap() eventListenerMap} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for eventListenerMap (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withEventListenerMap(@Nullable EventListenerMap value) {
    if (this.eventListenerMap == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        value,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by replacing the {@link IMenuitem#getWidgetListeners() widgetListeners} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the widgetListeners map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIMenuitem withWidgetListeners(@Nullable Map<String, ? extends String> entries) {
    if (this.widgetListeners == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        newValue,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by replacing the {@link IMenuitem#getWidgetOverrides() widgetOverrides} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the widgetOverrides map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIMenuitem withWidgetOverrides(@Nullable Map<String, ? extends String> entries) {
    if (this.widgetOverrides == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        newValue,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by replacing the {@link IMenuitem#getClientAttributes() clientAttributes} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the clientAttributes map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIMenuitem withClientAttributes(@Nullable Map<String, ? extends String> entries) {
    if (this.clientAttributes == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        newValue,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getLeft() left} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for left (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withLeft(@Nullable String value) {
    if (Objects.equals(this.left, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        value,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getTop() top} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for top (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withTop(@Nullable String value) {
    if (Objects.equals(this.top, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        value,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getZIndex() zIndex} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for zIndex
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withZIndex(int value) {
    if (this.zIndex == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        value,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getHeight() height} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for height (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withHeight(@Nullable String value) {
    if (Objects.equals(this.height, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        value,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getWidth() width} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for width (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withWidth(@Nullable String value) {
    if (Objects.equals(this.width, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        value,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getTooltiptext() tooltiptext} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tooltiptext (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withTooltiptext(@Nullable String value) {
    if (Objects.equals(this.tooltiptext, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        value,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getZclass() zclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for zclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withZclass(@Nullable String value) {
    if (Objects.equals(this.zclass, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        value,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getSclass() sclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withSclass(@Nullable String value) {
    if (Objects.equals(this.sclass, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        value,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getStyle() style} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for style (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withStyle(@Nullable String value) {
    if (Objects.equals(this.style, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        value,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getDraggable() draggable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for draggable
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withDraggable(String value) {
    String newValue = Objects.requireNonNull(value, "draggable");
    if (this.draggable.equals(newValue)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        newValue,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#isFocus() focus} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for focus
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withFocus(boolean value) {
    if (this.focus == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        value,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getDroppable() droppable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for droppable
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withDroppable(String value) {
    String newValue = Objects.requireNonNull(value, "droppable");
    if (this.droppable.equals(newValue)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        newValue,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getVflex() vflex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for vflex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withVflex(@Nullable String value) {
    if (Objects.equals(this.vflex, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        value,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getHflex() hflex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for hflex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withHflex(@Nullable String value) {
    if (Objects.equals(this.hflex, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        value,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getRenderdefer() renderdefer} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for renderdefer
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withRenderdefer(int value) {
    if (this.renderdefer == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        value,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getClientAction() clientAction} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for clientAction (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withClientAction(@Nullable String value) {
    if (Objects.equals(this.clientAction, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        value,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getTabindex() tabindex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tabindex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withTabindex(@Nullable Integer value) {
    if (Objects.equals(this.tabindex, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        value,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getCtrlKeys() ctrlKeys} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for ctrlKeys (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withCtrlKeys(@Nullable String value) {
    if (Objects.equals(this.ctrlKeys, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        value,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getContext() context} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for context (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withContext(@Nullable String value) {
    if (Objects.equals(this.context, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        value,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getPopup() popup} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for popup (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withPopup(@Nullable String value) {
    if (Objects.equals(this.popup, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        value,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getTooltip() tooltip} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tooltip (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withTooltip(@Nullable String value) {
    if (Objects.equals(this.tooltip, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        value,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getLabel() label} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for label
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withLabel(String value) {
    String newValue = Objects.requireNonNull(value, "label");
    if (this.label.equals(newValue)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        newValue,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getIconSclass() iconSclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for iconSclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withIconSclass(@Nullable String value) {
    if (Objects.equals(this.iconSclass, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        value,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IMenuitem#getIconSclasses() iconSclasses}.
   * The array is cloned before being saved as attribute values.
   * @param elements The non-null elements for iconSclasses
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIMenuitem withIconSclasses(@Nullable String... elements) {
    @Nullable String[] newValue = elements == null ? null : elements.clone();
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        newValue,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getIconTooltip() iconTooltip} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for iconTooltip (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withIconTooltip(@Nullable String value) {
    if (Objects.equals(this.iconTooltip, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        value,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IMenuitem#getIconTooltips() iconTooltips}.
   * The array is cloned before being saved as attribute values.
   * @param elements The non-null elements for iconTooltips
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIMenuitem withIconTooltips(@Nullable String... elements) {
    @Nullable String[] newValue = elements == null ? null : elements.clone();
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        newValue,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getImage() image} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for image (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withImage(@Nullable String value) {
    if (Objects.equals(this.image, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        value,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getHoverImage() hoverImage} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for hoverImage (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withHoverImage(@Nullable String value) {
    if (Objects.equals(this.hoverImage, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        value,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#isPreloadImage() preloadImage} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for preloadImage
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withPreloadImage(boolean value) {
    if (this.preloadImage == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        value,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#isDisabled() disabled} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for disabled
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withDisabled(boolean value) {
    if (this.disabled == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        value,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getWidgetClass() widgetClass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for widgetClass
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withWidgetClass(String value) {
    String newValue = Objects.requireNonNull(value, "widgetClass");
    if (this.widgetClass.equals(newValue)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        newValue,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getAutodisable() autodisable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for autodisable (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withAutodisable(@Nullable String value) {
    if (Objects.equals(this.autodisable, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        value,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getHref() href} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for href (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withHref(@Nullable String value) {
    if (Objects.equals(this.href, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        value,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getTarget() target} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for target (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withTarget(@Nullable String value) {
    if (Objects.equals(this.target, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        value,
        this.upload,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#getUpload() upload} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for upload (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withUpload(@Nullable String value) {
    if (Objects.equals(this.upload, value)) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        value,
        this.checkmark,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#isCheckmark() checkmark} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for checkmark
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withCheckmark(boolean value) {
    if (this.checkmark == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        value,
        this.checked,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#isChecked() checked} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for checked
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withChecked(boolean value) {
    if (this.checked == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        value,
        this.autocheck));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IMenuitem#isAutocheck() autocheck} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for autocheck
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIMenuitem withAutocheck(boolean value) {
    if (this.autocheck == value) return this;
    return validate(new ImmutableIMenuitem(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.disabled,
        this.widgetClass,
        this.autodisable,
        this.href,
        this.target,
        this.upload,
        this.checkmark,
        this.checked,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIMenuitem} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIMenuitem
        && equalTo((ImmutableIMenuitem) another);
  }

  private boolean equalTo(ImmutableIMenuitem another) {
    return id.equals(another.id)
        && Objects.equals(action, another.action)
        && Objects.equals(actions, another.actions)
        && visible == another.visible
        && mold.equals(another.mold)
        && Objects.equals(eventListenerMap, another.eventListenerMap)
        && Objects.equals(widgetListeners, another.widgetListeners)
        && Objects.equals(widgetOverrides, another.widgetOverrides)
        && Objects.equals(clientAttributes, another.clientAttributes)
        && Objects.equals(left, another.left)
        && Objects.equals(top, another.top)
        && zIndex == another.zIndex
        && Objects.equals(height, another.height)
        && Objects.equals(width, another.width)
        && Objects.equals(tooltiptext, another.tooltiptext)
        && Objects.equals(zclass, another.zclass)
        && Objects.equals(sclass, another.sclass)
        && Objects.equals(style, another.style)
        && draggable.equals(another.draggable)
        && focus == another.focus
        && droppable.equals(another.droppable)
        && Objects.equals(vflex, another.vflex)
        && Objects.equals(hflex, another.hflex)
        && renderdefer == another.renderdefer
        && Objects.equals(clientAction, another.clientAction)
        && Objects.equals(tabindex, another.tabindex)
        && Objects.equals(ctrlKeys, another.ctrlKeys)
        && Objects.equals(context, another.context)
        && Objects.equals(popup, another.popup)
        && Objects.equals(tooltip, another.tooltip)
        && label.equals(another.label)
        && Objects.equals(iconSclass, another.iconSclass)
        && Arrays.equals(iconSclasses, another.iconSclasses)
        && Objects.equals(iconTooltip, another.iconTooltip)
        && Arrays.equals(iconTooltips, another.iconTooltips)
        && Objects.equals(image, another.image)
        && Objects.equals(hoverImage, another.hoverImage)
        && preloadImage == another.preloadImage
        && disabled == another.disabled
        && widgetClass.equals(another.widgetClass)
        && Objects.equals(autodisable, another.autodisable)
        && Objects.equals(href, another.href)
        && Objects.equals(target, another.target)
        && Objects.equals(upload, another.upload)
        && checkmark == another.checkmark
        && checked == another.checked
        && autocheck == another.autocheck;
  }

  /**
   * Computes a hash code from attributes: {@code id}, {@code action}, {@code actions}, {@code visible}, {@code mold}, {@code eventListenerMap}, {@code widgetListeners}, {@code widgetOverrides}, {@code clientAttributes}, {@code left}, {@code top}, {@code zIndex}, {@code height}, {@code width}, {@code tooltiptext}, {@code zclass}, {@code sclass}, {@code style}, {@code draggable}, {@code focus}, {@code droppable}, {@code vflex}, {@code hflex}, {@code renderdefer}, {@code clientAction}, {@code tabindex}, {@code ctrlKeys}, {@code context}, {@code popup}, {@code tooltip}, {@code label}, {@code iconSclass}, {@code iconSclasses}, {@code iconTooltip}, {@code iconTooltips}, {@code image}, {@code hoverImage}, {@code preloadImage}, {@code disabled}, {@code widgetClass}, {@code autodisable}, {@code href}, {@code target}, {@code upload}, {@code checkmark}, {@code checked}, {@code autocheck}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + id.hashCode();
    h += (h << 5) + Objects.hashCode(action);
    h += (h << 5) + Objects.hashCode(actions);
    h += (h << 5) + Booleans.hashCode(visible);
    h += (h << 5) + mold.hashCode();
    h += (h << 5) + Objects.hashCode(eventListenerMap);
    h += (h << 5) + Objects.hashCode(widgetListeners);
    h += (h << 5) + Objects.hashCode(widgetOverrides);
    h += (h << 5) + Objects.hashCode(clientAttributes);
    h += (h << 5) + Objects.hashCode(left);
    h += (h << 5) + Objects.hashCode(top);
    h += (h << 5) + zIndex;
    h += (h << 5) + Objects.hashCode(height);
    h += (h << 5) + Objects.hashCode(width);
    h += (h << 5) + Objects.hashCode(tooltiptext);
    h += (h << 5) + Objects.hashCode(zclass);
    h += (h << 5) + Objects.hashCode(sclass);
    h += (h << 5) + Objects.hashCode(style);
    h += (h << 5) + draggable.hashCode();
    h += (h << 5) + Booleans.hashCode(focus);
    h += (h << 5) + droppable.hashCode();
    h += (h << 5) + Objects.hashCode(vflex);
    h += (h << 5) + Objects.hashCode(hflex);
    h += (h << 5) + renderdefer;
    h += (h << 5) + Objects.hashCode(clientAction);
    h += (h << 5) + Objects.hashCode(tabindex);
    h += (h << 5) + Objects.hashCode(ctrlKeys);
    h += (h << 5) + Objects.hashCode(context);
    h += (h << 5) + Objects.hashCode(popup);
    h += (h << 5) + Objects.hashCode(tooltip);
    h += (h << 5) + label.hashCode();
    h += (h << 5) + Objects.hashCode(iconSclass);
    h += (h << 5) + Arrays.hashCode(iconSclasses);
    h += (h << 5) + Objects.hashCode(iconTooltip);
    h += (h << 5) + Arrays.hashCode(iconTooltips);
    h += (h << 5) + Objects.hashCode(image);
    h += (h << 5) + Objects.hashCode(hoverImage);
    h += (h << 5) + Booleans.hashCode(preloadImage);
    h += (h << 5) + Booleans.hashCode(disabled);
    h += (h << 5) + widgetClass.hashCode();
    h += (h << 5) + Objects.hashCode(autodisable);
    h += (h << 5) + Objects.hashCode(href);
    h += (h << 5) + Objects.hashCode(target);
    h += (h << 5) + Objects.hashCode(upload);
    h += (h << 5) + Booleans.hashCode(checkmark);
    h += (h << 5) + Booleans.hashCode(checked);
    h += (h << 5) + Booleans.hashCode(autocheck);
    return h;
  }

  /**
   * Prints the immutable value {@code IMenuitem} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("IMenuitem")
        .omitNullValues()
        .add("id", id)
        .add("action", action)
        .add("actions", actions)
        .add("visible", visible)
        .add("mold", mold)
        .add("eventListenerMap", eventListenerMap)
        .add("widgetListeners", widgetListeners)
        .add("widgetOverrides", widgetOverrides)
        .add("clientAttributes", clientAttributes)
        .add("left", left)
        .add("top", top)
        .add("zIndex", zIndex)
        .add("height", height)
        .add("width", width)
        .add("tooltiptext", tooltiptext)
        .add("zclass", zclass)
        .add("sclass", sclass)
        .add("style", style)
        .add("draggable", draggable)
        .add("focus", focus)
        .add("droppable", droppable)
        .add("vflex", vflex)
        .add("hflex", hflex)
        .add("renderdefer", renderdefer)
        .add("clientAction", clientAction)
        .add("tabindex", tabindex)
        .add("ctrlKeys", ctrlKeys)
        .add("context", context)
        .add("popup", popup)
        .add("tooltip", tooltip)
        .add("label", label)
        .add("iconSclass", iconSclass)
        .add("iconSclasses", Arrays.toString(iconSclasses))
        .add("iconTooltip", iconTooltip)
        .add("iconTooltips", Arrays.toString(iconTooltips))
        .add("image", image)
        .add("hoverImage", hoverImage)
        .add("preloadImage", preloadImage)
        .add("disabled", disabled)
        .add("widgetClass", widgetClass)
        .add("autodisable", autodisable)
        .add("href", href)
        .add("target", target)
        .add("upload", upload)
        .add("checkmark", checkmark)
        .add("checked", checked)
        .add("autocheck", autocheck)
        .toString();
  }

  private transient volatile long lazyInitBitmap;

  private static final long Z_K_TYPE_LAZY_INIT_BIT = 0x1L;

  private transient Class<Menuitem> zKType;

  /**
   * {@inheritDoc}
   * <p>
   * Returns a lazily initialized value of the {@link IMenuitem#getZKType() zKType} attribute.
   * Initialized once and only once and stored for subsequent access with proper synchronization.
   * In case of any exception or error thrown by the lazy value initializer,
   * the result will not be memoised (i.e. remembered) and on next call computation
   * will be attempted again.
   * @return A lazily initialized value of the {@code zKType} attribute
   */
  @Override
  public Class<Menuitem> getZKType() {
    if ((lazyInitBitmap & Z_K_TYPE_LAZY_INIT_BIT) == 0) {
      synchronized (this) {
        if ((lazyInitBitmap & Z_K_TYPE_LAZY_INIT_BIT) == 0) {
          this.zKType = Objects.requireNonNull(IMenuitem.super.getZKType(), "zKType");
          lazyInitBitmap |= Z_K_TYPE_LAZY_INIT_BIT;
        }
      }
    }
    return zKType;
  }

  private static ImmutableIMenuitem validate(ImmutableIMenuitem instance) {
    instance.checkHflexAndWidth();
    instance.checkVflexAndHeight();
    instance = (ImmutableIMenuitem) instance.checkDroppable();
    instance = (ImmutableIMenuitem) instance.checkDraggable();
    instance.checkActions();
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link IMenuitem} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable IMenuitem instance
   */
  public static IMenuitem copyOf(IMenuitem instance) {
    if (instance instanceof ImmutableIMenuitem) {
      return (ImmutableIMenuitem) instance;
    }
    return new IMenuitem.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link IMenuitem IMenuitem}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "IMenuitem", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private static final long OPT_BIT_VISIBLE = 0x1L;
    private static final long OPT_BIT_Z_INDEX = 0x2L;
    private static final long OPT_BIT_FOCUS = 0x4L;
    private static final long OPT_BIT_RENDERDEFER = 0x8L;
    private static final long OPT_BIT_ICON_SCLASS = 0x10L;
    private static final long OPT_BIT_ICON_SCLASSES = 0x20L;
    private static final long OPT_BIT_ICON_TOOLTIP = 0x40L;
    private static final long OPT_BIT_ICON_TOOLTIPS = 0x80L;
    private static final long OPT_BIT_IMAGE = 0x100L;
    private static final long OPT_BIT_HOVER_IMAGE = 0x200L;
    private static final long OPT_BIT_PRELOAD_IMAGE = 0x400L;
    private static final long OPT_BIT_DISABLED = 0x800L;
    private static final long OPT_BIT_CHECKMARK = 0x1000L;
    private static final long OPT_BIT_CHECKED = 0x2000L;
    private static final long OPT_BIT_AUTOCHECK = 0x4000L;
    private long optBits;

    private @Nullable String id;
    private @Nullable ActionHandler action;
    private ImmutableList.Builder<ActionHandler> actions = null;
    private boolean visible;
    private @Nullable String mold;
    private @Nullable EventListenerMap eventListenerMap;
    private ImmutableMap.Builder<String, String> widgetListeners = null;
    private ImmutableMap.Builder<String, String> widgetOverrides = null;
    private ImmutableMap.Builder<String, String> clientAttributes = null;
    private @Nullable String left;
    private @Nullable String top;
    private int zIndex;
    private @Nullable String height;
    private @Nullable String width;
    private @Nullable String tooltiptext;
    private @Nullable String zclass;
    private @Nullable String sclass;
    private @Nullable String style;
    private @Nullable String draggable;
    private boolean focus;
    private @Nullable String droppable;
    private @Nullable String vflex;
    private @Nullable String hflex;
    private int renderdefer;
    private @Nullable String clientAction;
    private @Nullable Integer tabindex;
    private @Nullable String ctrlKeys;
    private @Nullable String context;
    private @Nullable String popup;
    private @Nullable String tooltip;
    private @Nullable String label;
    private @Nullable String iconSclass;
    private @Nullable String[] iconSclasses;
    private @Nullable String iconTooltip;
    private @Nullable String[] iconTooltips;
    private @Nullable String image;
    private @Nullable String hoverImage;
    private boolean preloadImage;
    private boolean disabled;
    private @Nullable String widgetClass;
    private @Nullable String autodisable;
    private @Nullable String href;
    private @Nullable String target;
    private @Nullable String upload;
    private boolean checkmark;
    private boolean checked;
    private boolean autocheck;

    /**
     * Creates a builder for {@link IMenuitem IMenuitem} instances.
     * <pre>
     * new IMenuitem.Builder()
     *    .setId(String) // optional {@link IMenuitem#getId() id}
     *    .setAction(org.zkoss.stateless.util.ActionHandler | null) // nullable {@link IMenuitem#getAction() action}
     *    .setActions(List&amp;lt;org.zkoss.stateless.util.ActionHandler&amp;gt; | null) // nullable {@link IMenuitem#getActions() actions}
     *    .setVisible(boolean) // optional {@link IMenuitem#isVisible() visible}
     *    .setMold(String) // optional {@link IMenuitem#getMold() mold}
     *    .setEventListenerMap(org.zkoss.zk.ui.sys.EventListenerMap | null) // nullable {@link IMenuitem#getEventListenerMap() eventListenerMap}
     *    .setWidgetListeners(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link IMenuitem#getWidgetListeners() widgetListeners}
     *    .setWidgetOverrides(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link IMenuitem#getWidgetOverrides() widgetOverrides}
     *    .setClientAttributes(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link IMenuitem#getClientAttributes() clientAttributes}
     *    .setLeft(String | null) // nullable {@link IMenuitem#getLeft() left}
     *    .setTop(String | null) // nullable {@link IMenuitem#getTop() top}
     *    .setZIndex(int) // optional {@link IMenuitem#getZIndex() zIndex}
     *    .setHeight(String | null) // nullable {@link IMenuitem#getHeight() height}
     *    .setWidth(String | null) // nullable {@link IMenuitem#getWidth() width}
     *    .setTooltiptext(String | null) // nullable {@link IMenuitem#getTooltiptext() tooltiptext}
     *    .setZclass(String | null) // nullable {@link IMenuitem#getZclass() zclass}
     *    .setSclass(String | null) // nullable {@link IMenuitem#getSclass() sclass}
     *    .setStyle(String | null) // nullable {@link IMenuitem#getStyle() style}
     *    .setDraggable(String) // optional {@link IMenuitem#getDraggable() draggable}
     *    .setFocus(boolean) // optional {@link IMenuitem#isFocus() focus}
     *    .setDroppable(String) // optional {@link IMenuitem#getDroppable() droppable}
     *    .setVflex(String | null) // nullable {@link IMenuitem#getVflex() vflex}
     *    .setHflex(String | null) // nullable {@link IMenuitem#getHflex() hflex}
     *    .setRenderdefer(int) // optional {@link IMenuitem#getRenderdefer() renderdefer}
     *    .setClientAction(String | null) // nullable {@link IMenuitem#getClientAction() clientAction}
     *    .setTabindex(Integer | null) // nullable {@link IMenuitem#getTabindex() tabindex}
     *    .setCtrlKeys(String | null) // nullable {@link IMenuitem#getCtrlKeys() ctrlKeys}
     *    .setContext(String | null) // nullable {@link IMenuitem#getContext() context}
     *    .setPopup(String | null) // nullable {@link IMenuitem#getPopup() popup}
     *    .setTooltip(String | null) // nullable {@link IMenuitem#getTooltip() tooltip}
     *    .setLabel(String) // optional {@link IMenuitem#getLabel() label}
     *    .setIconSclass(String | null) // nullable {@link IMenuitem#getIconSclass() iconSclass}
     *    .setIconSclasses(String[] | null) // nullable {@link IMenuitem#getIconSclasses() iconSclasses}
     *    .setIconTooltip(String | null) // nullable {@link IMenuitem#getIconTooltip() iconTooltip}
     *    .setIconTooltips(String[] | null) // nullable {@link IMenuitem#getIconTooltips() iconTooltips}
     *    .setImage(String | null) // nullable {@link IMenuitem#getImage() image}
     *    .setHoverImage(String | null) // nullable {@link IMenuitem#getHoverImage() hoverImage}
     *    .setPreloadImage(boolean) // optional {@link IMenuitem#isPreloadImage() preloadImage}
     *    .setDisabled(boolean) // optional {@link IMenuitem#isDisabled() disabled}
     *    .setWidgetClass(String) // optional {@link IMenuitem#getWidgetClass() widgetClass}
     *    .setAutodisable(String | null) // nullable {@link IMenuitem#getAutodisable() autodisable}
     *    .setHref(String | null) // nullable {@link IMenuitem#getHref() href}
     *    .setTarget(String | null) // nullable {@link IMenuitem#getTarget() target}
     *    .setUpload(String | null) // nullable {@link IMenuitem#getUpload() upload}
     *    .setCheckmark(boolean) // optional {@link IMenuitem#isCheckmark() checkmark}
     *    .setChecked(boolean) // optional {@link IMenuitem#isChecked() checked}
     *    .setAutocheck(boolean) // optional {@link IMenuitem#isAutocheck() autocheck}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof IMenuitem.Builder)) {
        throw new UnsupportedOperationException("Use: new IMenuitem.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code IMenuitem} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder from(IMenuitem instance) {
      Objects.requireNonNull(instance, "instance");
      setId(instance.getId());
      @Nullable ActionHandler actionValue = instance.getAction();
      if (actionValue != null) {
        setAction(actionValue);
      }
      @Nullable List<ActionHandler> actionsValue = instance.getActions();
      if (actionsValue != null) {
        addAllActions(actionsValue);
      }
      setVisible(instance.isVisible());
      setMold(instance.getMold());
      @Nullable EventListenerMap eventListenerMapValue = instance.getEventListenerMap();
      if (eventListenerMapValue != null) {
        setEventListenerMap(eventListenerMapValue);
      }
      @Nullable Map<String, String> widgetListenersValue = instance.getWidgetListeners();
      if (widgetListenersValue != null) {
        putAllWidgetListeners(widgetListenersValue);
      }
      @Nullable Map<String, String> widgetOverridesValue = instance.getWidgetOverrides();
      if (widgetOverridesValue != null) {
        putAllWidgetOverrides(widgetOverridesValue);
      }
      @Nullable Map<String, String> clientAttributesValue = instance.getClientAttributes();
      if (clientAttributesValue != null) {
        putAllClientAttributes(clientAttributesValue);
      }
      @Nullable String leftValue = instance.getLeft();
      if (leftValue != null) {
        setLeft(leftValue);
      }
      @Nullable String topValue = instance.getTop();
      if (topValue != null) {
        setTop(topValue);
      }
      setZIndex(instance.getZIndex());
      @Nullable String heightValue = instance.getHeight();
      if (heightValue != null) {
        setHeight(heightValue);
      }
      @Nullable String widthValue = instance.getWidth();
      if (widthValue != null) {
        setWidth(widthValue);
      }
      @Nullable String tooltiptextValue = instance.getTooltiptext();
      if (tooltiptextValue != null) {
        setTooltiptext(tooltiptextValue);
      }
      @Nullable String zclassValue = instance.getZclass();
      if (zclassValue != null) {
        setZclass(zclassValue);
      }
      @Nullable String sclassValue = instance.getSclass();
      if (sclassValue != null) {
        setSclass(sclassValue);
      }
      @Nullable String styleValue = instance.getStyle();
      if (styleValue != null) {
        setStyle(styleValue);
      }
      setDraggable(instance.getDraggable());
      setFocus(instance.isFocus());
      setDroppable(instance.getDroppable());
      @Nullable String vflexValue = instance.getVflex();
      if (vflexValue != null) {
        setVflex(vflexValue);
      }
      @Nullable String hflexValue = instance.getHflex();
      if (hflexValue != null) {
        setHflex(hflexValue);
      }
      setRenderdefer(instance.getRenderdefer());
      @Nullable String clientActionValue = instance.getClientAction();
      if (clientActionValue != null) {
        setClientAction(clientActionValue);
      }
      @Nullable Integer tabindexValue = instance.getTabindex();
      if (tabindexValue != null) {
        setTabindex(tabindexValue);
      }
      @Nullable String ctrlKeysValue = instance.getCtrlKeys();
      if (ctrlKeysValue != null) {
        setCtrlKeys(ctrlKeysValue);
      }
      @Nullable String contextValue = instance.getContext();
      if (contextValue != null) {
        setContext(contextValue);
      }
      @Nullable String popupValue = instance.getPopup();
      if (popupValue != null) {
        setPopup(popupValue);
      }
      @Nullable String tooltipValue = instance.getTooltip();
      if (tooltipValue != null) {
        setTooltip(tooltipValue);
      }
      setLabel(instance.getLabel());
      @Nullable String iconSclassValue = instance.getIconSclass();
      if (iconSclassValue != null) {
        setIconSclass(iconSclassValue);
      }
      @Nullable String[] iconSclassesValue = instance.getIconSclasses();
      if (iconSclassesValue != null) {
        setIconSclasses(iconSclassesValue);
      }
      @Nullable String iconTooltipValue = instance.getIconTooltip();
      if (iconTooltipValue != null) {
        setIconTooltip(iconTooltipValue);
      }
      @Nullable String[] iconTooltipsValue = instance.getIconTooltips();
      if (iconTooltipsValue != null) {
        setIconTooltips(iconTooltipsValue);
      }
      @Nullable String imageValue = instance.getImage();
      if (imageValue != null) {
        setImage(imageValue);
      }
      @Nullable String hoverImageValue = instance.getHoverImage();
      if (hoverImageValue != null) {
        setHoverImage(hoverImageValue);
      }
      setPreloadImage(instance.isPreloadImage());
      setDisabled(instance.isDisabled());
      setWidgetClass(instance.getWidgetClass());
      @Nullable String autodisableValue = instance.getAutodisable();
      if (autodisableValue != null) {
        setAutodisable(autodisableValue);
      }
      @Nullable String hrefValue = instance.getHref();
      if (hrefValue != null) {
        setHref(hrefValue);
      }
      @Nullable String targetValue = instance.getTarget();
      if (targetValue != null) {
        setTarget(targetValue);
      }
      @Nullable String uploadValue = instance.getUpload();
      if (uploadValue != null) {
        setUpload(uploadValue);
      }
      setCheckmark(instance.isCheckmark());
      setChecked(instance.isChecked());
      setAutocheck(instance.isAutocheck());
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getId() id} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getId() id}.</em>
     * @param id The value for id 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setId(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getAction() action} attribute.
     * @param action The value for action (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setAction(@Nullable ActionHandler action) {
      this.action = action;
      return (IMenuitem.Builder) this;
    }

    /**
     * Adds one element to {@link IMenuitem#getActions() actions} list.
     * @param element A actions element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder addActions(ActionHandler element) {
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.add(element);
      return (IMenuitem.Builder) this;
    }

    /**
     * Adds elements to {@link IMenuitem#getActions() actions} list.
     * @param elements An array of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder addActions(ActionHandler... elements) {
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.add(elements);
      return (IMenuitem.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link IMenuitem#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setActions(@Nullable Iterable<? extends ActionHandler> elements) {
      if (elements == null) {
        this.actions = null;
        return (IMenuitem.Builder) this;
      }
      this.actions = ImmutableList.builder();
      return addAllActions(elements);
    }

    /**
     * Adds elements to {@link IMenuitem#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder addAllActions(Iterable<? extends ActionHandler> elements) {
      Objects.requireNonNull(elements, "actions element");
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.addAll(elements);
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#isVisible() visible} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#isVisible() visible}.</em>
     * @param visible The value for visible 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setVisible(boolean visible) {
      this.visible = visible;
      optBits |= OPT_BIT_VISIBLE;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getMold() mold} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getMold() mold}.</em>
     * @param mold The value for mold 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setMold(String mold) {
      this.mold = Objects.requireNonNull(mold, "mold");
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getEventListenerMap() eventListenerMap} attribute.
     * @param eventListenerMap The value for eventListenerMap (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setEventListenerMap(@Nullable EventListenerMap eventListenerMap) {
      this.eventListenerMap = eventListenerMap;
      return (IMenuitem.Builder) this;
    }

    /**
     * Put one entry to the {@link IMenuitem#getWidgetListeners() widgetListeners} map.
     * @param key The key in the widgetListeners map
     * @param value The associated value in the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putWidgetListeners(String key, String value) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.put(key, value);
      return (IMenuitem.Builder) this;
    }

    /**
     * Put one entry to the {@link IMenuitem#getWidgetListeners() widgetListeners} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putWidgetListeners(Map.Entry<String, ? extends String> entry) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.put(entry);
      return (IMenuitem.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link IMenuitem#getWidgetListeners() widgetListeners} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setWidgetListeners(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.widgetListeners = null;
        return (IMenuitem.Builder) this;
      }
      this.widgetListeners = ImmutableMap.builder();
      return putAllWidgetListeners(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link IMenuitem#getWidgetListeners() widgetListeners} map. Nulls are not permitted
     * @param entries The entries that will be added to the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putAllWidgetListeners(Map<String, ? extends String> entries) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.putAll(entries);
      return (IMenuitem.Builder) this;
    }

    /**
     * Put one entry to the {@link IMenuitem#getWidgetOverrides() widgetOverrides} map.
     * @param key The key in the widgetOverrides map
     * @param value The associated value in the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putWidgetOverrides(String key, String value) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.put(key, value);
      return (IMenuitem.Builder) this;
    }

    /**
     * Put one entry to the {@link IMenuitem#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putWidgetOverrides(Map.Entry<String, ? extends String> entry) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.put(entry);
      return (IMenuitem.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link IMenuitem#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setWidgetOverrides(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.widgetOverrides = null;
        return (IMenuitem.Builder) this;
      }
      this.widgetOverrides = ImmutableMap.builder();
      return putAllWidgetOverrides(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link IMenuitem#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted
     * @param entries The entries that will be added to the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putAllWidgetOverrides(Map<String, ? extends String> entries) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.putAll(entries);
      return (IMenuitem.Builder) this;
    }

    /**
     * Put one entry to the {@link IMenuitem#getClientAttributes() clientAttributes} map.
     * @param key The key in the clientAttributes map
     * @param value The associated value in the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putClientAttributes(String key, String value) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.put(key, value);
      return (IMenuitem.Builder) this;
    }

    /**
     * Put one entry to the {@link IMenuitem#getClientAttributes() clientAttributes} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putClientAttributes(Map.Entry<String, ? extends String> entry) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.put(entry);
      return (IMenuitem.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link IMenuitem#getClientAttributes() clientAttributes} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setClientAttributes(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.clientAttributes = null;
        return (IMenuitem.Builder) this;
      }
      this.clientAttributes = ImmutableMap.builder();
      return putAllClientAttributes(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link IMenuitem#getClientAttributes() clientAttributes} map. Nulls are not permitted
     * @param entries The entries that will be added to the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder putAllClientAttributes(Map<String, ? extends String> entries) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.putAll(entries);
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getLeft() left} attribute.
     * @param left The value for left (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setLeft(@Nullable String left) {
      this.left = left;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getTop() top} attribute.
     * @param top The value for top (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setTop(@Nullable String top) {
      this.top = top;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getZIndex() zIndex} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getZIndex() zIndex}.</em>
     * @param zIndex The value for zIndex 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setZIndex(int zIndex) {
      this.zIndex = zIndex;
      optBits |= OPT_BIT_Z_INDEX;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getHeight() height} attribute.
     * @param height The value for height (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setHeight(@Nullable String height) {
      this.height = height;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getWidth() width} attribute.
     * @param width The value for width (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setWidth(@Nullable String width) {
      this.width = width;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getTooltiptext() tooltiptext} attribute.
     * @param tooltiptext The value for tooltiptext (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setTooltiptext(@Nullable String tooltiptext) {
      this.tooltiptext = tooltiptext;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getZclass() zclass} attribute.
     * @param zclass The value for zclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setZclass(@Nullable String zclass) {
      this.zclass = zclass;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getSclass() sclass} attribute.
     * @param sclass The value for sclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setSclass(@Nullable String sclass) {
      this.sclass = sclass;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getStyle() style} attribute.
     * @param style The value for style (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setStyle(@Nullable String style) {
      this.style = style;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getDraggable() draggable} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getDraggable() draggable}.</em>
     * @param draggable The value for draggable 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setDraggable(String draggable) {
      this.draggable = Objects.requireNonNull(draggable, "draggable");
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#isFocus() focus} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#isFocus() focus}.</em>
     * @param focus The value for focus 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setFocus(boolean focus) {
      this.focus = focus;
      optBits |= OPT_BIT_FOCUS;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getDroppable() droppable} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getDroppable() droppable}.</em>
     * @param droppable The value for droppable 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setDroppable(String droppable) {
      this.droppable = Objects.requireNonNull(droppable, "droppable");
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getVflex() vflex} attribute.
     * @param vflex The value for vflex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setVflex(@Nullable String vflex) {
      this.vflex = vflex;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getHflex() hflex} attribute.
     * @param hflex The value for hflex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setHflex(@Nullable String hflex) {
      this.hflex = hflex;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getRenderdefer() renderdefer} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getRenderdefer() renderdefer}.</em>
     * @param renderdefer The value for renderdefer 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setRenderdefer(int renderdefer) {
      this.renderdefer = renderdefer;
      optBits |= OPT_BIT_RENDERDEFER;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getClientAction() clientAction} attribute.
     * @param clientAction The value for clientAction (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setClientAction(@Nullable String clientAction) {
      this.clientAction = clientAction;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getTabindex() tabindex} attribute.
     * @param tabindex The value for tabindex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setTabindex(@Nullable Integer tabindex) {
      this.tabindex = tabindex;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getCtrlKeys() ctrlKeys} attribute.
     * @param ctrlKeys The value for ctrlKeys (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setCtrlKeys(@Nullable String ctrlKeys) {
      this.ctrlKeys = ctrlKeys;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getContext() context} attribute.
     * @param context The value for context (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setContext(@Nullable String context) {
      this.context = context;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getPopup() popup} attribute.
     * @param popup The value for popup (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setPopup(@Nullable String popup) {
      this.popup = popup;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getTooltip() tooltip} attribute.
     * @param tooltip The value for tooltip (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setTooltip(@Nullable String tooltip) {
      this.tooltip = tooltip;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getLabel() label} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getLabel() label}.</em>
     * @param label The value for label 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setLabel(String label) {
      this.label = Objects.requireNonNull(label, "label");
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getIconSclass() iconSclass} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getIconSclass() iconSclass}.</em>
     * @param iconSclass The value for iconSclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setIconSclass(@Nullable String iconSclass) {
      this.iconSclass = iconSclass;
      optBits |= OPT_BIT_ICON_SCLASS;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getIconSclasses() iconSclasses} attribute.
     * <p><em>If not set, this attribute will have a default value as defined by {@link IMenuitem#getIconSclasses() iconSclasses}.</em>
     * @param iconSclasses The elements for iconSclasses
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setIconSclasses(String... iconSclasses) {
      this.iconSclasses = iconSclasses;
      optBits |= OPT_BIT_ICON_SCLASSES;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getIconTooltip() iconTooltip} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getIconTooltip() iconTooltip}.</em>
     * @param iconTooltip The value for iconTooltip (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setIconTooltip(@Nullable String iconTooltip) {
      this.iconTooltip = iconTooltip;
      optBits |= OPT_BIT_ICON_TOOLTIP;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getIconTooltips() iconTooltips} attribute.
     * <p><em>If not set, this attribute will have a default value as defined by {@link IMenuitem#getIconTooltips() iconTooltips}.</em>
     * @param iconTooltips The elements for iconTooltips
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setIconTooltips(String... iconTooltips) {
      this.iconTooltips = iconTooltips;
      optBits |= OPT_BIT_ICON_TOOLTIPS;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getImage() image} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getImage() image}.</em>
     * @param image The value for image (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setImage(@Nullable String image) {
      this.image = image;
      optBits |= OPT_BIT_IMAGE;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getHoverImage() hoverImage} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getHoverImage() hoverImage}.</em>
     * @param hoverImage The value for hoverImage (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setHoverImage(@Nullable String hoverImage) {
      this.hoverImage = hoverImage;
      optBits |= OPT_BIT_HOVER_IMAGE;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#isPreloadImage() preloadImage} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#isPreloadImage() preloadImage}.</em>
     * @param preloadImage The value for preloadImage 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setPreloadImage(boolean preloadImage) {
      this.preloadImage = preloadImage;
      optBits |= OPT_BIT_PRELOAD_IMAGE;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#isDisabled() disabled} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#isDisabled() disabled}.</em>
     * @param disabled The value for disabled 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setDisabled(boolean disabled) {
      this.disabled = disabled;
      optBits |= OPT_BIT_DISABLED;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getWidgetClass() widgetClass} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#getWidgetClass() widgetClass}.</em>
     * @param widgetClass The value for widgetClass 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setWidgetClass(String widgetClass) {
      this.widgetClass = Objects.requireNonNull(widgetClass, "widgetClass");
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getAutodisable() autodisable} attribute.
     * @param autodisable The value for autodisable (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setAutodisable(@Nullable String autodisable) {
      this.autodisable = autodisable;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getHref() href} attribute.
     * @param href The value for href (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setHref(@Nullable String href) {
      this.href = href;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getTarget() target} attribute.
     * @param target The value for target (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setTarget(@Nullable String target) {
      this.target = target;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#getUpload() upload} attribute.
     * @param upload The value for upload (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setUpload(@Nullable String upload) {
      this.upload = upload;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#isCheckmark() checkmark} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#isCheckmark() checkmark}.</em>
     * @param checkmark The value for checkmark 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setCheckmark(boolean checkmark) {
      this.checkmark = checkmark;
      optBits |= OPT_BIT_CHECKMARK;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#isChecked() checked} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#isChecked() checked}.</em>
     * @param checked The value for checked 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setChecked(boolean checked) {
      this.checked = checked;
      optBits |= OPT_BIT_CHECKED;
      return (IMenuitem.Builder) this;
    }

    /**
     * Initializes the value for the {@link IMenuitem#isAutocheck() autocheck} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IMenuitem#isAutocheck() autocheck}.</em>
     * @param autocheck The value for autocheck 
     * @return {@code this} builder for use in a chained invocation
     */
    public final IMenuitem.Builder setAutocheck(boolean autocheck) {
      this.autocheck = autocheck;
      optBits |= OPT_BIT_AUTOCHECK;
      return (IMenuitem.Builder) this;
    }

    /**
     * Builds a new {@link IMenuitem IMenuitem}.
     * @return An immutable instance of IMenuitem
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public IMenuitem build() {
      return ImmutableIMenuitem.validate(new ImmutableIMenuitem(this));
    }

    private boolean visibleIsSet() {
      return (optBits & OPT_BIT_VISIBLE) != 0;
    }

    private boolean zIndexIsSet() {
      return (optBits & OPT_BIT_Z_INDEX) != 0;
    }

    private boolean focusIsSet() {
      return (optBits & OPT_BIT_FOCUS) != 0;
    }

    private boolean renderdeferIsSet() {
      return (optBits & OPT_BIT_RENDERDEFER) != 0;
    }

    private boolean iconSclassIsSet() {
      return (optBits & OPT_BIT_ICON_SCLASS) != 0;
    }

    private boolean iconSclassesIsSet() {
      return (optBits & OPT_BIT_ICON_SCLASSES) != 0;
    }

    private boolean iconTooltipIsSet() {
      return (optBits & OPT_BIT_ICON_TOOLTIP) != 0;
    }

    private boolean iconTooltipsIsSet() {
      return (optBits & OPT_BIT_ICON_TOOLTIPS) != 0;
    }

    private boolean imageIsSet() {
      return (optBits & OPT_BIT_IMAGE) != 0;
    }

    private boolean hoverImageIsSet() {
      return (optBits & OPT_BIT_HOVER_IMAGE) != 0;
    }

    private boolean preloadImageIsSet() {
      return (optBits & OPT_BIT_PRELOAD_IMAGE) != 0;
    }

    private boolean disabledIsSet() {
      return (optBits & OPT_BIT_DISABLED) != 0;
    }

    private boolean checkmarkIsSet() {
      return (optBits & OPT_BIT_CHECKMARK) != 0;
    }

    private boolean checkedIsSet() {
      return (optBits & OPT_BIT_CHECKED) != 0;
    }

    private boolean autocheckIsSet() {
      return (optBits & OPT_BIT_AUTOCHECK) != 0;
    }
  }
}
