package org.zkoss.stateless.sul;

import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.primitives.Booleans;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.TimeZone;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;
import org.zkoss.stateless.immutable.StatelessOnly;
import org.zkoss.stateless.util.ActionHandler;
import org.zkoss.zk.ui.sys.EventListenerMap;
import org.zkoss.zkmax.zul.Timepicker;

/**
 * Immutable implementation of {@link ITimepicker}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new ITimepicker.Builder()}.
 */
@Generated(from = "ITimepicker", generator = "Immutables")
@SuppressWarnings({"all"})
@ParametersAreNonnullByDefault
@javax.annotation.processing.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableITimepicker implements ITimepicker {
  private final String id;
  private final @Nullable ActionHandler action;
  private final @Nullable ImmutableList<ActionHandler> actions;
  private final boolean visible;
  private final String mold;
  private final @Nullable EventListenerMap eventListenerMap;
  private final @Nullable ImmutableMap<String, String> widgetListeners;
  private final @Nullable ImmutableMap<String, String> widgetOverrides;
  private final @Nullable ImmutableMap<String, String> clientAttributes;
  private final @Nullable String left;
  private final @Nullable String top;
  private final int zIndex;
  private final @Nullable String height;
  private final @Nullable String width;
  private final @Nullable String tooltiptext;
  private final @Nullable String zclass;
  private final @Nullable String sclass;
  private final @Nullable String style;
  private final String draggable;
  private final boolean focus;
  private final String droppable;
  private final @Nullable String vflex;
  private final @Nullable String hflex;
  private final int renderdefer;
  private final @Nullable String clientAction;
  private final @Nullable Integer tabindex;
  private final @Nullable String ctrlKeys;
  private final @Nullable String context;
  private final @Nullable String popup;
  private final @Nullable String tooltip;
  private final @Nullable String name;
  private final @Nullable String placeholder;
  private final @Nullable ImmutableMap<String, String> inputAttributes;
  private final @Nullable Date value;
  private final @Nullable String constraint;
  private final @Nullable String errorboxSclass;
  private final @Nullable String errorboxIconSclass;
  private final boolean disabled;
  private final boolean inplace;
  private final int maxlength;
  private final boolean instant;
  private final @Nullable String errorMessage;
  private final @Nullable String format;
  private final @Nullable Locale locale;
  private final TimeZone timeZone;
  private final @Nullable ZonedDateTime valueInZonedDateTime;
  private final @Nullable LocalDateTime valueInLocalDateTime;
  private final @Nullable LocalDate valueInLocalDate;
  private final @Nullable LocalTime valueInLocalTime;
  private final String widgetClass;
  private final @Nullable Date min;
  private final @Nullable Date max;
  private final int cols;
  private final int interval;
  private final boolean readonly;

  private ImmutableITimepicker(ImmutableITimepicker.Builder builder) {
    this.action = builder.action;
    this.actions = builder.actions == null ? null : builder.actions.build();
    this.eventListenerMap = builder.eventListenerMap;
    this.widgetListeners = builder.widgetListeners == null ? null : builder.widgetListeners.build();
    this.widgetOverrides = builder.widgetOverrides == null ? null : builder.widgetOverrides.build();
    this.clientAttributes = builder.clientAttributes == null ? null : builder.clientAttributes.build();
    this.left = builder.left;
    this.top = builder.top;
    this.height = builder.height;
    this.width = builder.width;
    this.tooltiptext = builder.tooltiptext;
    this.zclass = builder.zclass;
    this.sclass = builder.sclass;
    this.style = builder.style;
    this.vflex = builder.vflex;
    this.hflex = builder.hflex;
    this.clientAction = builder.clientAction;
    this.tabindex = builder.tabindex;
    this.ctrlKeys = builder.ctrlKeys;
    this.context = builder.context;
    this.popup = builder.popup;
    this.tooltip = builder.tooltip;
    this.name = builder.name;
    this.placeholder = builder.placeholder;
    this.inputAttributes = builder.inputAttributes == null ? null : builder.inputAttributes.build();
    this.value = builder.value;
    this.constraint = builder.constraint;
    this.errorboxSclass = builder.errorboxSclass;
    this.errorboxIconSclass = builder.errorboxIconSclass;
    this.errorMessage = builder.errorMessage;
    this.format = builder.format;
    this.locale = builder.locale;
    this.min = builder.min;
    this.max = builder.max;
    if (builder.id != null) {
      initShim.setId(builder.id);
    }
    if (builder.visibleIsSet()) {
      initShim.setVisible(builder.visible);
    }
    if (builder.mold != null) {
      initShim.setMold(builder.mold);
    }
    if (builder.zIndexIsSet()) {
      initShim.setZIndex(builder.zIndex);
    }
    if (builder.draggable != null) {
      initShim.setDraggable(builder.draggable);
    }
    if (builder.focusIsSet()) {
      initShim.setFocus(builder.focus);
    }
    if (builder.droppable != null) {
      initShim.setDroppable(builder.droppable);
    }
    if (builder.renderdeferIsSet()) {
      initShim.setRenderdefer(builder.renderdefer);
    }
    if (builder.disabledIsSet()) {
      initShim.setDisabled(builder.disabled);
    }
    if (builder.inplaceIsSet()) {
      initShim.setInplace(builder.inplace);
    }
    if (builder.maxlengthIsSet()) {
      initShim.setMaxlength(builder.maxlength);
    }
    if (builder.instantIsSet()) {
      initShim.setInstant(builder.instant);
    }
    if (builder.timeZone != null) {
      initShim.setTimeZone(builder.timeZone);
    }
    if (builder.valueInZonedDateTimeIsSet()) {
      initShim.setValueInZonedDateTime(builder.valueInZonedDateTime);
    }
    if (builder.valueInLocalDateTimeIsSet()) {
      initShim.setValueInLocalDateTime(builder.valueInLocalDateTime);
    }
    if (builder.valueInLocalDateIsSet()) {
      initShim.setValueInLocalDate(builder.valueInLocalDate);
    }
    if (builder.valueInLocalTimeIsSet()) {
      initShim.setValueInLocalTime(builder.valueInLocalTime);
    }
    if (builder.widgetClass != null) {
      initShim.setWidgetClass(builder.widgetClass);
    }
    if (builder.colsIsSet()) {
      initShim.setCols(builder.cols);
    }
    if (builder.intervalIsSet()) {
      initShim.setInterval(builder.interval);
    }
    if (builder.readonlyIsSet()) {
      initShim.setReadonly(builder.readonly);
    }
    this.id = initShim.getId();
    this.visible = initShim.isVisible();
    this.mold = initShim.getMold();
    this.zIndex = initShim.getZIndex();
    this.draggable = initShim.getDraggable();
    this.focus = initShim.isFocus();
    this.droppable = initShim.getDroppable();
    this.renderdefer = initShim.getRenderdefer();
    this.disabled = initShim.isDisabled();
    this.inplace = initShim.isInplace();
    this.maxlength = initShim.getMaxlength();
    this.instant = initShim.getInstant();
    this.timeZone = initShim.getTimeZone();
    this.valueInZonedDateTime = initShim.getValueInZonedDateTime();
    this.valueInLocalDateTime = initShim.getValueInLocalDateTime();
    this.valueInLocalDate = initShim.getValueInLocalDate();
    this.valueInLocalTime = initShim.getValueInLocalTime();
    this.widgetClass = initShim.getWidgetClass();
    this.cols = initShim.getCols();
    this.interval = initShim.getInterval();
    this.readonly = initShim.isReadonly();
    this.initShim = null;
  }

  private ImmutableITimepicker(
      String id,
      @Nullable ActionHandler action,
      @Nullable ImmutableList<ActionHandler> actions,
      boolean visible,
      String mold,
      @Nullable EventListenerMap eventListenerMap,
      @Nullable ImmutableMap<String, String> widgetListeners,
      @Nullable ImmutableMap<String, String> widgetOverrides,
      @Nullable ImmutableMap<String, String> clientAttributes,
      @Nullable String left,
      @Nullable String top,
      int zIndex,
      @Nullable String height,
      @Nullable String width,
      @Nullable String tooltiptext,
      @Nullable String zclass,
      @Nullable String sclass,
      @Nullable String style,
      String draggable,
      boolean focus,
      String droppable,
      @Nullable String vflex,
      @Nullable String hflex,
      int renderdefer,
      @Nullable String clientAction,
      @Nullable Integer tabindex,
      @Nullable String ctrlKeys,
      @Nullable String context,
      @Nullable String popup,
      @Nullable String tooltip,
      @Nullable String name,
      @Nullable String placeholder,
      @Nullable ImmutableMap<String, String> inputAttributes,
      @Nullable Date value,
      @Nullable String constraint,
      @Nullable String errorboxSclass,
      @Nullable String errorboxIconSclass,
      boolean disabled,
      boolean inplace,
      int maxlength,
      boolean instant,
      @Nullable String errorMessage,
      @Nullable String format,
      @Nullable Locale locale,
      TimeZone timeZone,
      @Nullable ZonedDateTime valueInZonedDateTime,
      @Nullable LocalDateTime valueInLocalDateTime,
      @Nullable LocalDate valueInLocalDate,
      @Nullable LocalTime valueInLocalTime,
      String widgetClass,
      @Nullable Date min,
      @Nullable Date max,
      int cols,
      int interval,
      boolean readonly) {
    this.id = id;
    this.action = action;
    this.actions = actions;
    this.visible = visible;
    this.mold = mold;
    this.eventListenerMap = eventListenerMap;
    this.widgetListeners = widgetListeners;
    this.widgetOverrides = widgetOverrides;
    this.clientAttributes = clientAttributes;
    this.left = left;
    this.top = top;
    this.zIndex = zIndex;
    this.height = height;
    this.width = width;
    this.tooltiptext = tooltiptext;
    this.zclass = zclass;
    this.sclass = sclass;
    this.style = style;
    this.draggable = draggable;
    this.focus = focus;
    this.droppable = droppable;
    this.vflex = vflex;
    this.hflex = hflex;
    this.renderdefer = renderdefer;
    this.clientAction = clientAction;
    this.tabindex = tabindex;
    this.ctrlKeys = ctrlKeys;
    this.context = context;
    this.popup = popup;
    this.tooltip = tooltip;
    this.name = name;
    this.placeholder = placeholder;
    this.inputAttributes = inputAttributes;
    this.value = value;
    this.constraint = constraint;
    this.errorboxSclass = errorboxSclass;
    this.errorboxIconSclass = errorboxIconSclass;
    this.disabled = disabled;
    this.inplace = inplace;
    this.maxlength = maxlength;
    this.instant = instant;
    this.errorMessage = errorMessage;
    this.format = format;
    this.locale = locale;
    this.timeZone = timeZone;
    this.valueInZonedDateTime = valueInZonedDateTime;
    this.valueInLocalDateTime = valueInLocalDateTime;
    this.valueInLocalDate = valueInLocalDate;
    this.valueInLocalTime = valueInLocalTime;
    this.widgetClass = widgetClass;
    this.min = min;
    this.max = max;
    this.cols = cols;
    this.interval = interval;
    this.readonly = readonly;
    this.initShim = null;
  }

  private static final byte STAGE_INITIALIZING = -1;
  private static final byte STAGE_UNINITIALIZED = 0;
  private static final byte STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  @Generated(from = "ITimepicker", generator = "Immutables")
  private final class InitShim {
    private byte idBuildStage = STAGE_UNINITIALIZED;
    private String id;

    String getId() {
      if (idBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (idBuildStage == STAGE_UNINITIALIZED) {
        idBuildStage = STAGE_INITIALIZING;
        this.id = Objects.requireNonNull(getIdInitialize(), "id");
        idBuildStage = STAGE_INITIALIZED;
      }
      return this.id;
    }

    void setId(String id) {
      this.id = id;
      idBuildStage = STAGE_INITIALIZED;
    }

    private byte visibleBuildStage = STAGE_UNINITIALIZED;
    private boolean visible;

    boolean isVisible() {
      if (visibleBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (visibleBuildStage == STAGE_UNINITIALIZED) {
        visibleBuildStage = STAGE_INITIALIZING;
        this.visible = isVisibleInitialize();
        visibleBuildStage = STAGE_INITIALIZED;
      }
      return this.visible;
    }

    void setVisible(boolean visible) {
      this.visible = visible;
      visibleBuildStage = STAGE_INITIALIZED;
    }

    private byte moldBuildStage = STAGE_UNINITIALIZED;
    private String mold;

    String getMold() {
      if (moldBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (moldBuildStage == STAGE_UNINITIALIZED) {
        moldBuildStage = STAGE_INITIALIZING;
        this.mold = Objects.requireNonNull(getMoldInitialize(), "mold");
        moldBuildStage = STAGE_INITIALIZED;
      }
      return this.mold;
    }

    void setMold(String mold) {
      this.mold = mold;
      moldBuildStage = STAGE_INITIALIZED;
    }

    private byte zIndexBuildStage = STAGE_UNINITIALIZED;
    private int zIndex;

    int getZIndex() {
      if (zIndexBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (zIndexBuildStage == STAGE_UNINITIALIZED) {
        zIndexBuildStage = STAGE_INITIALIZING;
        this.zIndex = getZIndexInitialize();
        zIndexBuildStage = STAGE_INITIALIZED;
      }
      return this.zIndex;
    }

    void setZIndex(int zIndex) {
      this.zIndex = zIndex;
      zIndexBuildStage = STAGE_INITIALIZED;
    }

    private byte draggableBuildStage = STAGE_UNINITIALIZED;
    private String draggable;

    String getDraggable() {
      if (draggableBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (draggableBuildStage == STAGE_UNINITIALIZED) {
        draggableBuildStage = STAGE_INITIALIZING;
        this.draggable = Objects.requireNonNull(getDraggableInitialize(), "draggable");
        draggableBuildStage = STAGE_INITIALIZED;
      }
      return this.draggable;
    }

    void setDraggable(String draggable) {
      this.draggable = draggable;
      draggableBuildStage = STAGE_INITIALIZED;
    }

    private byte focusBuildStage = STAGE_UNINITIALIZED;
    private boolean focus;

    boolean isFocus() {
      if (focusBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (focusBuildStage == STAGE_UNINITIALIZED) {
        focusBuildStage = STAGE_INITIALIZING;
        this.focus = isFocusInitialize();
        focusBuildStage = STAGE_INITIALIZED;
      }
      return this.focus;
    }

    void setFocus(boolean focus) {
      this.focus = focus;
      focusBuildStage = STAGE_INITIALIZED;
    }

    private byte droppableBuildStage = STAGE_UNINITIALIZED;
    private String droppable;

    String getDroppable() {
      if (droppableBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (droppableBuildStage == STAGE_UNINITIALIZED) {
        droppableBuildStage = STAGE_INITIALIZING;
        this.droppable = Objects.requireNonNull(getDroppableInitialize(), "droppable");
        droppableBuildStage = STAGE_INITIALIZED;
      }
      return this.droppable;
    }

    void setDroppable(String droppable) {
      this.droppable = droppable;
      droppableBuildStage = STAGE_INITIALIZED;
    }

    private byte renderdeferBuildStage = STAGE_UNINITIALIZED;
    private int renderdefer;

    int getRenderdefer() {
      if (renderdeferBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (renderdeferBuildStage == STAGE_UNINITIALIZED) {
        renderdeferBuildStage = STAGE_INITIALIZING;
        this.renderdefer = getRenderdeferInitialize();
        renderdeferBuildStage = STAGE_INITIALIZED;
      }
      return this.renderdefer;
    }

    void setRenderdefer(int renderdefer) {
      this.renderdefer = renderdefer;
      renderdeferBuildStage = STAGE_INITIALIZED;
    }

    private byte disabledBuildStage = STAGE_UNINITIALIZED;
    private boolean disabled;

    boolean isDisabled() {
      if (disabledBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (disabledBuildStage == STAGE_UNINITIALIZED) {
        disabledBuildStage = STAGE_INITIALIZING;
        this.disabled = isDisabledInitialize();
        disabledBuildStage = STAGE_INITIALIZED;
      }
      return this.disabled;
    }

    void setDisabled(boolean disabled) {
      this.disabled = disabled;
      disabledBuildStage = STAGE_INITIALIZED;
    }

    private byte inplaceBuildStage = STAGE_UNINITIALIZED;
    private boolean inplace;

    boolean isInplace() {
      if (inplaceBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (inplaceBuildStage == STAGE_UNINITIALIZED) {
        inplaceBuildStage = STAGE_INITIALIZING;
        this.inplace = isInplaceInitialize();
        inplaceBuildStage = STAGE_INITIALIZED;
      }
      return this.inplace;
    }

    void setInplace(boolean inplace) {
      this.inplace = inplace;
      inplaceBuildStage = STAGE_INITIALIZED;
    }

    private byte maxlengthBuildStage = STAGE_UNINITIALIZED;
    private int maxlength;

    int getMaxlength() {
      if (maxlengthBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (maxlengthBuildStage == STAGE_UNINITIALIZED) {
        maxlengthBuildStage = STAGE_INITIALIZING;
        this.maxlength = getMaxlengthInitialize();
        maxlengthBuildStage = STAGE_INITIALIZED;
      }
      return this.maxlength;
    }

    void setMaxlength(int maxlength) {
      this.maxlength = maxlength;
      maxlengthBuildStage = STAGE_INITIALIZED;
    }

    private byte instantBuildStage = STAGE_UNINITIALIZED;
    private boolean instant;

    boolean getInstant() {
      if (instantBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (instantBuildStage == STAGE_UNINITIALIZED) {
        instantBuildStage = STAGE_INITIALIZING;
        this.instant = getInstantInitialize();
        instantBuildStage = STAGE_INITIALIZED;
      }
      return this.instant;
    }

    void setInstant(boolean instant) {
      this.instant = instant;
      instantBuildStage = STAGE_INITIALIZED;
    }

    private byte timeZoneBuildStage = STAGE_UNINITIALIZED;
    private TimeZone timeZone;

    TimeZone getTimeZone() {
      if (timeZoneBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (timeZoneBuildStage == STAGE_UNINITIALIZED) {
        timeZoneBuildStage = STAGE_INITIALIZING;
        this.timeZone = Objects.requireNonNull(getTimeZoneInitialize(), "timeZone");
        timeZoneBuildStage = STAGE_INITIALIZED;
      }
      return this.timeZone;
    }

    void setTimeZone(TimeZone timeZone) {
      this.timeZone = timeZone;
      timeZoneBuildStage = STAGE_INITIALIZED;
    }

    private byte valueInZonedDateTimeBuildStage = STAGE_UNINITIALIZED;
    private ZonedDateTime valueInZonedDateTime;

    ZonedDateTime getValueInZonedDateTime() {
      if (valueInZonedDateTimeBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (valueInZonedDateTimeBuildStage == STAGE_UNINITIALIZED) {
        valueInZonedDateTimeBuildStage = STAGE_INITIALIZING;
        this.valueInZonedDateTime = getValueInZonedDateTimeInitialize();
        valueInZonedDateTimeBuildStage = STAGE_INITIALIZED;
      }
      return this.valueInZonedDateTime;
    }

    void setValueInZonedDateTime(ZonedDateTime valueInZonedDateTime) {
      this.valueInZonedDateTime = valueInZonedDateTime;
      valueInZonedDateTimeBuildStage = STAGE_INITIALIZED;
    }

    private byte valueInLocalDateTimeBuildStage = STAGE_UNINITIALIZED;
    private LocalDateTime valueInLocalDateTime;

    LocalDateTime getValueInLocalDateTime() {
      if (valueInLocalDateTimeBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (valueInLocalDateTimeBuildStage == STAGE_UNINITIALIZED) {
        valueInLocalDateTimeBuildStage = STAGE_INITIALIZING;
        this.valueInLocalDateTime = getValueInLocalDateTimeInitialize();
        valueInLocalDateTimeBuildStage = STAGE_INITIALIZED;
      }
      return this.valueInLocalDateTime;
    }

    void setValueInLocalDateTime(LocalDateTime valueInLocalDateTime) {
      this.valueInLocalDateTime = valueInLocalDateTime;
      valueInLocalDateTimeBuildStage = STAGE_INITIALIZED;
    }

    private byte valueInLocalDateBuildStage = STAGE_UNINITIALIZED;
    private LocalDate valueInLocalDate;

    LocalDate getValueInLocalDate() {
      if (valueInLocalDateBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (valueInLocalDateBuildStage == STAGE_UNINITIALIZED) {
        valueInLocalDateBuildStage = STAGE_INITIALIZING;
        this.valueInLocalDate = getValueInLocalDateInitialize();
        valueInLocalDateBuildStage = STAGE_INITIALIZED;
      }
      return this.valueInLocalDate;
    }

    void setValueInLocalDate(LocalDate valueInLocalDate) {
      this.valueInLocalDate = valueInLocalDate;
      valueInLocalDateBuildStage = STAGE_INITIALIZED;
    }

    private byte valueInLocalTimeBuildStage = STAGE_UNINITIALIZED;
    private LocalTime valueInLocalTime;

    LocalTime getValueInLocalTime() {
      if (valueInLocalTimeBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (valueInLocalTimeBuildStage == STAGE_UNINITIALIZED) {
        valueInLocalTimeBuildStage = STAGE_INITIALIZING;
        this.valueInLocalTime = getValueInLocalTimeInitialize();
        valueInLocalTimeBuildStage = STAGE_INITIALIZED;
      }
      return this.valueInLocalTime;
    }

    void setValueInLocalTime(LocalTime valueInLocalTime) {
      this.valueInLocalTime = valueInLocalTime;
      valueInLocalTimeBuildStage = STAGE_INITIALIZED;
    }

    private byte widgetClassBuildStage = STAGE_UNINITIALIZED;
    private String widgetClass;

    String getWidgetClass() {
      if (widgetClassBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (widgetClassBuildStage == STAGE_UNINITIALIZED) {
        widgetClassBuildStage = STAGE_INITIALIZING;
        this.widgetClass = Objects.requireNonNull(getWidgetClassInitialize(), "widgetClass");
        widgetClassBuildStage = STAGE_INITIALIZED;
      }
      return this.widgetClass;
    }

    void setWidgetClass(String widgetClass) {
      this.widgetClass = widgetClass;
      widgetClassBuildStage = STAGE_INITIALIZED;
    }

    private byte colsBuildStage = STAGE_UNINITIALIZED;
    private int cols;

    int getCols() {
      if (colsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (colsBuildStage == STAGE_UNINITIALIZED) {
        colsBuildStage = STAGE_INITIALIZING;
        this.cols = getColsInitialize();
        colsBuildStage = STAGE_INITIALIZED;
      }
      return this.cols;
    }

    void setCols(int cols) {
      this.cols = cols;
      colsBuildStage = STAGE_INITIALIZED;
    }

    private byte intervalBuildStage = STAGE_UNINITIALIZED;
    private int interval;

    int getInterval() {
      if (intervalBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (intervalBuildStage == STAGE_UNINITIALIZED) {
        intervalBuildStage = STAGE_INITIALIZING;
        this.interval = getIntervalInitialize();
        intervalBuildStage = STAGE_INITIALIZED;
      }
      return this.interval;
    }

    void setInterval(int interval) {
      this.interval = interval;
      intervalBuildStage = STAGE_INITIALIZED;
    }

    private byte readonlyBuildStage = STAGE_UNINITIALIZED;
    private boolean readonly;

    boolean isReadonly() {
      if (readonlyBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (readonlyBuildStage == STAGE_UNINITIALIZED) {
        readonlyBuildStage = STAGE_INITIALIZING;
        this.readonly = isReadonlyInitialize();
        readonlyBuildStage = STAGE_INITIALIZED;
      }
      return this.readonly;
    }

    void setReadonly(boolean readonly) {
      this.readonly = readonly;
      readonlyBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      List<String> attributes = new ArrayList<>();
      if (idBuildStage == STAGE_INITIALIZING) attributes.add("id");
      if (visibleBuildStage == STAGE_INITIALIZING) attributes.add("visible");
      if (moldBuildStage == STAGE_INITIALIZING) attributes.add("mold");
      if (zIndexBuildStage == STAGE_INITIALIZING) attributes.add("zIndex");
      if (draggableBuildStage == STAGE_INITIALIZING) attributes.add("draggable");
      if (focusBuildStage == STAGE_INITIALIZING) attributes.add("focus");
      if (droppableBuildStage == STAGE_INITIALIZING) attributes.add("droppable");
      if (renderdeferBuildStage == STAGE_INITIALIZING) attributes.add("renderdefer");
      if (disabledBuildStage == STAGE_INITIALIZING) attributes.add("disabled");
      if (inplaceBuildStage == STAGE_INITIALIZING) attributes.add("inplace");
      if (maxlengthBuildStage == STAGE_INITIALIZING) attributes.add("maxlength");
      if (instantBuildStage == STAGE_INITIALIZING) attributes.add("instant");
      if (timeZoneBuildStage == STAGE_INITIALIZING) attributes.add("timeZone");
      if (valueInZonedDateTimeBuildStage == STAGE_INITIALIZING) attributes.add("valueInZonedDateTime");
      if (valueInLocalDateTimeBuildStage == STAGE_INITIALIZING) attributes.add("valueInLocalDateTime");
      if (valueInLocalDateBuildStage == STAGE_INITIALIZING) attributes.add("valueInLocalDate");
      if (valueInLocalTimeBuildStage == STAGE_INITIALIZING) attributes.add("valueInLocalTime");
      if (widgetClassBuildStage == STAGE_INITIALIZING) attributes.add("widgetClass");
      if (colsBuildStage == STAGE_INITIALIZING) attributes.add("cols");
      if (intervalBuildStage == STAGE_INITIALIZING) attributes.add("interval");
      if (readonlyBuildStage == STAGE_INITIALIZING) attributes.add("readonly");
      return "Cannot build ITimepicker, attribute initializers form cycle " + attributes;
    }
  }

  private String getIdInitialize() {
    return ITimepicker.super.getId();
  }

  private boolean isVisibleInitialize() {
    return ITimepicker.super.isVisible();
  }

  private String getMoldInitialize() {
    return ITimepicker.super.getMold();
  }

  private int getZIndexInitialize() {
    return ITimepicker.super.getZIndex();
  }

  private String getDraggableInitialize() {
    return ITimepicker.super.getDraggable();
  }

  private boolean isFocusInitialize() {
    return ITimepicker.super.isFocus();
  }

  private String getDroppableInitialize() {
    return ITimepicker.super.getDroppable();
  }

  private int getRenderdeferInitialize() {
    return ITimepicker.super.getRenderdefer();
  }

  private boolean isDisabledInitialize() {
    return ITimepicker.super.isDisabled();
  }

  private boolean isInplaceInitialize() {
    return ITimepicker.super.isInplace();
  }

  private int getMaxlengthInitialize() {
    return ITimepicker.super.getMaxlength();
  }

  private boolean getInstantInitialize() {
    return ITimepicker.super.getInstant();
  }

  private TimeZone getTimeZoneInitialize() {
    return ITimepicker.super.getTimeZone();
  }

  private @Nullable ZonedDateTime getValueInZonedDateTimeInitialize() {
    return ITimepicker.super.getValueInZonedDateTime();
  }

  private @Nullable LocalDateTime getValueInLocalDateTimeInitialize() {
    return ITimepicker.super.getValueInLocalDateTime();
  }

  private @Nullable LocalDate getValueInLocalDateInitialize() {
    return ITimepicker.super.getValueInLocalDate();
  }

  private @Nullable LocalTime getValueInLocalTimeInitialize() {
    return ITimepicker.super.getValueInLocalTime();
  }

  private String getWidgetClassInitialize() {
    return ITimepicker.super.getWidgetClass();
  }

  private int getColsInitialize() {
    return ITimepicker.super.getCols();
  }

  private int getIntervalInitialize() {
    return ITimepicker.super.getInterval();
  }

  private boolean isReadonlyInitialize() {
    return ITimepicker.super.isReadonly();
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @Override
  public String getId() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getId()
        : this.id;
  }

  /**
   * @return The value of the {@code action} attribute
   */
  @StatelessOnly
  @Override
  public @Nullable ActionHandler getAction() {
    return action;
  }

  /**
   * @return The value of the {@code actions} attribute
   */
  @StatelessOnly
  @Override
  public @Nullable ImmutableList<ActionHandler> getActions() {
    return actions;
  }

  /**
   * @return The value of the {@code visible} attribute
   */
  @Override
  public boolean isVisible() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isVisible()
        : this.visible;
  }

  /**
   * @return The value of the {@code mold} attribute
   */
  @Override
  public String getMold() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getMold()
        : this.mold;
  }

  /**
   * @return The value of the {@code eventListenerMap} attribute
   */
  @Override
  public @Nullable EventListenerMap getEventListenerMap() {
    return eventListenerMap;
  }

  /**
   * @return The value of the {@code widgetListeners} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getWidgetListeners() {
    return widgetListeners;
  }

  /**
   * @return The value of the {@code widgetOverrides} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getWidgetOverrides() {
    return widgetOverrides;
  }

  /**
   * @return The value of the {@code clientAttributes} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getClientAttributes() {
    return clientAttributes;
  }

  /**
   * @return The value of the {@code left} attribute
   */
  @Override
  public @Nullable String getLeft() {
    return left;
  }

  /**
   * @return The value of the {@code top} attribute
   */
  @Override
  public @Nullable String getTop() {
    return top;
  }

  /**
   * @return The value of the {@code zIndex} attribute
   */
  @Override
  public int getZIndex() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getZIndex()
        : this.zIndex;
  }

  /**
   * @return The value of the {@code height} attribute
   */
  @Override
  public @Nullable String getHeight() {
    return height;
  }

  /**
   * @return The value of the {@code width} attribute
   */
  @Override
  public @Nullable String getWidth() {
    return width;
  }

  /**
   * @return The value of the {@code tooltiptext} attribute
   */
  @Override
  public @Nullable String getTooltiptext() {
    return tooltiptext;
  }

  /**
   * @return The value of the {@code zclass} attribute
   */
  @Override
  public @Nullable String getZclass() {
    return zclass;
  }

  /**
   * @return The value of the {@code sclass} attribute
   */
  @Override
  public @Nullable String getSclass() {
    return sclass;
  }

  /**
   * @return The value of the {@code style} attribute
   */
  @Override
  public @Nullable String getStyle() {
    return style;
  }

  /**
   * @return The value of the {@code draggable} attribute
   */
  @Override
  public String getDraggable() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDraggable()
        : this.draggable;
  }

  /**
   * @return The value of the {@code focus} attribute
   */
  @Override
  public boolean isFocus() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isFocus()
        : this.focus;
  }

  /**
   * @return The value of the {@code droppable} attribute
   */
  @Override
  public String getDroppable() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDroppable()
        : this.droppable;
  }

  /**
   * @return The value of the {@code vflex} attribute
   */
  @Override
  public @Nullable String getVflex() {
    return vflex;
  }

  /**
   * @return The value of the {@code hflex} attribute
   */
  @Override
  public @Nullable String getHflex() {
    return hflex;
  }

  /**
   * @return The value of the {@code renderdefer} attribute
   */
  @Override
  public int getRenderdefer() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getRenderdefer()
        : this.renderdefer;
  }

  /**
   * @return The value of the {@code clientAction} attribute
   */
  @Override
  public @Nullable String getClientAction() {
    return clientAction;
  }

  /**
   * @return The value of the {@code tabindex} attribute
   */
  @Override
  public @Nullable Integer getTabindex() {
    return tabindex;
  }

  /**
   * @return The value of the {@code ctrlKeys} attribute
   */
  @Override
  public @Nullable String getCtrlKeys() {
    return ctrlKeys;
  }

  /**
   * @return The value of the {@code context} attribute
   */
  @Override
  public @Nullable String getContext() {
    return context;
  }

  /**
   * @return The value of the {@code popup} attribute
   */
  @Override
  public @Nullable String getPopup() {
    return popup;
  }

  /**
   * @return The value of the {@code tooltip} attribute
   */
  @Override
  public @Nullable String getTooltip() {
    return tooltip;
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @Override
  public @Nullable String getName() {
    return name;
  }

  /**
   * @return The value of the {@code placeholder} attribute
   */
  @Override
  public @Nullable String getPlaceholder() {
    return placeholder;
  }

  /**
   * @return The value of the {@code inputAttributes} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getInputAttributes() {
    return inputAttributes;
  }

  /**
   * @return The value of the {@code value} attribute
   */
  @Override
  public @Nullable Date getValue() {
    return value;
  }

  /**
   * @return The value of the {@code constraint} attribute
   */
  @Override
  public @Nullable String getConstraint() {
    return constraint;
  }

  /**
   * @return The value of the {@code errorboxSclass} attribute
   */
  @Override
  public @Nullable String getErrorboxSclass() {
    return errorboxSclass;
  }

  /**
   * @return The value of the {@code errorboxIconSclass} attribute
   */
  @Override
  public @Nullable String getErrorboxIconSclass() {
    return errorboxIconSclass;
  }

  /**
   * @return The value of the {@code disabled} attribute
   */
  @Override
  public boolean isDisabled() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isDisabled()
        : this.disabled;
  }

  /**
   * @return The value of the {@code inplace} attribute
   */
  @Override
  public boolean isInplace() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isInplace()
        : this.inplace;
  }

  /**
   * @return The value of the {@code maxlength} attribute
   */
  @Override
  public int getMaxlength() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getMaxlength()
        : this.maxlength;
  }

  /**
   * @return The value of the {@code instant} attribute
   */
  @Override
  public boolean getInstant() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getInstant()
        : this.instant;
  }

  /**
   * @return The value of the {@code errorMessage} attribute
   */
  @Override
  public @Nullable String getErrorMessage() {
    return errorMessage;
  }

  /**
   * @return The value of the {@code format} attribute
   */
  @Override
  public @Nullable String getFormat() {
    return format;
  }

  /**
   * @return The value of the {@code locale} attribute
   */
  @Override
  public @Nullable Locale getLocale() {
    return locale;
  }

  /**
   * @return The value of the {@code timeZone} attribute
   */
  @Override
  public TimeZone getTimeZone() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getTimeZone()
        : this.timeZone;
  }

  /**
   * @return The value of the {@code valueInZonedDateTime} attribute
   */
  @Override
  public @Nullable ZonedDateTime getValueInZonedDateTime() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getValueInZonedDateTime()
        : this.valueInZonedDateTime;
  }

  /**
   * @return The value of the {@code valueInLocalDateTime} attribute
   */
  @Override
  public @Nullable LocalDateTime getValueInLocalDateTime() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getValueInLocalDateTime()
        : this.valueInLocalDateTime;
  }

  /**
   * @return The value of the {@code valueInLocalDate} attribute
   */
  @Override
  public @Nullable LocalDate getValueInLocalDate() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getValueInLocalDate()
        : this.valueInLocalDate;
  }

  /**
   * @return The value of the {@code valueInLocalTime} attribute
   */
  @Override
  public @Nullable LocalTime getValueInLocalTime() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getValueInLocalTime()
        : this.valueInLocalTime;
  }

  /**
   * @return The value of the {@code widgetClass} attribute
   */
  @Override
  public String getWidgetClass() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getWidgetClass()
        : this.widgetClass;
  }

  /**
   * @return The value of the {@code min} attribute
   */
  @Override
  public @Nullable Date getMin() {
    return min;
  }

  /**
   * @return The value of the {@code max} attribute
   */
  @Override
  public @Nullable Date getMax() {
    return max;
  }

  /**
   * @return The value of the {@code cols} attribute
   */
  @Override
  public int getCols() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getCols()
        : this.cols;
  }

  /**
   * @return The value of the {@code interval} attribute
   */
  @Override
  public int getInterval() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getInterval()
        : this.interval;
  }

  /**
   * @return The value of the {@code readonly} attribute
   */
  @Override
  public boolean isReadonly() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isReadonly()
        : this.readonly;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getId() id} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for id
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (this.id.equals(newValue)) return this;
    return validate(new ImmutableITimepicker(
        newValue,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getAction() action} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for action (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withAction(@Nullable ActionHandler value) {
    if (this.action == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        value,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ITimepicker#getActions() actions}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITimepicker withActions(@Nullable ActionHandler... elements) {
    if (elements == null) {
      return validate(new ImmutableITimepicker(
          this.id,
          this.action,
          null,
          this.visible,
          this.mold,
          this.eventListenerMap,
          this.widgetListeners,
          this.widgetOverrides,
          this.clientAttributes,
          this.left,
          this.top,
          this.zIndex,
          this.height,
          this.width,
          this.tooltiptext,
          this.zclass,
          this.sclass,
          this.style,
          this.draggable,
          this.focus,
          this.droppable,
          this.vflex,
          this.hflex,
          this.renderdefer,
          this.clientAction,
          this.tabindex,
          this.ctrlKeys,
          this.context,
          this.popup,
          this.tooltip,
          this.name,
          this.placeholder,
          this.inputAttributes,
          this.value,
          this.constraint,
          this.errorboxSclass,
          this.errorboxIconSclass,
          this.disabled,
          this.inplace,
          this.maxlength,
          this.instant,
          this.errorMessage,
          this.format,
          this.locale,
          this.timeZone,
          this.valueInZonedDateTime,
          this.valueInLocalDateTime,
          this.valueInLocalDate,
          this.valueInLocalTime,
          this.widgetClass,
          this.min,
          this.max,
          this.cols,
          this.interval,
          this.readonly));
    }
    @Nullable ImmutableList<ActionHandler> newValue = elements == null ? null : ImmutableList.copyOf(elements);
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        newValue,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ITimepicker#getActions() actions}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of actions elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITimepicker withActions(@Nullable Iterable<? extends ActionHandler> elements) {
    if (this.actions == elements) return this;
    @Nullable ImmutableList<ActionHandler> newValue = elements == null ? null : ImmutableList.copyOf(elements);
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        newValue,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#isVisible() visible} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for visible
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withVisible(boolean value) {
    if (this.visible == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        value,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getMold() mold} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for mold
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withMold(String value) {
    String newValue = Objects.requireNonNull(value, "mold");
    if (this.mold.equals(newValue)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        newValue,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getEventListenerMap() eventListenerMap} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for eventListenerMap (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withEventListenerMap(@Nullable EventListenerMap value) {
    if (this.eventListenerMap == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        value,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by replacing the {@link ITimepicker#getWidgetListeners() widgetListeners} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the widgetListeners map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITimepicker withWidgetListeners(@Nullable Map<String, ? extends String> entries) {
    if (this.widgetListeners == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        newValue,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by replacing the {@link ITimepicker#getWidgetOverrides() widgetOverrides} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the widgetOverrides map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITimepicker withWidgetOverrides(@Nullable Map<String, ? extends String> entries) {
    if (this.widgetOverrides == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        newValue,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by replacing the {@link ITimepicker#getClientAttributes() clientAttributes} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the clientAttributes map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITimepicker withClientAttributes(@Nullable Map<String, ? extends String> entries) {
    if (this.clientAttributes == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        newValue,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getLeft() left} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for left (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withLeft(@Nullable String value) {
    if (Objects.equals(this.left, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        value,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getTop() top} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for top (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withTop(@Nullable String value) {
    if (Objects.equals(this.top, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        value,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getZIndex() zIndex} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for zIndex
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withZIndex(int value) {
    if (this.zIndex == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        value,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getHeight() height} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for height (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withHeight(@Nullable String value) {
    if (Objects.equals(this.height, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        value,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getWidth() width} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for width (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withWidth(@Nullable String value) {
    if (Objects.equals(this.width, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        value,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getTooltiptext() tooltiptext} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tooltiptext (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withTooltiptext(@Nullable String value) {
    if (Objects.equals(this.tooltiptext, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        value,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getZclass() zclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for zclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withZclass(@Nullable String value) {
    if (Objects.equals(this.zclass, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        value,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getSclass() sclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withSclass(@Nullable String value) {
    if (Objects.equals(this.sclass, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        value,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getStyle() style} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for style (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withStyle(@Nullable String value) {
    if (Objects.equals(this.style, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        value,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getDraggable() draggable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for draggable
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withDraggable(String value) {
    String newValue = Objects.requireNonNull(value, "draggable");
    if (this.draggable.equals(newValue)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        newValue,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#isFocus() focus} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for focus
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withFocus(boolean value) {
    if (this.focus == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        value,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getDroppable() droppable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for droppable
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withDroppable(String value) {
    String newValue = Objects.requireNonNull(value, "droppable");
    if (this.droppable.equals(newValue)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        newValue,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getVflex() vflex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for vflex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withVflex(@Nullable String value) {
    if (Objects.equals(this.vflex, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        value,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getHflex() hflex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for hflex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withHflex(@Nullable String value) {
    if (Objects.equals(this.hflex, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        value,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getRenderdefer() renderdefer} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for renderdefer
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withRenderdefer(int value) {
    if (this.renderdefer == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        value,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getClientAction() clientAction} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for clientAction (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withClientAction(@Nullable String value) {
    if (Objects.equals(this.clientAction, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        value,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getTabindex() tabindex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tabindex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withTabindex(@Nullable Integer value) {
    if (Objects.equals(this.tabindex, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        value,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getCtrlKeys() ctrlKeys} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for ctrlKeys (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withCtrlKeys(@Nullable String value) {
    if (Objects.equals(this.ctrlKeys, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        value,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getContext() context} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for context (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withContext(@Nullable String value) {
    if (Objects.equals(this.context, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        value,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getPopup() popup} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for popup (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withPopup(@Nullable String value) {
    if (Objects.equals(this.popup, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        value,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getTooltip() tooltip} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tooltip (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withTooltip(@Nullable String value) {
    if (Objects.equals(this.tooltip, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        value,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withName(@Nullable String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        value,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getPlaceholder() placeholder} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for placeholder (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withPlaceholder(@Nullable String value) {
    if (Objects.equals(this.placeholder, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        value,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by replacing the {@link ITimepicker#getInputAttributes() inputAttributes} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the inputAttributes map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITimepicker withInputAttributes(@Nullable Map<String, ? extends String> entries) {
    if (this.inputAttributes == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        newValue,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getValue() value} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for value (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withValue(@Nullable Date value) {
    if (this.value == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getConstraint() constraint} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for constraint (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withConstraint(@Nullable String value) {
    if (Objects.equals(this.constraint, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        value,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getErrorboxSclass() errorboxSclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for errorboxSclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withErrorboxSclass(@Nullable String value) {
    if (Objects.equals(this.errorboxSclass, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        value,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getErrorboxIconSclass() errorboxIconSclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for errorboxIconSclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withErrorboxIconSclass(@Nullable String value) {
    if (Objects.equals(this.errorboxIconSclass, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        value,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#isDisabled() disabled} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for disabled
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withDisabled(boolean value) {
    if (this.disabled == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        value,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#isInplace() inplace} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for inplace
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withInplace(boolean value) {
    if (this.inplace == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        value,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getMaxlength() maxlength} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for maxlength
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withMaxlength(int value) {
    if (this.maxlength == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        value,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getInstant() instant} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for instant
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withInstant(boolean value) {
    if (this.instant == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        value,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getErrorMessage() errorMessage} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for errorMessage (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withErrorMessage(@Nullable String value) {
    if (Objects.equals(this.errorMessage, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        value,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getFormat() format} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for format (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withFormat(@Nullable String value) {
    if (Objects.equals(this.format, value)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        value,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getLocale() locale} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for locale (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withLocale(@Nullable Locale value) {
    if (this.locale == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        value,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getTimeZone() timeZone} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for timeZone
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withTimeZone(TimeZone value) {
    if (this.timeZone == value) return this;
    TimeZone newValue = Objects.requireNonNull(value, "timeZone");
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        newValue,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getValueInZonedDateTime() valueInZonedDateTime} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for valueInZonedDateTime (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withValueInZonedDateTime(@Nullable ZonedDateTime value) {
    if (this.valueInZonedDateTime == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        value,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getValueInLocalDateTime() valueInLocalDateTime} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for valueInLocalDateTime (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withValueInLocalDateTime(@Nullable LocalDateTime value) {
    if (this.valueInLocalDateTime == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        value,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getValueInLocalDate() valueInLocalDate} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for valueInLocalDate (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withValueInLocalDate(@Nullable LocalDate value) {
    if (this.valueInLocalDate == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        value,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getValueInLocalTime() valueInLocalTime} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for valueInLocalTime (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withValueInLocalTime(@Nullable LocalTime value) {
    if (this.valueInLocalTime == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        value,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getWidgetClass() widgetClass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for widgetClass
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withWidgetClass(String value) {
    String newValue = Objects.requireNonNull(value, "widgetClass");
    if (this.widgetClass.equals(newValue)) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        newValue,
        this.min,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getMin() min} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for min (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withMin(@Nullable Date value) {
    if (this.min == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        value,
        this.max,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getMax() max} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for max (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withMax(@Nullable Date value) {
    if (this.max == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        value,
        this.cols,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getCols() cols} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cols
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withCols(int value) {
    if (this.cols == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        value,
        this.interval,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#getInterval() interval} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for interval
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withInterval(int value) {
    if (this.interval == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        value,
        this.readonly));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITimepicker#isReadonly() readonly} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for readonly
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITimepicker withReadonly(boolean value) {
    if (this.readonly == value) return this;
    return validate(new ImmutableITimepicker(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.name,
        this.placeholder,
        this.inputAttributes,
        this.value,
        this.constraint,
        this.errorboxSclass,
        this.errorboxIconSclass,
        this.disabled,
        this.inplace,
        this.maxlength,
        this.instant,
        this.errorMessage,
        this.format,
        this.locale,
        this.timeZone,
        this.valueInZonedDateTime,
        this.valueInLocalDateTime,
        this.valueInLocalDate,
        this.valueInLocalTime,
        this.widgetClass,
        this.min,
        this.max,
        this.cols,
        this.interval,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableITimepicker} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableITimepicker
        && equalTo((ImmutableITimepicker) another);
  }

  private boolean equalTo(ImmutableITimepicker another) {
    return id.equals(another.id)
        && Objects.equals(action, another.action)
        && Objects.equals(actions, another.actions)
        && visible == another.visible
        && mold.equals(another.mold)
        && Objects.equals(eventListenerMap, another.eventListenerMap)
        && Objects.equals(widgetListeners, another.widgetListeners)
        && Objects.equals(widgetOverrides, another.widgetOverrides)
        && Objects.equals(clientAttributes, another.clientAttributes)
        && Objects.equals(left, another.left)
        && Objects.equals(top, another.top)
        && zIndex == another.zIndex
        && Objects.equals(height, another.height)
        && Objects.equals(width, another.width)
        && Objects.equals(tooltiptext, another.tooltiptext)
        && Objects.equals(zclass, another.zclass)
        && Objects.equals(sclass, another.sclass)
        && Objects.equals(style, another.style)
        && draggable.equals(another.draggable)
        && focus == another.focus
        && droppable.equals(another.droppable)
        && Objects.equals(vflex, another.vflex)
        && Objects.equals(hflex, another.hflex)
        && renderdefer == another.renderdefer
        && Objects.equals(clientAction, another.clientAction)
        && Objects.equals(tabindex, another.tabindex)
        && Objects.equals(ctrlKeys, another.ctrlKeys)
        && Objects.equals(context, another.context)
        && Objects.equals(popup, another.popup)
        && Objects.equals(tooltip, another.tooltip)
        && Objects.equals(name, another.name)
        && Objects.equals(placeholder, another.placeholder)
        && Objects.equals(inputAttributes, another.inputAttributes)
        && Objects.equals(value, another.value)
        && Objects.equals(constraint, another.constraint)
        && Objects.equals(errorboxSclass, another.errorboxSclass)
        && Objects.equals(errorboxIconSclass, another.errorboxIconSclass)
        && disabled == another.disabled
        && inplace == another.inplace
        && maxlength == another.maxlength
        && instant == another.instant
        && Objects.equals(errorMessage, another.errorMessage)
        && Objects.equals(format, another.format)
        && Objects.equals(locale, another.locale)
        && timeZone.equals(another.timeZone)
        && Objects.equals(valueInZonedDateTime, another.valueInZonedDateTime)
        && Objects.equals(valueInLocalDateTime, another.valueInLocalDateTime)
        && Objects.equals(valueInLocalDate, another.valueInLocalDate)
        && Objects.equals(valueInLocalTime, another.valueInLocalTime)
        && widgetClass.equals(another.widgetClass)
        && Objects.equals(min, another.min)
        && Objects.equals(max, another.max)
        && cols == another.cols
        && interval == another.interval
        && readonly == another.readonly;
  }

  /**
   * Computes a hash code from attributes: {@code id}, {@code action}, {@code actions}, {@code visible}, {@code mold}, {@code eventListenerMap}, {@code widgetListeners}, {@code widgetOverrides}, {@code clientAttributes}, {@code left}, {@code top}, {@code zIndex}, {@code height}, {@code width}, {@code tooltiptext}, {@code zclass}, {@code sclass}, {@code style}, {@code draggable}, {@code focus}, {@code droppable}, {@code vflex}, {@code hflex}, {@code renderdefer}, {@code clientAction}, {@code tabindex}, {@code ctrlKeys}, {@code context}, {@code popup}, {@code tooltip}, {@code name}, {@code placeholder}, {@code inputAttributes}, {@code value}, {@code constraint}, {@code errorboxSclass}, {@code errorboxIconSclass}, {@code disabled}, {@code inplace}, {@code maxlength}, {@code instant}, {@code errorMessage}, {@code format}, {@code locale}, {@code timeZone}, {@code valueInZonedDateTime}, {@code valueInLocalDateTime}, {@code valueInLocalDate}, {@code valueInLocalTime}, {@code widgetClass}, {@code min}, {@code max}, {@code cols}, {@code interval}, {@code readonly}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + id.hashCode();
    h += (h << 5) + Objects.hashCode(action);
    h += (h << 5) + Objects.hashCode(actions);
    h += (h << 5) + Booleans.hashCode(visible);
    h += (h << 5) + mold.hashCode();
    h += (h << 5) + Objects.hashCode(eventListenerMap);
    h += (h << 5) + Objects.hashCode(widgetListeners);
    h += (h << 5) + Objects.hashCode(widgetOverrides);
    h += (h << 5) + Objects.hashCode(clientAttributes);
    h += (h << 5) + Objects.hashCode(left);
    h += (h << 5) + Objects.hashCode(top);
    h += (h << 5) + zIndex;
    h += (h << 5) + Objects.hashCode(height);
    h += (h << 5) + Objects.hashCode(width);
    h += (h << 5) + Objects.hashCode(tooltiptext);
    h += (h << 5) + Objects.hashCode(zclass);
    h += (h << 5) + Objects.hashCode(sclass);
    h += (h << 5) + Objects.hashCode(style);
    h += (h << 5) + draggable.hashCode();
    h += (h << 5) + Booleans.hashCode(focus);
    h += (h << 5) + droppable.hashCode();
    h += (h << 5) + Objects.hashCode(vflex);
    h += (h << 5) + Objects.hashCode(hflex);
    h += (h << 5) + renderdefer;
    h += (h << 5) + Objects.hashCode(clientAction);
    h += (h << 5) + Objects.hashCode(tabindex);
    h += (h << 5) + Objects.hashCode(ctrlKeys);
    h += (h << 5) + Objects.hashCode(context);
    h += (h << 5) + Objects.hashCode(popup);
    h += (h << 5) + Objects.hashCode(tooltip);
    h += (h << 5) + Objects.hashCode(name);
    h += (h << 5) + Objects.hashCode(placeholder);
    h += (h << 5) + Objects.hashCode(inputAttributes);
    h += (h << 5) + Objects.hashCode(value);
    h += (h << 5) + Objects.hashCode(constraint);
    h += (h << 5) + Objects.hashCode(errorboxSclass);
    h += (h << 5) + Objects.hashCode(errorboxIconSclass);
    h += (h << 5) + Booleans.hashCode(disabled);
    h += (h << 5) + Booleans.hashCode(inplace);
    h += (h << 5) + maxlength;
    h += (h << 5) + Booleans.hashCode(instant);
    h += (h << 5) + Objects.hashCode(errorMessage);
    h += (h << 5) + Objects.hashCode(format);
    h += (h << 5) + Objects.hashCode(locale);
    h += (h << 5) + timeZone.hashCode();
    h += (h << 5) + Objects.hashCode(valueInZonedDateTime);
    h += (h << 5) + Objects.hashCode(valueInLocalDateTime);
    h += (h << 5) + Objects.hashCode(valueInLocalDate);
    h += (h << 5) + Objects.hashCode(valueInLocalTime);
    h += (h << 5) + widgetClass.hashCode();
    h += (h << 5) + Objects.hashCode(min);
    h += (h << 5) + Objects.hashCode(max);
    h += (h << 5) + cols;
    h += (h << 5) + interval;
    h += (h << 5) + Booleans.hashCode(readonly);
    return h;
  }

  /**
   * Prints the immutable value {@code ITimepicker} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("ITimepicker")
        .omitNullValues()
        .add("id", id)
        .add("action", action)
        .add("actions", actions)
        .add("visible", visible)
        .add("mold", mold)
        .add("eventListenerMap", eventListenerMap)
        .add("widgetListeners", widgetListeners)
        .add("widgetOverrides", widgetOverrides)
        .add("clientAttributes", clientAttributes)
        .add("left", left)
        .add("top", top)
        .add("zIndex", zIndex)
        .add("height", height)
        .add("width", width)
        .add("tooltiptext", tooltiptext)
        .add("zclass", zclass)
        .add("sclass", sclass)
        .add("style", style)
        .add("draggable", draggable)
        .add("focus", focus)
        .add("droppable", droppable)
        .add("vflex", vflex)
        .add("hflex", hflex)
        .add("renderdefer", renderdefer)
        .add("clientAction", clientAction)
        .add("tabindex", tabindex)
        .add("ctrlKeys", ctrlKeys)
        .add("context", context)
        .add("popup", popup)
        .add("tooltip", tooltip)
        .add("name", name)
        .add("placeholder", placeholder)
        .add("inputAttributes", inputAttributes)
        .add("value", value)
        .add("constraint", constraint)
        .add("errorboxSclass", errorboxSclass)
        .add("errorboxIconSclass", errorboxIconSclass)
        .add("disabled", disabled)
        .add("inplace", inplace)
        .add("maxlength", maxlength)
        .add("instant", instant)
        .add("errorMessage", errorMessage)
        .add("format", format)
        .add("locale", locale)
        .add("timeZone", timeZone)
        .add("valueInZonedDateTime", valueInZonedDateTime)
        .add("valueInLocalDateTime", valueInLocalDateTime)
        .add("valueInLocalDate", valueInLocalDate)
        .add("valueInLocalTime", valueInLocalTime)
        .add("widgetClass", widgetClass)
        .add("min", min)
        .add("max", max)
        .add("cols", cols)
        .add("interval", interval)
        .add("readonly", readonly)
        .toString();
  }

  private transient volatile long lazyInitBitmap;

  private static final long ZONE_ID_LAZY_INIT_BIT = 0x1L;

  private transient ZoneId zoneId;

  /**
   * {@inheritDoc}
   * <p>
   * Returns a lazily initialized value of the {@link ITimepicker#getZoneId() zoneId} attribute.
   * Initialized once and only once and stored for subsequent access with proper synchronization.
   * In case of any exception or error thrown by the lazy value initializer,
   * the result will not be memoised (i.e. remembered) and on next call computation
   * will be attempted again.
   * @return A lazily initialized value of the {@code zoneId} attribute
   */
  @Override
  public ZoneId getZoneId() {
    if ((lazyInitBitmap & ZONE_ID_LAZY_INIT_BIT) == 0) {
      synchronized (this) {
        if ((lazyInitBitmap & ZONE_ID_LAZY_INIT_BIT) == 0) {
          this.zoneId = ITimepicker.super.getZoneId();
          lazyInitBitmap |= ZONE_ID_LAZY_INIT_BIT;
        }
      }
    }
    return zoneId;
  }

  private static final long Z_K_TYPE_LAZY_INIT_BIT = 0x2L;

  private transient Class<Timepicker> zKType;

  /**
   * {@inheritDoc}
   * <p>
   * Returns a lazily initialized value of the {@link ITimepicker#getZKType() zKType} attribute.
   * Initialized once and only once and stored for subsequent access with proper synchronization.
   * In case of any exception or error thrown by the lazy value initializer,
   * the result will not be memoised (i.e. remembered) and on next call computation
   * will be attempted again.
   * @return A lazily initialized value of the {@code zKType} attribute
   */
  @Override
  public Class<Timepicker> getZKType() {
    if ((lazyInitBitmap & Z_K_TYPE_LAZY_INIT_BIT) == 0) {
      synchronized (this) {
        if ((lazyInitBitmap & Z_K_TYPE_LAZY_INIT_BIT) == 0) {
          this.zKType = Objects.requireNonNull(ITimepicker.super.getZKType(), "zKType");
          lazyInitBitmap |= Z_K_TYPE_LAZY_INIT_BIT;
        }
      }
    }
    return zKType;
  }

  private static final long REAL_FORMAT_LAZY_INIT_BIT = 0x4L;

  private transient String realFormat;

  /**
   * {@inheritDoc}
   * <p>
   * Returns a lazily initialized value of the {@link ITimepicker#getRealFormat() realFormat} attribute.
   * Initialized once and only once and stored for subsequent access with proper synchronization.
   * In case of any exception or error thrown by the lazy value initializer,
   * the result will not be memoised (i.e. remembered) and on next call computation
   * will be attempted again.
   * @return A lazily initialized value of the {@code realFormat} attribute
   */
  @Override
  public String getRealFormat() {
    if ((lazyInitBitmap & REAL_FORMAT_LAZY_INIT_BIT) == 0) {
      synchronized (this) {
        if ((lazyInitBitmap & REAL_FORMAT_LAZY_INIT_BIT) == 0) {
          this.realFormat = Objects.requireNonNull(ITimepicker.super.getRealFormat(), "realFormat");
          lazyInitBitmap |= REAL_FORMAT_LAZY_INIT_BIT;
        }
      }
    }
    return realFormat;
  }

  private static ImmutableITimepicker validate(ImmutableITimepicker instance) {
    instance = (ImmutableITimepicker) instance.updateValue();
    instance.checkInterval();
    instance.checkCols();
    instance.checkHflexAndWidth();
    instance.checkVflexAndHeight();
    instance = (ImmutableITimepicker) instance.checkDroppable();
    instance = (ImmutableITimepicker) instance.checkDraggable();
    instance.checkActions();
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link ITimepicker} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable ITimepicker instance
   */
  public static ITimepicker copyOf(ITimepicker instance) {
    if (instance instanceof ImmutableITimepicker) {
      return (ImmutableITimepicker) instance;
    }
    return new ITimepicker.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ITimepicker ITimepicker}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "ITimepicker", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private static final long OPT_BIT_VISIBLE = 0x1L;
    private static final long OPT_BIT_Z_INDEX = 0x2L;
    private static final long OPT_BIT_FOCUS = 0x4L;
    private static final long OPT_BIT_RENDERDEFER = 0x8L;
    private static final long OPT_BIT_DISABLED = 0x10L;
    private static final long OPT_BIT_INPLACE = 0x20L;
    private static final long OPT_BIT_MAXLENGTH = 0x40L;
    private static final long OPT_BIT_INSTANT = 0x80L;
    private static final long OPT_BIT_VALUE_IN_ZONED_DATE_TIME = 0x100L;
    private static final long OPT_BIT_VALUE_IN_LOCAL_DATE_TIME = 0x200L;
    private static final long OPT_BIT_VALUE_IN_LOCAL_DATE = 0x400L;
    private static final long OPT_BIT_VALUE_IN_LOCAL_TIME = 0x800L;
    private static final long OPT_BIT_COLS = 0x1000L;
    private static final long OPT_BIT_INTERVAL = 0x2000L;
    private static final long OPT_BIT_READONLY = 0x4000L;
    private long optBits;

    private @Nullable String id;
    private @Nullable ActionHandler action;
    private ImmutableList.Builder<ActionHandler> actions = null;
    private boolean visible;
    private @Nullable String mold;
    private @Nullable EventListenerMap eventListenerMap;
    private ImmutableMap.Builder<String, String> widgetListeners = null;
    private ImmutableMap.Builder<String, String> widgetOverrides = null;
    private ImmutableMap.Builder<String, String> clientAttributes = null;
    private @Nullable String left;
    private @Nullable String top;
    private int zIndex;
    private @Nullable String height;
    private @Nullable String width;
    private @Nullable String tooltiptext;
    private @Nullable String zclass;
    private @Nullable String sclass;
    private @Nullable String style;
    private @Nullable String draggable;
    private boolean focus;
    private @Nullable String droppable;
    private @Nullable String vflex;
    private @Nullable String hflex;
    private int renderdefer;
    private @Nullable String clientAction;
    private @Nullable Integer tabindex;
    private @Nullable String ctrlKeys;
    private @Nullable String context;
    private @Nullable String popup;
    private @Nullable String tooltip;
    private @Nullable String name;
    private @Nullable String placeholder;
    private ImmutableMap.Builder<String, String> inputAttributes = null;
    private @Nullable Date value;
    private @Nullable String constraint;
    private @Nullable String errorboxSclass;
    private @Nullable String errorboxIconSclass;
    private boolean disabled;
    private boolean inplace;
    private int maxlength;
    private boolean instant;
    private @Nullable String errorMessage;
    private @Nullable String format;
    private @Nullable Locale locale;
    private @Nullable TimeZone timeZone;
    private @Nullable ZonedDateTime valueInZonedDateTime;
    private @Nullable LocalDateTime valueInLocalDateTime;
    private @Nullable LocalDate valueInLocalDate;
    private @Nullable LocalTime valueInLocalTime;
    private @Nullable String widgetClass;
    private @Nullable Date min;
    private @Nullable Date max;
    private int cols;
    private int interval;
    private boolean readonly;

    /**
     * Creates a builder for {@link ITimepicker ITimepicker} instances.
     * <pre>
     * new ITimepicker.Builder()
     *    .setId(String) // optional {@link ITimepicker#getId() id}
     *    .setAction(org.zkoss.stateless.util.ActionHandler | null) // nullable {@link ITimepicker#getAction() action}
     *    .setActions(List&amp;lt;org.zkoss.stateless.util.ActionHandler&amp;gt; | null) // nullable {@link ITimepicker#getActions() actions}
     *    .setVisible(boolean) // optional {@link ITimepicker#isVisible() visible}
     *    .setMold(String) // optional {@link ITimepicker#getMold() mold}
     *    .setEventListenerMap(org.zkoss.zk.ui.sys.EventListenerMap | null) // nullable {@link ITimepicker#getEventListenerMap() eventListenerMap}
     *    .setWidgetListeners(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link ITimepicker#getWidgetListeners() widgetListeners}
     *    .setWidgetOverrides(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link ITimepicker#getWidgetOverrides() widgetOverrides}
     *    .setClientAttributes(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link ITimepicker#getClientAttributes() clientAttributes}
     *    .setLeft(String | null) // nullable {@link ITimepicker#getLeft() left}
     *    .setTop(String | null) // nullable {@link ITimepicker#getTop() top}
     *    .setZIndex(int) // optional {@link ITimepicker#getZIndex() zIndex}
     *    .setHeight(String | null) // nullable {@link ITimepicker#getHeight() height}
     *    .setWidth(String | null) // nullable {@link ITimepicker#getWidth() width}
     *    .setTooltiptext(String | null) // nullable {@link ITimepicker#getTooltiptext() tooltiptext}
     *    .setZclass(String | null) // nullable {@link ITimepicker#getZclass() zclass}
     *    .setSclass(String | null) // nullable {@link ITimepicker#getSclass() sclass}
     *    .setStyle(String | null) // nullable {@link ITimepicker#getStyle() style}
     *    .setDraggable(String) // optional {@link ITimepicker#getDraggable() draggable}
     *    .setFocus(boolean) // optional {@link ITimepicker#isFocus() focus}
     *    .setDroppable(String) // optional {@link ITimepicker#getDroppable() droppable}
     *    .setVflex(String | null) // nullable {@link ITimepicker#getVflex() vflex}
     *    .setHflex(String | null) // nullable {@link ITimepicker#getHflex() hflex}
     *    .setRenderdefer(int) // optional {@link ITimepicker#getRenderdefer() renderdefer}
     *    .setClientAction(String | null) // nullable {@link ITimepicker#getClientAction() clientAction}
     *    .setTabindex(Integer | null) // nullable {@link ITimepicker#getTabindex() tabindex}
     *    .setCtrlKeys(String | null) // nullable {@link ITimepicker#getCtrlKeys() ctrlKeys}
     *    .setContext(String | null) // nullable {@link ITimepicker#getContext() context}
     *    .setPopup(String | null) // nullable {@link ITimepicker#getPopup() popup}
     *    .setTooltip(String | null) // nullable {@link ITimepicker#getTooltip() tooltip}
     *    .setName(String | null) // nullable {@link ITimepicker#getName() name}
     *    .setPlaceholder(String | null) // nullable {@link ITimepicker#getPlaceholder() placeholder}
     *    .setInputAttributes(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link ITimepicker#getInputAttributes() inputAttributes}
     *    .setValue(Date | null) // nullable {@link ITimepicker#getValue() value}
     *    .setConstraint(String | null) // nullable {@link ITimepicker#getConstraint() constraint}
     *    .setErrorboxSclass(String | null) // nullable {@link ITimepicker#getErrorboxSclass() errorboxSclass}
     *    .setErrorboxIconSclass(String | null) // nullable {@link ITimepicker#getErrorboxIconSclass() errorboxIconSclass}
     *    .setDisabled(boolean) // optional {@link ITimepicker#isDisabled() disabled}
     *    .setInplace(boolean) // optional {@link ITimepicker#isInplace() inplace}
     *    .setMaxlength(int) // optional {@link ITimepicker#getMaxlength() maxlength}
     *    .setInstant(boolean) // optional {@link ITimepicker#getInstant() instant}
     *    .setErrorMessage(String | null) // nullable {@link ITimepicker#getErrorMessage() errorMessage}
     *    .setFormat(String | null) // nullable {@link ITimepicker#getFormat() format}
     *    .setLocale(Locale | null) // nullable {@link ITimepicker#getLocale() locale}
     *    .setTimeZone(TimeZone) // optional {@link ITimepicker#getTimeZone() timeZone}
     *    .setValueInZonedDateTime(java.time.ZonedDateTime | null) // nullable {@link ITimepicker#getValueInZonedDateTime() valueInZonedDateTime}
     *    .setValueInLocalDateTime(java.time.LocalDateTime | null) // nullable {@link ITimepicker#getValueInLocalDateTime() valueInLocalDateTime}
     *    .setValueInLocalDate(java.time.LocalDate | null) // nullable {@link ITimepicker#getValueInLocalDate() valueInLocalDate}
     *    .setValueInLocalTime(java.time.LocalTime | null) // nullable {@link ITimepicker#getValueInLocalTime() valueInLocalTime}
     *    .setWidgetClass(String) // optional {@link ITimepicker#getWidgetClass() widgetClass}
     *    .setMin(Date | null) // nullable {@link ITimepicker#getMin() min}
     *    .setMax(Date | null) // nullable {@link ITimepicker#getMax() max}
     *    .setCols(int) // optional {@link ITimepicker#getCols() cols}
     *    .setInterval(int) // optional {@link ITimepicker#getInterval() interval}
     *    .setReadonly(boolean) // optional {@link ITimepicker#isReadonly() readonly}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof ITimepicker.Builder)) {
        throw new UnsupportedOperationException("Use: new ITimepicker.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code ITimepicker} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder from(ITimepicker instance) {
      Objects.requireNonNull(instance, "instance");
      setId(instance.getId());
      @Nullable ActionHandler actionValue = instance.getAction();
      if (actionValue != null) {
        setAction(actionValue);
      }
      @Nullable List<ActionHandler> actionsValue = instance.getActions();
      if (actionsValue != null) {
        addAllActions(actionsValue);
      }
      setVisible(instance.isVisible());
      setMold(instance.getMold());
      @Nullable EventListenerMap eventListenerMapValue = instance.getEventListenerMap();
      if (eventListenerMapValue != null) {
        setEventListenerMap(eventListenerMapValue);
      }
      @Nullable Map<String, String> widgetListenersValue = instance.getWidgetListeners();
      if (widgetListenersValue != null) {
        putAllWidgetListeners(widgetListenersValue);
      }
      @Nullable Map<String, String> widgetOverridesValue = instance.getWidgetOverrides();
      if (widgetOverridesValue != null) {
        putAllWidgetOverrides(widgetOverridesValue);
      }
      @Nullable Map<String, String> clientAttributesValue = instance.getClientAttributes();
      if (clientAttributesValue != null) {
        putAllClientAttributes(clientAttributesValue);
      }
      @Nullable String leftValue = instance.getLeft();
      if (leftValue != null) {
        setLeft(leftValue);
      }
      @Nullable String topValue = instance.getTop();
      if (topValue != null) {
        setTop(topValue);
      }
      setZIndex(instance.getZIndex());
      @Nullable String heightValue = instance.getHeight();
      if (heightValue != null) {
        setHeight(heightValue);
      }
      @Nullable String widthValue = instance.getWidth();
      if (widthValue != null) {
        setWidth(widthValue);
      }
      @Nullable String tooltiptextValue = instance.getTooltiptext();
      if (tooltiptextValue != null) {
        setTooltiptext(tooltiptextValue);
      }
      @Nullable String zclassValue = instance.getZclass();
      if (zclassValue != null) {
        setZclass(zclassValue);
      }
      @Nullable String sclassValue = instance.getSclass();
      if (sclassValue != null) {
        setSclass(sclassValue);
      }
      @Nullable String styleValue = instance.getStyle();
      if (styleValue != null) {
        setStyle(styleValue);
      }
      setDraggable(instance.getDraggable());
      setFocus(instance.isFocus());
      setDroppable(instance.getDroppable());
      @Nullable String vflexValue = instance.getVflex();
      if (vflexValue != null) {
        setVflex(vflexValue);
      }
      @Nullable String hflexValue = instance.getHflex();
      if (hflexValue != null) {
        setHflex(hflexValue);
      }
      setRenderdefer(instance.getRenderdefer());
      @Nullable String clientActionValue = instance.getClientAction();
      if (clientActionValue != null) {
        setClientAction(clientActionValue);
      }
      @Nullable Integer tabindexValue = instance.getTabindex();
      if (tabindexValue != null) {
        setTabindex(tabindexValue);
      }
      @Nullable String ctrlKeysValue = instance.getCtrlKeys();
      if (ctrlKeysValue != null) {
        setCtrlKeys(ctrlKeysValue);
      }
      @Nullable String contextValue = instance.getContext();
      if (contextValue != null) {
        setContext(contextValue);
      }
      @Nullable String popupValue = instance.getPopup();
      if (popupValue != null) {
        setPopup(popupValue);
      }
      @Nullable String tooltipValue = instance.getTooltip();
      if (tooltipValue != null) {
        setTooltip(tooltipValue);
      }
      @Nullable String nameValue = instance.getName();
      if (nameValue != null) {
        setName(nameValue);
      }
      @Nullable String placeholderValue = instance.getPlaceholder();
      if (placeholderValue != null) {
        setPlaceholder(placeholderValue);
      }
      @Nullable Map<String, String> inputAttributesValue = instance.getInputAttributes();
      if (inputAttributesValue != null) {
        putAllInputAttributes(inputAttributesValue);
      }
      @Nullable Date valueValue = instance.getValue();
      if (valueValue != null) {
        setValue(valueValue);
      }
      @Nullable String constraintValue = instance.getConstraint();
      if (constraintValue != null) {
        setConstraint(constraintValue);
      }
      @Nullable String errorboxSclassValue = instance.getErrorboxSclass();
      if (errorboxSclassValue != null) {
        setErrorboxSclass(errorboxSclassValue);
      }
      @Nullable String errorboxIconSclassValue = instance.getErrorboxIconSclass();
      if (errorboxIconSclassValue != null) {
        setErrorboxIconSclass(errorboxIconSclassValue);
      }
      setDisabled(instance.isDisabled());
      setInplace(instance.isInplace());
      setMaxlength(instance.getMaxlength());
      setInstant(instance.getInstant());
      @Nullable String errorMessageValue = instance.getErrorMessage();
      if (errorMessageValue != null) {
        setErrorMessage(errorMessageValue);
      }
      @Nullable String formatValue = instance.getFormat();
      if (formatValue != null) {
        setFormat(formatValue);
      }
      @Nullable Locale localeValue = instance.getLocale();
      if (localeValue != null) {
        setLocale(localeValue);
      }
      setTimeZone(instance.getTimeZone());
      @Nullable ZonedDateTime valueInZonedDateTimeValue = instance.getValueInZonedDateTime();
      if (valueInZonedDateTimeValue != null) {
        setValueInZonedDateTime(valueInZonedDateTimeValue);
      }
      @Nullable LocalDateTime valueInLocalDateTimeValue = instance.getValueInLocalDateTime();
      if (valueInLocalDateTimeValue != null) {
        setValueInLocalDateTime(valueInLocalDateTimeValue);
      }
      @Nullable LocalDate valueInLocalDateValue = instance.getValueInLocalDate();
      if (valueInLocalDateValue != null) {
        setValueInLocalDate(valueInLocalDateValue);
      }
      @Nullable LocalTime valueInLocalTimeValue = instance.getValueInLocalTime();
      if (valueInLocalTimeValue != null) {
        setValueInLocalTime(valueInLocalTimeValue);
      }
      setWidgetClass(instance.getWidgetClass());
      @Nullable Date minValue = instance.getMin();
      if (minValue != null) {
        setMin(minValue);
      }
      @Nullable Date maxValue = instance.getMax();
      if (maxValue != null) {
        setMax(maxValue);
      }
      setCols(instance.getCols());
      setInterval(instance.getInterval());
      setReadonly(instance.isReadonly());
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getId() id} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getId() id}.</em>
     * @param id The value for id 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setId(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getAction() action} attribute.
     * @param action The value for action (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setAction(@Nullable ActionHandler action) {
      this.action = action;
      return (ITimepicker.Builder) this;
    }

    /**
     * Adds one element to {@link ITimepicker#getActions() actions} list.
     * @param element A actions element
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder addActions(ActionHandler element) {
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.add(element);
      return (ITimepicker.Builder) this;
    }

    /**
     * Adds elements to {@link ITimepicker#getActions() actions} list.
     * @param elements An array of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder addActions(ActionHandler... elements) {
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.add(elements);
      return (ITimepicker.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link ITimepicker#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setActions(@Nullable Iterable<? extends ActionHandler> elements) {
      if (elements == null) {
        this.actions = null;
        return (ITimepicker.Builder) this;
      }
      this.actions = ImmutableList.builder();
      return addAllActions(elements);
    }

    /**
     * Adds elements to {@link ITimepicker#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder addAllActions(Iterable<? extends ActionHandler> elements) {
      Objects.requireNonNull(elements, "actions element");
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.addAll(elements);
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#isVisible() visible} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#isVisible() visible}.</em>
     * @param visible The value for visible 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setVisible(boolean visible) {
      this.visible = visible;
      optBits |= OPT_BIT_VISIBLE;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getMold() mold} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getMold() mold}.</em>
     * @param mold The value for mold 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setMold(String mold) {
      this.mold = Objects.requireNonNull(mold, "mold");
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getEventListenerMap() eventListenerMap} attribute.
     * @param eventListenerMap The value for eventListenerMap (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setEventListenerMap(@Nullable EventListenerMap eventListenerMap) {
      this.eventListenerMap = eventListenerMap;
      return (ITimepicker.Builder) this;
    }

    /**
     * Put one entry to the {@link ITimepicker#getWidgetListeners() widgetListeners} map.
     * @param key The key in the widgetListeners map
     * @param value The associated value in the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putWidgetListeners(String key, String value) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.put(key, value);
      return (ITimepicker.Builder) this;
    }

    /**
     * Put one entry to the {@link ITimepicker#getWidgetListeners() widgetListeners} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putWidgetListeners(Map.Entry<String, ? extends String> entry) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.put(entry);
      return (ITimepicker.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ITimepicker#getWidgetListeners() widgetListeners} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setWidgetListeners(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.widgetListeners = null;
        return (ITimepicker.Builder) this;
      }
      this.widgetListeners = ImmutableMap.builder();
      return putAllWidgetListeners(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ITimepicker#getWidgetListeners() widgetListeners} map. Nulls are not permitted
     * @param entries The entries that will be added to the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putAllWidgetListeners(Map<String, ? extends String> entries) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.putAll(entries);
      return (ITimepicker.Builder) this;
    }

    /**
     * Put one entry to the {@link ITimepicker#getWidgetOverrides() widgetOverrides} map.
     * @param key The key in the widgetOverrides map
     * @param value The associated value in the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putWidgetOverrides(String key, String value) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.put(key, value);
      return (ITimepicker.Builder) this;
    }

    /**
     * Put one entry to the {@link ITimepicker#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putWidgetOverrides(Map.Entry<String, ? extends String> entry) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.put(entry);
      return (ITimepicker.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ITimepicker#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setWidgetOverrides(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.widgetOverrides = null;
        return (ITimepicker.Builder) this;
      }
      this.widgetOverrides = ImmutableMap.builder();
      return putAllWidgetOverrides(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ITimepicker#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted
     * @param entries The entries that will be added to the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putAllWidgetOverrides(Map<String, ? extends String> entries) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.putAll(entries);
      return (ITimepicker.Builder) this;
    }

    /**
     * Put one entry to the {@link ITimepicker#getClientAttributes() clientAttributes} map.
     * @param key The key in the clientAttributes map
     * @param value The associated value in the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putClientAttributes(String key, String value) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.put(key, value);
      return (ITimepicker.Builder) this;
    }

    /**
     * Put one entry to the {@link ITimepicker#getClientAttributes() clientAttributes} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putClientAttributes(Map.Entry<String, ? extends String> entry) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.put(entry);
      return (ITimepicker.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ITimepicker#getClientAttributes() clientAttributes} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setClientAttributes(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.clientAttributes = null;
        return (ITimepicker.Builder) this;
      }
      this.clientAttributes = ImmutableMap.builder();
      return putAllClientAttributes(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ITimepicker#getClientAttributes() clientAttributes} map. Nulls are not permitted
     * @param entries The entries that will be added to the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putAllClientAttributes(Map<String, ? extends String> entries) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.putAll(entries);
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getLeft() left} attribute.
     * @param left The value for left (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setLeft(@Nullable String left) {
      this.left = left;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getTop() top} attribute.
     * @param top The value for top (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setTop(@Nullable String top) {
      this.top = top;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getZIndex() zIndex} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getZIndex() zIndex}.</em>
     * @param zIndex The value for zIndex 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setZIndex(int zIndex) {
      this.zIndex = zIndex;
      optBits |= OPT_BIT_Z_INDEX;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getHeight() height} attribute.
     * @param height The value for height (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setHeight(@Nullable String height) {
      this.height = height;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getWidth() width} attribute.
     * @param width The value for width (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setWidth(@Nullable String width) {
      this.width = width;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getTooltiptext() tooltiptext} attribute.
     * @param tooltiptext The value for tooltiptext (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setTooltiptext(@Nullable String tooltiptext) {
      this.tooltiptext = tooltiptext;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getZclass() zclass} attribute.
     * @param zclass The value for zclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setZclass(@Nullable String zclass) {
      this.zclass = zclass;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getSclass() sclass} attribute.
     * @param sclass The value for sclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setSclass(@Nullable String sclass) {
      this.sclass = sclass;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getStyle() style} attribute.
     * @param style The value for style (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setStyle(@Nullable String style) {
      this.style = style;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getDraggable() draggable} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getDraggable() draggable}.</em>
     * @param draggable The value for draggable 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setDraggable(String draggable) {
      this.draggable = Objects.requireNonNull(draggable, "draggable");
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#isFocus() focus} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#isFocus() focus}.</em>
     * @param focus The value for focus 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setFocus(boolean focus) {
      this.focus = focus;
      optBits |= OPT_BIT_FOCUS;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getDroppable() droppable} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getDroppable() droppable}.</em>
     * @param droppable The value for droppable 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setDroppable(String droppable) {
      this.droppable = Objects.requireNonNull(droppable, "droppable");
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getVflex() vflex} attribute.
     * @param vflex The value for vflex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setVflex(@Nullable String vflex) {
      this.vflex = vflex;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getHflex() hflex} attribute.
     * @param hflex The value for hflex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setHflex(@Nullable String hflex) {
      this.hflex = hflex;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getRenderdefer() renderdefer} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getRenderdefer() renderdefer}.</em>
     * @param renderdefer The value for renderdefer 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setRenderdefer(int renderdefer) {
      this.renderdefer = renderdefer;
      optBits |= OPT_BIT_RENDERDEFER;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getClientAction() clientAction} attribute.
     * @param clientAction The value for clientAction (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setClientAction(@Nullable String clientAction) {
      this.clientAction = clientAction;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getTabindex() tabindex} attribute.
     * @param tabindex The value for tabindex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setTabindex(@Nullable Integer tabindex) {
      this.tabindex = tabindex;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getCtrlKeys() ctrlKeys} attribute.
     * @param ctrlKeys The value for ctrlKeys (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setCtrlKeys(@Nullable String ctrlKeys) {
      this.ctrlKeys = ctrlKeys;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getContext() context} attribute.
     * @param context The value for context (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setContext(@Nullable String context) {
      this.context = context;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getPopup() popup} attribute.
     * @param popup The value for popup (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setPopup(@Nullable String popup) {
      this.popup = popup;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getTooltip() tooltip} attribute.
     * @param tooltip The value for tooltip (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setTooltip(@Nullable String tooltip) {
      this.tooltip = tooltip;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setName(@Nullable String name) {
      this.name = name;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getPlaceholder() placeholder} attribute.
     * @param placeholder The value for placeholder (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setPlaceholder(@Nullable String placeholder) {
      this.placeholder = placeholder;
      return (ITimepicker.Builder) this;
    }

    /**
     * Put one entry to the {@link ITimepicker#getInputAttributes() inputAttributes} map.
     * @param key The key in the inputAttributes map
     * @param value The associated value in the inputAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putInputAttributes(String key, String value) {
      if (this.inputAttributes == null) {
        this.inputAttributes = ImmutableMap.builder();
      }
      this.inputAttributes.put(key, value);
      return (ITimepicker.Builder) this;
    }

    /**
     * Put one entry to the {@link ITimepicker#getInputAttributes() inputAttributes} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putInputAttributes(Map.Entry<String, ? extends String> entry) {
      if (this.inputAttributes == null) {
        this.inputAttributes = ImmutableMap.builder();
      }
      this.inputAttributes.put(entry);
      return (ITimepicker.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ITimepicker#getInputAttributes() inputAttributes} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the inputAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setInputAttributes(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.inputAttributes = null;
        return (ITimepicker.Builder) this;
      }
      this.inputAttributes = ImmutableMap.builder();
      return putAllInputAttributes(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ITimepicker#getInputAttributes() inputAttributes} map. Nulls are not permitted
     * @param entries The entries that will be added to the inputAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder putAllInputAttributes(Map<String, ? extends String> entries) {
      if (this.inputAttributes == null) {
        this.inputAttributes = ImmutableMap.builder();
      }
      this.inputAttributes.putAll(entries);
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getValue() value} attribute.
     * @param value The value for value (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setValue(@Nullable Date value) {
      this.value = value;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getConstraint() constraint} attribute.
     * @param constraint The value for constraint (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setConstraint(@Nullable String constraint) {
      this.constraint = constraint;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getErrorboxSclass() errorboxSclass} attribute.
     * @param errorboxSclass The value for errorboxSclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setErrorboxSclass(@Nullable String errorboxSclass) {
      this.errorboxSclass = errorboxSclass;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getErrorboxIconSclass() errorboxIconSclass} attribute.
     * @param errorboxIconSclass The value for errorboxIconSclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setErrorboxIconSclass(@Nullable String errorboxIconSclass) {
      this.errorboxIconSclass = errorboxIconSclass;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#isDisabled() disabled} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#isDisabled() disabled}.</em>
     * @param disabled The value for disabled 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setDisabled(boolean disabled) {
      this.disabled = disabled;
      optBits |= OPT_BIT_DISABLED;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#isInplace() inplace} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#isInplace() inplace}.</em>
     * @param inplace The value for inplace 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setInplace(boolean inplace) {
      this.inplace = inplace;
      optBits |= OPT_BIT_INPLACE;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getMaxlength() maxlength} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getMaxlength() maxlength}.</em>
     * @param maxlength The value for maxlength 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setMaxlength(int maxlength) {
      this.maxlength = maxlength;
      optBits |= OPT_BIT_MAXLENGTH;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getInstant() instant} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getInstant() instant}.</em>
     * @param instant The value for instant 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setInstant(boolean instant) {
      this.instant = instant;
      optBits |= OPT_BIT_INSTANT;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getErrorMessage() errorMessage} attribute.
     * @param errorMessage The value for errorMessage (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setErrorMessage(@Nullable String errorMessage) {
      this.errorMessage = errorMessage;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getFormat() format} attribute.
     * @param format The value for format (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setFormat(@Nullable String format) {
      this.format = format;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getLocale() locale} attribute.
     * @param locale The value for locale (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setLocale(@Nullable Locale locale) {
      this.locale = locale;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getTimeZone() timeZone} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getTimeZone() timeZone}.</em>
     * @param timeZone The value for timeZone 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setTimeZone(TimeZone timeZone) {
      this.timeZone = Objects.requireNonNull(timeZone, "timeZone");
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getValueInZonedDateTime() valueInZonedDateTime} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getValueInZonedDateTime() valueInZonedDateTime}.</em>
     * @param valueInZonedDateTime The value for valueInZonedDateTime (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setValueInZonedDateTime(@Nullable ZonedDateTime valueInZonedDateTime) {
      this.valueInZonedDateTime = valueInZonedDateTime;
      optBits |= OPT_BIT_VALUE_IN_ZONED_DATE_TIME;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getValueInLocalDateTime() valueInLocalDateTime} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getValueInLocalDateTime() valueInLocalDateTime}.</em>
     * @param valueInLocalDateTime The value for valueInLocalDateTime (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setValueInLocalDateTime(@Nullable LocalDateTime valueInLocalDateTime) {
      this.valueInLocalDateTime = valueInLocalDateTime;
      optBits |= OPT_BIT_VALUE_IN_LOCAL_DATE_TIME;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getValueInLocalDate() valueInLocalDate} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getValueInLocalDate() valueInLocalDate}.</em>
     * @param valueInLocalDate The value for valueInLocalDate (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setValueInLocalDate(@Nullable LocalDate valueInLocalDate) {
      this.valueInLocalDate = valueInLocalDate;
      optBits |= OPT_BIT_VALUE_IN_LOCAL_DATE;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getValueInLocalTime() valueInLocalTime} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getValueInLocalTime() valueInLocalTime}.</em>
     * @param valueInLocalTime The value for valueInLocalTime (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setValueInLocalTime(@Nullable LocalTime valueInLocalTime) {
      this.valueInLocalTime = valueInLocalTime;
      optBits |= OPT_BIT_VALUE_IN_LOCAL_TIME;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getWidgetClass() widgetClass} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getWidgetClass() widgetClass}.</em>
     * @param widgetClass The value for widgetClass 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setWidgetClass(String widgetClass) {
      this.widgetClass = Objects.requireNonNull(widgetClass, "widgetClass");
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getMin() min} attribute.
     * @param min The value for min (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setMin(@Nullable Date min) {
      this.min = min;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getMax() max} attribute.
     * @param max The value for max (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setMax(@Nullable Date max) {
      this.max = max;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getCols() cols} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getCols() cols}.</em>
     * @param cols The value for cols 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setCols(int cols) {
      this.cols = cols;
      optBits |= OPT_BIT_COLS;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#getInterval() interval} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#getInterval() interval}.</em>
     * @param interval The value for interval 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setInterval(int interval) {
      this.interval = interval;
      optBits |= OPT_BIT_INTERVAL;
      return (ITimepicker.Builder) this;
    }

    /**
     * Initializes the value for the {@link ITimepicker#isReadonly() readonly} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITimepicker#isReadonly() readonly}.</em>
     * @param readonly The value for readonly 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITimepicker.Builder setReadonly(boolean readonly) {
      this.readonly = readonly;
      optBits |= OPT_BIT_READONLY;
      return (ITimepicker.Builder) this;
    }

    /**
     * Builds a new {@link ITimepicker ITimepicker}.
     * @return An immutable instance of ITimepicker
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ITimepicker build() {
      return ImmutableITimepicker.validate(new ImmutableITimepicker(this));
    }

    private boolean visibleIsSet() {
      return (optBits & OPT_BIT_VISIBLE) != 0;
    }

    private boolean zIndexIsSet() {
      return (optBits & OPT_BIT_Z_INDEX) != 0;
    }

    private boolean focusIsSet() {
      return (optBits & OPT_BIT_FOCUS) != 0;
    }

    private boolean renderdeferIsSet() {
      return (optBits & OPT_BIT_RENDERDEFER) != 0;
    }

    private boolean disabledIsSet() {
      return (optBits & OPT_BIT_DISABLED) != 0;
    }

    private boolean inplaceIsSet() {
      return (optBits & OPT_BIT_INPLACE) != 0;
    }

    private boolean maxlengthIsSet() {
      return (optBits & OPT_BIT_MAXLENGTH) != 0;
    }

    private boolean instantIsSet() {
      return (optBits & OPT_BIT_INSTANT) != 0;
    }

    private boolean valueInZonedDateTimeIsSet() {
      return (optBits & OPT_BIT_VALUE_IN_ZONED_DATE_TIME) != 0;
    }

    private boolean valueInLocalDateTimeIsSet() {
      return (optBits & OPT_BIT_VALUE_IN_LOCAL_DATE_TIME) != 0;
    }

    private boolean valueInLocalDateIsSet() {
      return (optBits & OPT_BIT_VALUE_IN_LOCAL_DATE) != 0;
    }

    private boolean valueInLocalTimeIsSet() {
      return (optBits & OPT_BIT_VALUE_IN_LOCAL_TIME) != 0;
    }

    private boolean colsIsSet() {
      return (optBits & OPT_BIT_COLS) != 0;
    }

    private boolean intervalIsSet() {
      return (optBits & OPT_BIT_INTERVAL) != 0;
    }

    private boolean readonlyIsSet() {
      return (optBits & OPT_BIT_READONLY) != 0;
    }
  }
}
