package org.zkoss.stateless.sul;

import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.primitives.Booleans;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;
import org.zkoss.stateless.immutable.StatelessOnly;
import org.zkoss.stateless.util.ActionHandler;
import org.zkoss.zk.ui.sys.EventListenerMap;
import org.zkoss.zul.Treecol;

/**
 * Immutable implementation of {@link ITreecol}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new ITreecol.Builder()}.
 */
@Generated(from = "ITreecol", generator = "Immutables")
@SuppressWarnings({"all"})
@ParametersAreNonnullByDefault
@javax.annotation.processing.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableITreecol<I extends IAnyGroup>
    implements ITreecol<I> {
  private final String id;
  private final @Nullable ActionHandler action;
  private final @Nullable ImmutableList<ActionHandler> actions;
  private final boolean visible;
  private final String mold;
  private final @Nullable EventListenerMap eventListenerMap;
  private final @Nullable ImmutableMap<String, String> widgetListeners;
  private final @Nullable ImmutableMap<String, String> widgetOverrides;
  private final @Nullable ImmutableMap<String, String> clientAttributes;
  private final @Nullable String left;
  private final @Nullable String top;
  private final int zIndex;
  private final @Nullable String height;
  private final @Nullable String width;
  private final @Nullable String tooltiptext;
  private final @Nullable String zclass;
  private final @Nullable String sclass;
  private final @Nullable String style;
  private final String draggable;
  private final boolean focus;
  private final String droppable;
  private final @Nullable String vflex;
  private final @Nullable String hflex;
  private final int renderdefer;
  private final @Nullable String clientAction;
  private final @Nullable Integer tabindex;
  private final @Nullable String ctrlKeys;
  private final @Nullable String context;
  private final @Nullable String popup;
  private final @Nullable String tooltip;
  private final String label;
  private final @Nullable String iconSclass;
  private final @Nullable String[] iconSclasses;
  private final @Nullable String iconTooltip;
  private final @Nullable String[] iconTooltips;
  private final @Nullable String image;
  private final @Nullable String hoverImage;
  private final boolean preloadImage;
  private final @Nullable String valign;
  private final @Nullable String align;
  private final ImmutableList<I> children;
  private final String widgetClass;
  private final String sortDirection;
  private final int maxlength;
  private final String sortAscendingString;
  private final String sortDescendingString;
  private final @Nullable Comparator<?> sortAscending;
  private final @Nullable Comparator<?> sortDescending;

  private ImmutableITreecol(ImmutableITreecol.Builder<I> builder) {
    this.action = builder.action;
    this.actions = builder.actions == null ? null : builder.actions.build();
    this.eventListenerMap = builder.eventListenerMap;
    this.widgetListeners = builder.widgetListeners == null ? null : builder.widgetListeners.build();
    this.widgetOverrides = builder.widgetOverrides == null ? null : builder.widgetOverrides.build();
    this.clientAttributes = builder.clientAttributes == null ? null : builder.clientAttributes.build();
    this.left = builder.left;
    this.top = builder.top;
    this.height = builder.height;
    this.width = builder.width;
    this.tooltiptext = builder.tooltiptext;
    this.zclass = builder.zclass;
    this.sclass = builder.sclass;
    this.style = builder.style;
    this.vflex = builder.vflex;
    this.hflex = builder.hflex;
    this.clientAction = builder.clientAction;
    this.tabindex = builder.tabindex;
    this.ctrlKeys = builder.ctrlKeys;
    this.context = builder.context;
    this.popup = builder.popup;
    this.tooltip = builder.tooltip;
    this.valign = builder.valign;
    this.align = builder.align;
    this.children = builder.children.build();
    this.sortAscending = builder.sortAscending;
    this.sortDescending = builder.sortDescending;
    if (builder.id != null) {
      initShim.setId(builder.id);
    }
    if (builder.visibleIsSet()) {
      initShim.setVisible(builder.visible);
    }
    if (builder.mold != null) {
      initShim.setMold(builder.mold);
    }
    if (builder.zIndexIsSet()) {
      initShim.setZIndex(builder.zIndex);
    }
    if (builder.draggable != null) {
      initShim.setDraggable(builder.draggable);
    }
    if (builder.focusIsSet()) {
      initShim.setFocus(builder.focus);
    }
    if (builder.droppable != null) {
      initShim.setDroppable(builder.droppable);
    }
    if (builder.renderdeferIsSet()) {
      initShim.setRenderdefer(builder.renderdefer);
    }
    if (builder.label != null) {
      initShim.setLabel(builder.label);
    }
    if (builder.iconSclassIsSet()) {
      initShim.setIconSclass(builder.iconSclass);
    }
    if (builder.iconSclassesIsSet()) {
      initShim.setIconSclasses(builder.iconSclasses);
    }
    if (builder.iconTooltipIsSet()) {
      initShim.setIconTooltip(builder.iconTooltip);
    }
    if (builder.iconTooltipsIsSet()) {
      initShim.setIconTooltips(builder.iconTooltips);
    }
    if (builder.imageIsSet()) {
      initShim.setImage(builder.image);
    }
    if (builder.hoverImageIsSet()) {
      initShim.setHoverImage(builder.hoverImage);
    }
    if (builder.preloadImageIsSet()) {
      initShim.setPreloadImage(builder.preloadImage);
    }
    if (builder.widgetClass != null) {
      initShim.setWidgetClass(builder.widgetClass);
    }
    if (builder.sortDirection != null) {
      initShim.setSortDirection(builder.sortDirection);
    }
    if (builder.maxlengthIsSet()) {
      initShim.setMaxlength(builder.maxlength);
    }
    if (builder.sortAscendingString != null) {
      initShim.setSortAscendingString(builder.sortAscendingString);
    }
    if (builder.sortDescendingString != null) {
      initShim.setSortDescendingString(builder.sortDescendingString);
    }
    this.id = initShim.getId();
    this.visible = initShim.isVisible();
    this.mold = initShim.getMold();
    this.zIndex = initShim.getZIndex();
    this.draggable = initShim.getDraggable();
    this.focus = initShim.isFocus();
    this.droppable = initShim.getDroppable();
    this.renderdefer = initShim.getRenderdefer();
    this.label = initShim.getLabel();
    this.iconSclass = initShim.getIconSclass();
    this.iconSclasses = initShim.getIconSclasses();
    this.iconTooltip = initShim.getIconTooltip();
    this.iconTooltips = initShim.getIconTooltips();
    this.image = initShim.getImage();
    this.hoverImage = initShim.getHoverImage();
    this.preloadImage = initShim.isPreloadImage();
    this.widgetClass = initShim.getWidgetClass();
    this.sortDirection = initShim.getSortDirection();
    this.maxlength = initShim.getMaxlength();
    this.sortAscendingString = initShim.getSortAscendingString();
    this.sortDescendingString = initShim.getSortDescendingString();
    this.initShim = null;
  }

  private ImmutableITreecol(
      String id,
      @Nullable ActionHandler action,
      @Nullable ImmutableList<ActionHandler> actions,
      boolean visible,
      String mold,
      @Nullable EventListenerMap eventListenerMap,
      @Nullable ImmutableMap<String, String> widgetListeners,
      @Nullable ImmutableMap<String, String> widgetOverrides,
      @Nullable ImmutableMap<String, String> clientAttributes,
      @Nullable String left,
      @Nullable String top,
      int zIndex,
      @Nullable String height,
      @Nullable String width,
      @Nullable String tooltiptext,
      @Nullable String zclass,
      @Nullable String sclass,
      @Nullable String style,
      String draggable,
      boolean focus,
      String droppable,
      @Nullable String vflex,
      @Nullable String hflex,
      int renderdefer,
      @Nullable String clientAction,
      @Nullable Integer tabindex,
      @Nullable String ctrlKeys,
      @Nullable String context,
      @Nullable String popup,
      @Nullable String tooltip,
      String label,
      @Nullable String iconSclass,
      @Nullable String[] iconSclasses,
      @Nullable String iconTooltip,
      @Nullable String[] iconTooltips,
      @Nullable String image,
      @Nullable String hoverImage,
      boolean preloadImage,
      @Nullable String valign,
      @Nullable String align,
      ImmutableList<I> children,
      String widgetClass,
      String sortDirection,
      int maxlength,
      String sortAscendingString,
      String sortDescendingString,
      @Nullable Comparator<?> sortAscending,
      @Nullable Comparator<?> sortDescending) {
    this.id = id;
    this.action = action;
    this.actions = actions;
    this.visible = visible;
    this.mold = mold;
    this.eventListenerMap = eventListenerMap;
    this.widgetListeners = widgetListeners;
    this.widgetOverrides = widgetOverrides;
    this.clientAttributes = clientAttributes;
    this.left = left;
    this.top = top;
    this.zIndex = zIndex;
    this.height = height;
    this.width = width;
    this.tooltiptext = tooltiptext;
    this.zclass = zclass;
    this.sclass = sclass;
    this.style = style;
    this.draggable = draggable;
    this.focus = focus;
    this.droppable = droppable;
    this.vflex = vflex;
    this.hflex = hflex;
    this.renderdefer = renderdefer;
    this.clientAction = clientAction;
    this.tabindex = tabindex;
    this.ctrlKeys = ctrlKeys;
    this.context = context;
    this.popup = popup;
    this.tooltip = tooltip;
    this.label = label;
    this.iconSclass = iconSclass;
    this.iconSclasses = iconSclasses;
    this.iconTooltip = iconTooltip;
    this.iconTooltips = iconTooltips;
    this.image = image;
    this.hoverImage = hoverImage;
    this.preloadImage = preloadImage;
    this.valign = valign;
    this.align = align;
    this.children = children;
    this.widgetClass = widgetClass;
    this.sortDirection = sortDirection;
    this.maxlength = maxlength;
    this.sortAscendingString = sortAscendingString;
    this.sortDescendingString = sortDescendingString;
    this.sortAscending = sortAscending;
    this.sortDescending = sortDescending;
    this.initShim = null;
  }

  private static final byte STAGE_INITIALIZING = -1;
  private static final byte STAGE_UNINITIALIZED = 0;
  private static final byte STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  @Generated(from = "ITreecol", generator = "Immutables")
  private final class InitShim {
    private byte idBuildStage = STAGE_UNINITIALIZED;
    private String id;

    String getId() {
      if (idBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (idBuildStage == STAGE_UNINITIALIZED) {
        idBuildStage = STAGE_INITIALIZING;
        this.id = Objects.requireNonNull(getIdInitialize(), "id");
        idBuildStage = STAGE_INITIALIZED;
      }
      return this.id;
    }

    void setId(String id) {
      this.id = id;
      idBuildStage = STAGE_INITIALIZED;
    }

    private byte visibleBuildStage = STAGE_UNINITIALIZED;
    private boolean visible;

    boolean isVisible() {
      if (visibleBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (visibleBuildStage == STAGE_UNINITIALIZED) {
        visibleBuildStage = STAGE_INITIALIZING;
        this.visible = isVisibleInitialize();
        visibleBuildStage = STAGE_INITIALIZED;
      }
      return this.visible;
    }

    void setVisible(boolean visible) {
      this.visible = visible;
      visibleBuildStage = STAGE_INITIALIZED;
    }

    private byte moldBuildStage = STAGE_UNINITIALIZED;
    private String mold;

    String getMold() {
      if (moldBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (moldBuildStage == STAGE_UNINITIALIZED) {
        moldBuildStage = STAGE_INITIALIZING;
        this.mold = Objects.requireNonNull(getMoldInitialize(), "mold");
        moldBuildStage = STAGE_INITIALIZED;
      }
      return this.mold;
    }

    void setMold(String mold) {
      this.mold = mold;
      moldBuildStage = STAGE_INITIALIZED;
    }

    private byte zIndexBuildStage = STAGE_UNINITIALIZED;
    private int zIndex;

    int getZIndex() {
      if (zIndexBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (zIndexBuildStage == STAGE_UNINITIALIZED) {
        zIndexBuildStage = STAGE_INITIALIZING;
        this.zIndex = getZIndexInitialize();
        zIndexBuildStage = STAGE_INITIALIZED;
      }
      return this.zIndex;
    }

    void setZIndex(int zIndex) {
      this.zIndex = zIndex;
      zIndexBuildStage = STAGE_INITIALIZED;
    }

    private byte draggableBuildStage = STAGE_UNINITIALIZED;
    private String draggable;

    String getDraggable() {
      if (draggableBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (draggableBuildStage == STAGE_UNINITIALIZED) {
        draggableBuildStage = STAGE_INITIALIZING;
        this.draggable = Objects.requireNonNull(getDraggableInitialize(), "draggable");
        draggableBuildStage = STAGE_INITIALIZED;
      }
      return this.draggable;
    }

    void setDraggable(String draggable) {
      this.draggable = draggable;
      draggableBuildStage = STAGE_INITIALIZED;
    }

    private byte focusBuildStage = STAGE_UNINITIALIZED;
    private boolean focus;

    boolean isFocus() {
      if (focusBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (focusBuildStage == STAGE_UNINITIALIZED) {
        focusBuildStage = STAGE_INITIALIZING;
        this.focus = isFocusInitialize();
        focusBuildStage = STAGE_INITIALIZED;
      }
      return this.focus;
    }

    void setFocus(boolean focus) {
      this.focus = focus;
      focusBuildStage = STAGE_INITIALIZED;
    }

    private byte droppableBuildStage = STAGE_UNINITIALIZED;
    private String droppable;

    String getDroppable() {
      if (droppableBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (droppableBuildStage == STAGE_UNINITIALIZED) {
        droppableBuildStage = STAGE_INITIALIZING;
        this.droppable = Objects.requireNonNull(getDroppableInitialize(), "droppable");
        droppableBuildStage = STAGE_INITIALIZED;
      }
      return this.droppable;
    }

    void setDroppable(String droppable) {
      this.droppable = droppable;
      droppableBuildStage = STAGE_INITIALIZED;
    }

    private byte renderdeferBuildStage = STAGE_UNINITIALIZED;
    private int renderdefer;

    int getRenderdefer() {
      if (renderdeferBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (renderdeferBuildStage == STAGE_UNINITIALIZED) {
        renderdeferBuildStage = STAGE_INITIALIZING;
        this.renderdefer = getRenderdeferInitialize();
        renderdeferBuildStage = STAGE_INITIALIZED;
      }
      return this.renderdefer;
    }

    void setRenderdefer(int renderdefer) {
      this.renderdefer = renderdefer;
      renderdeferBuildStage = STAGE_INITIALIZED;
    }

    private byte labelBuildStage = STAGE_UNINITIALIZED;
    private String label;

    String getLabel() {
      if (labelBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (labelBuildStage == STAGE_UNINITIALIZED) {
        labelBuildStage = STAGE_INITIALIZING;
        this.label = Objects.requireNonNull(getLabelInitialize(), "label");
        labelBuildStage = STAGE_INITIALIZED;
      }
      return this.label;
    }

    void setLabel(String label) {
      this.label = label;
      labelBuildStage = STAGE_INITIALIZED;
    }

    private byte iconSclassBuildStage = STAGE_UNINITIALIZED;
    private String iconSclass;

    String getIconSclass() {
      if (iconSclassBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconSclassBuildStage == STAGE_UNINITIALIZED) {
        iconSclassBuildStage = STAGE_INITIALIZING;
        this.iconSclass = getIconSclassInitialize();
        iconSclassBuildStage = STAGE_INITIALIZED;
      }
      return this.iconSclass;
    }

    void setIconSclass(String iconSclass) {
      this.iconSclass = iconSclass;
      iconSclassBuildStage = STAGE_INITIALIZED;
    }

    private byte iconSclassesBuildStage = STAGE_UNINITIALIZED;
    private String[] iconSclasses;

    String[] getIconSclasses() {
      if (iconSclassesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconSclassesBuildStage == STAGE_UNINITIALIZED) {
        iconSclassesBuildStage = STAGE_INITIALIZING;
        this.iconSclasses = getIconSclassesInitialize();
        iconSclassesBuildStage = STAGE_INITIALIZED;
      }
      return this.iconSclasses;
    }

    void setIconSclasses(String[] iconSclasses) {
      this.iconSclasses = iconSclasses;
      iconSclassesBuildStage = STAGE_INITIALIZED;
    }

    private byte iconTooltipBuildStage = STAGE_UNINITIALIZED;
    private String iconTooltip;

    String getIconTooltip() {
      if (iconTooltipBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconTooltipBuildStage == STAGE_UNINITIALIZED) {
        iconTooltipBuildStage = STAGE_INITIALIZING;
        this.iconTooltip = getIconTooltipInitialize();
        iconTooltipBuildStage = STAGE_INITIALIZED;
      }
      return this.iconTooltip;
    }

    void setIconTooltip(String iconTooltip) {
      this.iconTooltip = iconTooltip;
      iconTooltipBuildStage = STAGE_INITIALIZED;
    }

    private byte iconTooltipsBuildStage = STAGE_UNINITIALIZED;
    private String[] iconTooltips;

    String[] getIconTooltips() {
      if (iconTooltipsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (iconTooltipsBuildStage == STAGE_UNINITIALIZED) {
        iconTooltipsBuildStage = STAGE_INITIALIZING;
        this.iconTooltips = getIconTooltipsInitialize();
        iconTooltipsBuildStage = STAGE_INITIALIZED;
      }
      return this.iconTooltips;
    }

    void setIconTooltips(String[] iconTooltips) {
      this.iconTooltips = iconTooltips;
      iconTooltipsBuildStage = STAGE_INITIALIZED;
    }

    private byte imageBuildStage = STAGE_UNINITIALIZED;
    private String image;

    String getImage() {
      if (imageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (imageBuildStage == STAGE_UNINITIALIZED) {
        imageBuildStage = STAGE_INITIALIZING;
        this.image = getImageInitialize();
        imageBuildStage = STAGE_INITIALIZED;
      }
      return this.image;
    }

    void setImage(String image) {
      this.image = image;
      imageBuildStage = STAGE_INITIALIZED;
    }

    private byte hoverImageBuildStage = STAGE_UNINITIALIZED;
    private String hoverImage;

    String getHoverImage() {
      if (hoverImageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (hoverImageBuildStage == STAGE_UNINITIALIZED) {
        hoverImageBuildStage = STAGE_INITIALIZING;
        this.hoverImage = getHoverImageInitialize();
        hoverImageBuildStage = STAGE_INITIALIZED;
      }
      return this.hoverImage;
    }

    void setHoverImage(String hoverImage) {
      this.hoverImage = hoverImage;
      hoverImageBuildStage = STAGE_INITIALIZED;
    }

    private byte preloadImageBuildStage = STAGE_UNINITIALIZED;
    private boolean preloadImage;

    boolean isPreloadImage() {
      if (preloadImageBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (preloadImageBuildStage == STAGE_UNINITIALIZED) {
        preloadImageBuildStage = STAGE_INITIALIZING;
        this.preloadImage = isPreloadImageInitialize();
        preloadImageBuildStage = STAGE_INITIALIZED;
      }
      return this.preloadImage;
    }

    void setPreloadImage(boolean preloadImage) {
      this.preloadImage = preloadImage;
      preloadImageBuildStage = STAGE_INITIALIZED;
    }

    private byte widgetClassBuildStage = STAGE_UNINITIALIZED;
    private String widgetClass;

    String getWidgetClass() {
      if (widgetClassBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (widgetClassBuildStage == STAGE_UNINITIALIZED) {
        widgetClassBuildStage = STAGE_INITIALIZING;
        this.widgetClass = Objects.requireNonNull(getWidgetClassInitialize(), "widgetClass");
        widgetClassBuildStage = STAGE_INITIALIZED;
      }
      return this.widgetClass;
    }

    void setWidgetClass(String widgetClass) {
      this.widgetClass = widgetClass;
      widgetClassBuildStage = STAGE_INITIALIZED;
    }

    private byte sortDirectionBuildStage = STAGE_UNINITIALIZED;
    private String sortDirection;

    String getSortDirection() {
      if (sortDirectionBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (sortDirectionBuildStage == STAGE_UNINITIALIZED) {
        sortDirectionBuildStage = STAGE_INITIALIZING;
        this.sortDirection = Objects.requireNonNull(getSortDirectionInitialize(), "sortDirection");
        sortDirectionBuildStage = STAGE_INITIALIZED;
      }
      return this.sortDirection;
    }

    void setSortDirection(String sortDirection) {
      this.sortDirection = sortDirection;
      sortDirectionBuildStage = STAGE_INITIALIZED;
    }

    private byte maxlengthBuildStage = STAGE_UNINITIALIZED;
    private int maxlength;

    int getMaxlength() {
      if (maxlengthBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (maxlengthBuildStage == STAGE_UNINITIALIZED) {
        maxlengthBuildStage = STAGE_INITIALIZING;
        this.maxlength = getMaxlengthInitialize();
        maxlengthBuildStage = STAGE_INITIALIZED;
      }
      return this.maxlength;
    }

    void setMaxlength(int maxlength) {
      this.maxlength = maxlength;
      maxlengthBuildStage = STAGE_INITIALIZED;
    }

    private byte sortAscendingStringBuildStage = STAGE_UNINITIALIZED;
    private String sortAscendingString;

    String getSortAscendingString() {
      if (sortAscendingStringBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (sortAscendingStringBuildStage == STAGE_UNINITIALIZED) {
        sortAscendingStringBuildStage = STAGE_INITIALIZING;
        this.sortAscendingString = Objects.requireNonNull(getSortAscendingStringInitialize(), "sortAscendingString");
        sortAscendingStringBuildStage = STAGE_INITIALIZED;
      }
      return this.sortAscendingString;
    }

    void setSortAscendingString(String sortAscendingString) {
      this.sortAscendingString = sortAscendingString;
      sortAscendingStringBuildStage = STAGE_INITIALIZED;
    }

    private byte sortDescendingStringBuildStage = STAGE_UNINITIALIZED;
    private String sortDescendingString;

    String getSortDescendingString() {
      if (sortDescendingStringBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (sortDescendingStringBuildStage == STAGE_UNINITIALIZED) {
        sortDescendingStringBuildStage = STAGE_INITIALIZING;
        this.sortDescendingString = Objects.requireNonNull(getSortDescendingStringInitialize(), "sortDescendingString");
        sortDescendingStringBuildStage = STAGE_INITIALIZED;
      }
      return this.sortDescendingString;
    }

    void setSortDescendingString(String sortDescendingString) {
      this.sortDescendingString = sortDescendingString;
      sortDescendingStringBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      List<String> attributes = new ArrayList<>();
      if (idBuildStage == STAGE_INITIALIZING) attributes.add("id");
      if (visibleBuildStage == STAGE_INITIALIZING) attributes.add("visible");
      if (moldBuildStage == STAGE_INITIALIZING) attributes.add("mold");
      if (zIndexBuildStage == STAGE_INITIALIZING) attributes.add("zIndex");
      if (draggableBuildStage == STAGE_INITIALIZING) attributes.add("draggable");
      if (focusBuildStage == STAGE_INITIALIZING) attributes.add("focus");
      if (droppableBuildStage == STAGE_INITIALIZING) attributes.add("droppable");
      if (renderdeferBuildStage == STAGE_INITIALIZING) attributes.add("renderdefer");
      if (labelBuildStage == STAGE_INITIALIZING) attributes.add("label");
      if (iconSclassBuildStage == STAGE_INITIALIZING) attributes.add("iconSclass");
      if (iconSclassesBuildStage == STAGE_INITIALIZING) attributes.add("iconSclasses");
      if (iconTooltipBuildStage == STAGE_INITIALIZING) attributes.add("iconTooltip");
      if (iconTooltipsBuildStage == STAGE_INITIALIZING) attributes.add("iconTooltips");
      if (imageBuildStage == STAGE_INITIALIZING) attributes.add("image");
      if (hoverImageBuildStage == STAGE_INITIALIZING) attributes.add("hoverImage");
      if (preloadImageBuildStage == STAGE_INITIALIZING) attributes.add("preloadImage");
      if (widgetClassBuildStage == STAGE_INITIALIZING) attributes.add("widgetClass");
      if (sortDirectionBuildStage == STAGE_INITIALIZING) attributes.add("sortDirection");
      if (maxlengthBuildStage == STAGE_INITIALIZING) attributes.add("maxlength");
      if (sortAscendingStringBuildStage == STAGE_INITIALIZING) attributes.add("sortAscendingString");
      if (sortDescendingStringBuildStage == STAGE_INITIALIZING) attributes.add("sortDescendingString");
      return "Cannot build ITreecol, attribute initializers form cycle " + attributes;
    }
  }

  private String getIdInitialize() {
    return ITreecol.super.getId();
  }

  private boolean isVisibleInitialize() {
    return ITreecol.super.isVisible();
  }

  private String getMoldInitialize() {
    return ITreecol.super.getMold();
  }

  private int getZIndexInitialize() {
    return ITreecol.super.getZIndex();
  }

  private String getDraggableInitialize() {
    return ITreecol.super.getDraggable();
  }

  private boolean isFocusInitialize() {
    return ITreecol.super.isFocus();
  }

  private String getDroppableInitialize() {
    return ITreecol.super.getDroppable();
  }

  private int getRenderdeferInitialize() {
    return ITreecol.super.getRenderdefer();
  }

  private String getLabelInitialize() {
    return ITreecol.super.getLabel();
  }

  private @Nullable String getIconSclassInitialize() {
    return ITreecol.super.getIconSclass();
  }

  private @Nullable String[] getIconSclassesInitialize() {
    return ITreecol.super.getIconSclasses();
  }

  private @Nullable String getIconTooltipInitialize() {
    return ITreecol.super.getIconTooltip();
  }

  private @Nullable String[] getIconTooltipsInitialize() {
    return ITreecol.super.getIconTooltips();
  }

  private @Nullable String getImageInitialize() {
    return ITreecol.super.getImage();
  }

  private @Nullable String getHoverImageInitialize() {
    return ITreecol.super.getHoverImage();
  }

  private boolean isPreloadImageInitialize() {
    return ITreecol.super.isPreloadImage();
  }

  private String getWidgetClassInitialize() {
    return ITreecol.super.getWidgetClass();
  }

  private String getSortDirectionInitialize() {
    return ITreecol.super.getSortDirection();
  }

  private int getMaxlengthInitialize() {
    return ITreecol.super.getMaxlength();
  }

  private String getSortAscendingStringInitialize() {
    return ITreecol.super.getSortAscendingString();
  }

  private String getSortDescendingStringInitialize() {
    return ITreecol.super.getSortDescendingString();
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @Override
  public String getId() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getId()
        : this.id;
  }

  /**
   * @return The value of the {@code action} attribute
   */
  @StatelessOnly
  @Override
  public @Nullable ActionHandler getAction() {
    return action;
  }

  /**
   * @return The value of the {@code actions} attribute
   */
  @StatelessOnly
  @Override
  public @Nullable ImmutableList<ActionHandler> getActions() {
    return actions;
  }

  /**
   * @return The value of the {@code visible} attribute
   */
  @Override
  public boolean isVisible() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isVisible()
        : this.visible;
  }

  /**
   * @return The value of the {@code mold} attribute
   */
  @Override
  public String getMold() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getMold()
        : this.mold;
  }

  /**
   * @return The value of the {@code eventListenerMap} attribute
   */
  @Override
  public @Nullable EventListenerMap getEventListenerMap() {
    return eventListenerMap;
  }

  /**
   * @return The value of the {@code widgetListeners} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getWidgetListeners() {
    return widgetListeners;
  }

  /**
   * @return The value of the {@code widgetOverrides} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getWidgetOverrides() {
    return widgetOverrides;
  }

  /**
   * @return The value of the {@code clientAttributes} attribute
   */
  @Override
  public @Nullable ImmutableMap<String, String> getClientAttributes() {
    return clientAttributes;
  }

  /**
   * @return The value of the {@code left} attribute
   */
  @Override
  public @Nullable String getLeft() {
    return left;
  }

  /**
   * @return The value of the {@code top} attribute
   */
  @Override
  public @Nullable String getTop() {
    return top;
  }

  /**
   * @return The value of the {@code zIndex} attribute
   */
  @Override
  public int getZIndex() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getZIndex()
        : this.zIndex;
  }

  /**
   * @return The value of the {@code height} attribute
   */
  @Override
  public @Nullable String getHeight() {
    return height;
  }

  /**
   * @return The value of the {@code width} attribute
   */
  @Override
  public @Nullable String getWidth() {
    return width;
  }

  /**
   * @return The value of the {@code tooltiptext} attribute
   */
  @Override
  public @Nullable String getTooltiptext() {
    return tooltiptext;
  }

  /**
   * @return The value of the {@code zclass} attribute
   */
  @Override
  public @Nullable String getZclass() {
    return zclass;
  }

  /**
   * @return The value of the {@code sclass} attribute
   */
  @Override
  public @Nullable String getSclass() {
    return sclass;
  }

  /**
   * @return The value of the {@code style} attribute
   */
  @Override
  public @Nullable String getStyle() {
    return style;
  }

  /**
   * @return The value of the {@code draggable} attribute
   */
  @Override
  public String getDraggable() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDraggable()
        : this.draggable;
  }

  /**
   * @return The value of the {@code focus} attribute
   */
  @Override
  public boolean isFocus() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isFocus()
        : this.focus;
  }

  /**
   * @return The value of the {@code droppable} attribute
   */
  @Override
  public String getDroppable() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDroppable()
        : this.droppable;
  }

  /**
   * @return The value of the {@code vflex} attribute
   */
  @Override
  public @Nullable String getVflex() {
    return vflex;
  }

  /**
   * @return The value of the {@code hflex} attribute
   */
  @Override
  public @Nullable String getHflex() {
    return hflex;
  }

  /**
   * @return The value of the {@code renderdefer} attribute
   */
  @Override
  public int getRenderdefer() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getRenderdefer()
        : this.renderdefer;
  }

  /**
   * @return The value of the {@code clientAction} attribute
   */
  @Override
  public @Nullable String getClientAction() {
    return clientAction;
  }

  /**
   * @return The value of the {@code tabindex} attribute
   */
  @Override
  public @Nullable Integer getTabindex() {
    return tabindex;
  }

  /**
   * @return The value of the {@code ctrlKeys} attribute
   */
  @Override
  public @Nullable String getCtrlKeys() {
    return ctrlKeys;
  }

  /**
   * @return The value of the {@code context} attribute
   */
  @Override
  public @Nullable String getContext() {
    return context;
  }

  /**
   * @return The value of the {@code popup} attribute
   */
  @Override
  public @Nullable String getPopup() {
    return popup;
  }

  /**
   * @return The value of the {@code tooltip} attribute
   */
  @Override
  public @Nullable String getTooltip() {
    return tooltip;
  }

  /**
   * @return The value of the {@code label} attribute
   */
  @Override
  public String getLabel() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getLabel()
        : this.label;
  }

  /**
   * @return The value of the {@code iconSclass} attribute
   */
  @Override
  public @Nullable String getIconSclass() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconSclass()
        : this.iconSclass;
  }

  /**
   * @return A cloned {@code iconSclasses} array
   */
  @Override
  public @Nullable String[] getIconSclasses() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconSclasses()
        : this.iconSclasses;
  }

  /**
   * @return The value of the {@code iconTooltip} attribute
   */
  @Override
  public @Nullable String getIconTooltip() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconTooltip()
        : this.iconTooltip;
  }

  /**
   * @return A cloned {@code iconTooltips} array
   */
  @Override
  public @Nullable String[] getIconTooltips() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getIconTooltips()
        : this.iconTooltips;
  }

  /**
   * @return The value of the {@code image} attribute
   */
  @Override
  public @Nullable String getImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getImage()
        : this.image;
  }

  /**
   * @return The value of the {@code hoverImage} attribute
   */
  @Override
  public @Nullable String getHoverImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getHoverImage()
        : this.hoverImage;
  }

  /**
   * @return The value of the {@code preloadImage} attribute
   */
  @StatelessOnly
  @Override
  public boolean isPreloadImage() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isPreloadImage()
        : this.preloadImage;
  }

  /**
   * @return The value of the {@code valign} attribute
   */
  @Override
  public @Nullable String getValign() {
    return valign;
  }

  /**
   * @return The value of the {@code align} attribute
   */
  @Override
  public @Nullable String getAlign() {
    return align;
  }

  /**
   * @return The value of the {@code children} attribute
   */
  @StatelessOnly
  @Override
  public ImmutableList<I> getChildren() {
    return children;
  }

  /**
   * @return The value of the {@code widgetClass} attribute
   */
  @Override
  public String getWidgetClass() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getWidgetClass()
        : this.widgetClass;
  }

  /**
   * @return The value of the {@code sortDirection} attribute
   */
  @Override
  public String getSortDirection() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getSortDirection()
        : this.sortDirection;
  }

  /**
   * @return The value of the {@code maxlength} attribute
   */
  @Override
  public int getMaxlength() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getMaxlength()
        : this.maxlength;
  }

  /**
   * @return The value of the {@code sortAscendingString} attribute
   */
  @StatelessOnly
  @Override
  public String getSortAscendingString() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getSortAscendingString()
        : this.sortAscendingString;
  }

  /**
   * @return The value of the {@code sortDescendingString} attribute
   */
  @StatelessOnly
  @Override
  public String getSortDescendingString() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getSortDescendingString()
        : this.sortDescendingString;
  }

  /**
   * @return The value of the {@code sortAscending} attribute
   */
  @Override
  public @Nullable Comparator<?> getSortAscending() {
    return sortAscending;
  }

  /**
   * @return The value of the {@code sortDescending} attribute
   */
  @Override
  public @Nullable Comparator<?> getSortDescending() {
    return sortDescending;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getId() id} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for id
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (this.id.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        newValue,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getAction() action} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for action (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withAction(@Nullable ActionHandler value) {
    if (this.action == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        value,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ITreecol#getActions() actions}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITreecol<I> withActions(@Nullable ActionHandler... elements) {
    if (elements == null) {
      return validate(new ImmutableITreecol<>(
          this.id,
          this.action,
          null,
          this.visible,
          this.mold,
          this.eventListenerMap,
          this.widgetListeners,
          this.widgetOverrides,
          this.clientAttributes,
          this.left,
          this.top,
          this.zIndex,
          this.height,
          this.width,
          this.tooltiptext,
          this.zclass,
          this.sclass,
          this.style,
          this.draggable,
          this.focus,
          this.droppable,
          this.vflex,
          this.hflex,
          this.renderdefer,
          this.clientAction,
          this.tabindex,
          this.ctrlKeys,
          this.context,
          this.popup,
          this.tooltip,
          this.label,
          this.iconSclass,
          this.iconSclasses,
          this.iconTooltip,
          this.iconTooltips,
          this.image,
          this.hoverImage,
          this.preloadImage,
          this.valign,
          this.align,
          this.children,
          this.widgetClass,
          this.sortDirection,
          this.maxlength,
          this.sortAscendingString,
          this.sortDescendingString,
          this.sortAscending,
          this.sortDescending));
    }
    @Nullable ImmutableList<ActionHandler> newValue = elements == null ? null : ImmutableList.copyOf(elements);
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        newValue,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ITreecol#getActions() actions}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of actions elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITreecol<I> withActions(@Nullable Iterable<? extends ActionHandler> elements) {
    if (this.actions == elements) return this;
    @Nullable ImmutableList<ActionHandler> newValue = elements == null ? null : ImmutableList.copyOf(elements);
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        newValue,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#isVisible() visible} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for visible
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withVisible(boolean value) {
    if (this.visible == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        value,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getMold() mold} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for mold
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withMold(String value) {
    String newValue = Objects.requireNonNull(value, "mold");
    if (this.mold.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        newValue,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getEventListenerMap() eventListenerMap} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for eventListenerMap (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withEventListenerMap(@Nullable EventListenerMap value) {
    if (this.eventListenerMap == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        value,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by replacing the {@link ITreecol#getWidgetListeners() widgetListeners} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the widgetListeners map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITreecol<I> withWidgetListeners(@Nullable Map<String, ? extends String> entries) {
    if (this.widgetListeners == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        newValue,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by replacing the {@link ITreecol#getWidgetOverrides() widgetOverrides} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the widgetOverrides map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITreecol<I> withWidgetOverrides(@Nullable Map<String, ? extends String> entries) {
    if (this.widgetOverrides == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        newValue,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by replacing the {@link ITreecol#getClientAttributes() clientAttributes} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the clientAttributes map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITreecol<I> withClientAttributes(@Nullable Map<String, ? extends String> entries) {
    if (this.clientAttributes == entries) return this;
    @Nullable ImmutableMap<String, String> newValue = entries == null ? null : ImmutableMap.copyOf(entries);
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        newValue,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getLeft() left} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for left (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withLeft(@Nullable String value) {
    if (Objects.equals(this.left, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        value,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getTop() top} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for top (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withTop(@Nullable String value) {
    if (Objects.equals(this.top, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        value,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getZIndex() zIndex} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for zIndex
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withZIndex(int value) {
    if (this.zIndex == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        value,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getHeight() height} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for height (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withHeight(@Nullable String value) {
    if (Objects.equals(this.height, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        value,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getWidth() width} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for width (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withWidth(@Nullable String value) {
    if (Objects.equals(this.width, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        value,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getTooltiptext() tooltiptext} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tooltiptext (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withTooltiptext(@Nullable String value) {
    if (Objects.equals(this.tooltiptext, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        value,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getZclass() zclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for zclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withZclass(@Nullable String value) {
    if (Objects.equals(this.zclass, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        value,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getSclass() sclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withSclass(@Nullable String value) {
    if (Objects.equals(this.sclass, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        value,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getStyle() style} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for style (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withStyle(@Nullable String value) {
    if (Objects.equals(this.style, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        value,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getDraggable() draggable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for draggable
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withDraggable(String value) {
    String newValue = Objects.requireNonNull(value, "draggable");
    if (this.draggable.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        newValue,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#isFocus() focus} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for focus
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withFocus(boolean value) {
    if (this.focus == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        value,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getDroppable() droppable} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for droppable
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withDroppable(String value) {
    String newValue = Objects.requireNonNull(value, "droppable");
    if (this.droppable.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        newValue,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getVflex() vflex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for vflex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withVflex(@Nullable String value) {
    if (Objects.equals(this.vflex, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        value,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getHflex() hflex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for hflex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withHflex(@Nullable String value) {
    if (Objects.equals(this.hflex, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        value,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getRenderdefer() renderdefer} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for renderdefer
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withRenderdefer(int value) {
    if (this.renderdefer == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        value,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getClientAction() clientAction} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for clientAction (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withClientAction(@Nullable String value) {
    if (Objects.equals(this.clientAction, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        value,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getTabindex() tabindex} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tabindex (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withTabindex(@Nullable Integer value) {
    if (Objects.equals(this.tabindex, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        value,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getCtrlKeys() ctrlKeys} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for ctrlKeys (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withCtrlKeys(@Nullable String value) {
    if (Objects.equals(this.ctrlKeys, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        value,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getContext() context} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for context (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withContext(@Nullable String value) {
    if (Objects.equals(this.context, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        value,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getPopup() popup} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for popup (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withPopup(@Nullable String value) {
    if (Objects.equals(this.popup, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        value,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getTooltip() tooltip} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for tooltip (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withTooltip(@Nullable String value) {
    if (Objects.equals(this.tooltip, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        value,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getLabel() label} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for label
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withLabel(String value) {
    String newValue = Objects.requireNonNull(value, "label");
    if (this.label.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        newValue,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getIconSclass() iconSclass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for iconSclass (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withIconSclass(@Nullable String value) {
    if (Objects.equals(this.iconSclass, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        value,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ITreecol#getIconSclasses() iconSclasses}.
   * The array is cloned before being saved as attribute values.
   * @param elements The non-null elements for iconSclasses
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITreecol<I> withIconSclasses(@Nullable String... elements) {
    @Nullable String[] newValue = elements == null ? null : elements.clone();
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        newValue,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getIconTooltip() iconTooltip} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for iconTooltip (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withIconTooltip(@Nullable String value) {
    if (Objects.equals(this.iconTooltip, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        value,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ITreecol#getIconTooltips() iconTooltips}.
   * The array is cloned before being saved as attribute values.
   * @param elements The non-null elements for iconTooltips
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITreecol<I> withIconTooltips(@Nullable String... elements) {
    @Nullable String[] newValue = elements == null ? null : elements.clone();
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        newValue,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getImage() image} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for image (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withImage(@Nullable String value) {
    if (Objects.equals(this.image, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        value,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getHoverImage() hoverImage} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for hoverImage (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withHoverImage(@Nullable String value) {
    if (Objects.equals(this.hoverImage, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        value,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#isPreloadImage() preloadImage} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for preloadImage
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withPreloadImage(boolean value) {
    if (this.preloadImage == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        value,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getValign() valign} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for valign (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withValign(@Nullable String value) {
    if (Objects.equals(this.valign, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        value,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getAlign() align} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for align (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withAlign(@Nullable String value) {
    if (Objects.equals(this.align, value)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        value,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ITreecol#getChildren() children}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  @SafeVarargs @SuppressWarnings("varargs")
  public final ImmutableITreecol<I> withChildren(I... elements) {
    ImmutableList<I> newValue = ImmutableList.copyOf(elements);
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        newValue,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ITreecol#getChildren() children}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of children elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableITreecol<I> withChildren(Iterable<? extends I> elements) {
    if (this.children == elements) return this;
    ImmutableList<I> newValue = ImmutableList.copyOf(elements);
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        newValue,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getWidgetClass() widgetClass} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for widgetClass
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withWidgetClass(String value) {
    String newValue = Objects.requireNonNull(value, "widgetClass");
    if (this.widgetClass.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        newValue,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getSortDirection() sortDirection} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sortDirection
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withSortDirection(String value) {
    String newValue = Objects.requireNonNull(value, "sortDirection");
    if (this.sortDirection.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        newValue,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getMaxlength() maxlength} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for maxlength
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withMaxlength(int value) {
    if (this.maxlength == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        value,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getSortAscendingString() sortAscendingString} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sortAscendingString
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withSortAscendingString(String value) {
    String newValue = Objects.requireNonNull(value, "sortAscendingString");
    if (this.sortAscendingString.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        newValue,
        this.sortDescendingString,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getSortDescendingString() sortDescendingString} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sortDescendingString
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withSortDescendingString(String value) {
    String newValue = Objects.requireNonNull(value, "sortDescendingString");
    if (this.sortDescendingString.equals(newValue)) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        newValue,
        this.sortAscending,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getSortAscending() sortAscending} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sortAscending (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withSortAscending(@Nullable Comparator<?> value) {
    if (this.sortAscending == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        value,
        this.sortDescending));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ITreecol#getSortDescending() sortDescending} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for sortDescending (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableITreecol<I> withSortDescending(@Nullable Comparator<?> value) {
    if (this.sortDescending == value) return this;
    return validate(new ImmutableITreecol<>(
        this.id,
        this.action,
        this.actions,
        this.visible,
        this.mold,
        this.eventListenerMap,
        this.widgetListeners,
        this.widgetOverrides,
        this.clientAttributes,
        this.left,
        this.top,
        this.zIndex,
        this.height,
        this.width,
        this.tooltiptext,
        this.zclass,
        this.sclass,
        this.style,
        this.draggable,
        this.focus,
        this.droppable,
        this.vflex,
        this.hflex,
        this.renderdefer,
        this.clientAction,
        this.tabindex,
        this.ctrlKeys,
        this.context,
        this.popup,
        this.tooltip,
        this.label,
        this.iconSclass,
        this.iconSclasses,
        this.iconTooltip,
        this.iconTooltips,
        this.image,
        this.hoverImage,
        this.preloadImage,
        this.valign,
        this.align,
        this.children,
        this.widgetClass,
        this.sortDirection,
        this.maxlength,
        this.sortAscendingString,
        this.sortDescendingString,
        this.sortAscending,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableITreecol} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableITreecol<?>
        && equalTo((ImmutableITreecol<?>) another);
  }

  private boolean equalTo(ImmutableITreecol<?> another) {
    return id.equals(another.id)
        && Objects.equals(action, another.action)
        && Objects.equals(actions, another.actions)
        && visible == another.visible
        && mold.equals(another.mold)
        && Objects.equals(eventListenerMap, another.eventListenerMap)
        && Objects.equals(widgetListeners, another.widgetListeners)
        && Objects.equals(widgetOverrides, another.widgetOverrides)
        && Objects.equals(clientAttributes, another.clientAttributes)
        && Objects.equals(left, another.left)
        && Objects.equals(top, another.top)
        && zIndex == another.zIndex
        && Objects.equals(height, another.height)
        && Objects.equals(width, another.width)
        && Objects.equals(tooltiptext, another.tooltiptext)
        && Objects.equals(zclass, another.zclass)
        && Objects.equals(sclass, another.sclass)
        && Objects.equals(style, another.style)
        && draggable.equals(another.draggable)
        && focus == another.focus
        && droppable.equals(another.droppable)
        && Objects.equals(vflex, another.vflex)
        && Objects.equals(hflex, another.hflex)
        && renderdefer == another.renderdefer
        && Objects.equals(clientAction, another.clientAction)
        && Objects.equals(tabindex, another.tabindex)
        && Objects.equals(ctrlKeys, another.ctrlKeys)
        && Objects.equals(context, another.context)
        && Objects.equals(popup, another.popup)
        && Objects.equals(tooltip, another.tooltip)
        && label.equals(another.label)
        && Objects.equals(iconSclass, another.iconSclass)
        && Arrays.equals(iconSclasses, another.iconSclasses)
        && Objects.equals(iconTooltip, another.iconTooltip)
        && Arrays.equals(iconTooltips, another.iconTooltips)
        && Objects.equals(image, another.image)
        && Objects.equals(hoverImage, another.hoverImage)
        && preloadImage == another.preloadImage
        && Objects.equals(valign, another.valign)
        && Objects.equals(align, another.align)
        && children.equals(another.children)
        && widgetClass.equals(another.widgetClass)
        && sortDirection.equals(another.sortDirection)
        && maxlength == another.maxlength
        && sortAscendingString.equals(another.sortAscendingString)
        && sortDescendingString.equals(another.sortDescendingString)
        && Objects.equals(sortAscending, another.sortAscending)
        && Objects.equals(sortDescending, another.sortDescending);
  }

  /**
   * Computes a hash code from attributes: {@code id}, {@code action}, {@code actions}, {@code visible}, {@code mold}, {@code eventListenerMap}, {@code widgetListeners}, {@code widgetOverrides}, {@code clientAttributes}, {@code left}, {@code top}, {@code zIndex}, {@code height}, {@code width}, {@code tooltiptext}, {@code zclass}, {@code sclass}, {@code style}, {@code draggable}, {@code focus}, {@code droppable}, {@code vflex}, {@code hflex}, {@code renderdefer}, {@code clientAction}, {@code tabindex}, {@code ctrlKeys}, {@code context}, {@code popup}, {@code tooltip}, {@code label}, {@code iconSclass}, {@code iconSclasses}, {@code iconTooltip}, {@code iconTooltips}, {@code image}, {@code hoverImage}, {@code preloadImage}, {@code valign}, {@code align}, {@code children}, {@code widgetClass}, {@code sortDirection}, {@code maxlength}, {@code sortAscendingString}, {@code sortDescendingString}, {@code sortAscending}, {@code sortDescending}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + id.hashCode();
    h += (h << 5) + Objects.hashCode(action);
    h += (h << 5) + Objects.hashCode(actions);
    h += (h << 5) + Booleans.hashCode(visible);
    h += (h << 5) + mold.hashCode();
    h += (h << 5) + Objects.hashCode(eventListenerMap);
    h += (h << 5) + Objects.hashCode(widgetListeners);
    h += (h << 5) + Objects.hashCode(widgetOverrides);
    h += (h << 5) + Objects.hashCode(clientAttributes);
    h += (h << 5) + Objects.hashCode(left);
    h += (h << 5) + Objects.hashCode(top);
    h += (h << 5) + zIndex;
    h += (h << 5) + Objects.hashCode(height);
    h += (h << 5) + Objects.hashCode(width);
    h += (h << 5) + Objects.hashCode(tooltiptext);
    h += (h << 5) + Objects.hashCode(zclass);
    h += (h << 5) + Objects.hashCode(sclass);
    h += (h << 5) + Objects.hashCode(style);
    h += (h << 5) + draggable.hashCode();
    h += (h << 5) + Booleans.hashCode(focus);
    h += (h << 5) + droppable.hashCode();
    h += (h << 5) + Objects.hashCode(vflex);
    h += (h << 5) + Objects.hashCode(hflex);
    h += (h << 5) + renderdefer;
    h += (h << 5) + Objects.hashCode(clientAction);
    h += (h << 5) + Objects.hashCode(tabindex);
    h += (h << 5) + Objects.hashCode(ctrlKeys);
    h += (h << 5) + Objects.hashCode(context);
    h += (h << 5) + Objects.hashCode(popup);
    h += (h << 5) + Objects.hashCode(tooltip);
    h += (h << 5) + label.hashCode();
    h += (h << 5) + Objects.hashCode(iconSclass);
    h += (h << 5) + Arrays.hashCode(iconSclasses);
    h += (h << 5) + Objects.hashCode(iconTooltip);
    h += (h << 5) + Arrays.hashCode(iconTooltips);
    h += (h << 5) + Objects.hashCode(image);
    h += (h << 5) + Objects.hashCode(hoverImage);
    h += (h << 5) + Booleans.hashCode(preloadImage);
    h += (h << 5) + Objects.hashCode(valign);
    h += (h << 5) + Objects.hashCode(align);
    h += (h << 5) + children.hashCode();
    h += (h << 5) + widgetClass.hashCode();
    h += (h << 5) + sortDirection.hashCode();
    h += (h << 5) + maxlength;
    h += (h << 5) + sortAscendingString.hashCode();
    h += (h << 5) + sortDescendingString.hashCode();
    h += (h << 5) + Objects.hashCode(sortAscending);
    h += (h << 5) + Objects.hashCode(sortDescending);
    return h;
  }

  /**
   * Prints the immutable value {@code ITreecol} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("ITreecol")
        .omitNullValues()
        .add("id", id)
        .add("action", action)
        .add("actions", actions)
        .add("visible", visible)
        .add("mold", mold)
        .add("eventListenerMap", eventListenerMap)
        .add("widgetListeners", widgetListeners)
        .add("widgetOverrides", widgetOverrides)
        .add("clientAttributes", clientAttributes)
        .add("left", left)
        .add("top", top)
        .add("zIndex", zIndex)
        .add("height", height)
        .add("width", width)
        .add("tooltiptext", tooltiptext)
        .add("zclass", zclass)
        .add("sclass", sclass)
        .add("style", style)
        .add("draggable", draggable)
        .add("focus", focus)
        .add("droppable", droppable)
        .add("vflex", vflex)
        .add("hflex", hflex)
        .add("renderdefer", renderdefer)
        .add("clientAction", clientAction)
        .add("tabindex", tabindex)
        .add("ctrlKeys", ctrlKeys)
        .add("context", context)
        .add("popup", popup)
        .add("tooltip", tooltip)
        .add("label", label)
        .add("iconSclass", iconSclass)
        .add("iconSclasses", Arrays.toString(iconSclasses))
        .add("iconTooltip", iconTooltip)
        .add("iconTooltips", Arrays.toString(iconTooltips))
        .add("image", image)
        .add("hoverImage", hoverImage)
        .add("preloadImage", preloadImage)
        .add("valign", valign)
        .add("align", align)
        .add("children", children)
        .add("widgetClass", widgetClass)
        .add("sortDirection", sortDirection)
        .add("maxlength", maxlength)
        .add("sortAscendingString", sortAscendingString)
        .add("sortDescendingString", sortDescendingString)
        .add("sortAscending", sortAscending)
        .add("sortDescending", sortDescending)
        .toString();
  }

  private transient volatile long lazyInitBitmap;

  private static final long Z_K_TYPE_LAZY_INIT_BIT = 0x1L;

  private transient Class<Treecol> zKType;

  /**
   * {@inheritDoc}
   * <p>
   * Returns a lazily initialized value of the {@link ITreecol#getZKType() zKType} attribute.
   * Initialized once and only once and stored for subsequent access with proper synchronization.
   * In case of any exception or error thrown by the lazy value initializer,
   * the result will not be memoised (i.e. remembered) and on next call computation
   * will be attempted again.
   * @return A lazily initialized value of the {@code zKType} attribute
   */
  @Override
  public Class<Treecol> getZKType() {
    if ((lazyInitBitmap & Z_K_TYPE_LAZY_INIT_BIT) == 0) {
      synchronized (this) {
        if ((lazyInitBitmap & Z_K_TYPE_LAZY_INIT_BIT) == 0) {
          this.zKType = Objects.requireNonNull(ITreecol.super.getZKType(), "zKType");
          lazyInitBitmap |= Z_K_TYPE_LAZY_INIT_BIT;
        }
      }
    }
    return zKType;
  }

  private static <I extends IAnyGroup> ImmutableITreecol<I> validate(ImmutableITreecol<I> instance) {
    instance.checkSortDirection();
    instance = (ImmutableITreecol<I>) instance.checkSortAscendingAndDescending();
    instance.checkHflexAndWidth();
    instance.checkVflexAndHeight();
    instance = (ImmutableITreecol<I>) instance.checkDroppable();
    instance = (ImmutableITreecol<I>) instance.checkDraggable();
    instance.checkActions();
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link ITreecol} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param <I> generic parameter I
   * @param instance The instance to copy
   * @return A copied immutable ITreecol instance
   */
  public static <I extends IAnyGroup> ITreecol<I> copyOf(ITreecol<I> instance) {
    if (instance instanceof ImmutableITreecol<?>) {
      return (ImmutableITreecol<I>) instance;
    }
    return new ITreecol.Builder<I>()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ITreecol ITreecol}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "ITreecol", generator = "Immutables")
  @NotThreadSafe
  public static class Builder<I extends IAnyGroup> {
    private static final long OPT_BIT_VISIBLE = 0x1L;
    private static final long OPT_BIT_Z_INDEX = 0x2L;
    private static final long OPT_BIT_FOCUS = 0x4L;
    private static final long OPT_BIT_RENDERDEFER = 0x8L;
    private static final long OPT_BIT_ICON_SCLASS = 0x10L;
    private static final long OPT_BIT_ICON_SCLASSES = 0x20L;
    private static final long OPT_BIT_ICON_TOOLTIP = 0x40L;
    private static final long OPT_BIT_ICON_TOOLTIPS = 0x80L;
    private static final long OPT_BIT_IMAGE = 0x100L;
    private static final long OPT_BIT_HOVER_IMAGE = 0x200L;
    private static final long OPT_BIT_PRELOAD_IMAGE = 0x400L;
    private static final long OPT_BIT_MAXLENGTH = 0x800L;
    private long optBits;

    private @Nullable String id;
    private @Nullable ActionHandler action;
    private ImmutableList.Builder<ActionHandler> actions = null;
    private boolean visible;
    private @Nullable String mold;
    private @Nullable EventListenerMap eventListenerMap;
    private ImmutableMap.Builder<String, String> widgetListeners = null;
    private ImmutableMap.Builder<String, String> widgetOverrides = null;
    private ImmutableMap.Builder<String, String> clientAttributes = null;
    private @Nullable String left;
    private @Nullable String top;
    private int zIndex;
    private @Nullable String height;
    private @Nullable String width;
    private @Nullable String tooltiptext;
    private @Nullable String zclass;
    private @Nullable String sclass;
    private @Nullable String style;
    private @Nullable String draggable;
    private boolean focus;
    private @Nullable String droppable;
    private @Nullable String vflex;
    private @Nullable String hflex;
    private int renderdefer;
    private @Nullable String clientAction;
    private @Nullable Integer tabindex;
    private @Nullable String ctrlKeys;
    private @Nullable String context;
    private @Nullable String popup;
    private @Nullable String tooltip;
    private @Nullable String label;
    private @Nullable String iconSclass;
    private @Nullable String[] iconSclasses;
    private @Nullable String iconTooltip;
    private @Nullable String[] iconTooltips;
    private @Nullable String image;
    private @Nullable String hoverImage;
    private boolean preloadImage;
    private @Nullable String valign;
    private @Nullable String align;
    private ImmutableList.Builder<I> children = ImmutableList.builder();
    private @Nullable String widgetClass;
    private @Nullable String sortDirection;
    private int maxlength;
    private @Nullable String sortAscendingString;
    private @Nullable String sortDescendingString;
    private @Nullable Comparator<?> sortAscending;
    private @Nullable Comparator<?> sortDescending;

    /**
     * Creates a builder for {@link ITreecol ITreecol} instances.
     * <pre>
     * new ITreecol.Builder&amp;lt;I&amp;gt;()
     *    .setId(String) // optional {@link ITreecol#getId() id}
     *    .setAction(org.zkoss.stateless.util.ActionHandler | null) // nullable {@link ITreecol#getAction() action}
     *    .setActions(List&amp;lt;org.zkoss.stateless.util.ActionHandler&amp;gt; | null) // nullable {@link ITreecol#getActions() actions}
     *    .setVisible(boolean) // optional {@link ITreecol#isVisible() visible}
     *    .setMold(String) // optional {@link ITreecol#getMold() mold}
     *    .setEventListenerMap(org.zkoss.zk.ui.sys.EventListenerMap | null) // nullable {@link ITreecol#getEventListenerMap() eventListenerMap}
     *    .setWidgetListeners(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link ITreecol#getWidgetListeners() widgetListeners}
     *    .setWidgetOverrides(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link ITreecol#getWidgetOverrides() widgetOverrides}
     *    .setClientAttributes(Map&amp;lt;String, String&amp;gt; | null) // nullable {@link ITreecol#getClientAttributes() clientAttributes}
     *    .setLeft(String | null) // nullable {@link ITreecol#getLeft() left}
     *    .setTop(String | null) // nullable {@link ITreecol#getTop() top}
     *    .setZIndex(int) // optional {@link ITreecol#getZIndex() zIndex}
     *    .setHeight(String | null) // nullable {@link ITreecol#getHeight() height}
     *    .setWidth(String | null) // nullable {@link ITreecol#getWidth() width}
     *    .setTooltiptext(String | null) // nullable {@link ITreecol#getTooltiptext() tooltiptext}
     *    .setZclass(String | null) // nullable {@link ITreecol#getZclass() zclass}
     *    .setSclass(String | null) // nullable {@link ITreecol#getSclass() sclass}
     *    .setStyle(String | null) // nullable {@link ITreecol#getStyle() style}
     *    .setDraggable(String) // optional {@link ITreecol#getDraggable() draggable}
     *    .setFocus(boolean) // optional {@link ITreecol#isFocus() focus}
     *    .setDroppable(String) // optional {@link ITreecol#getDroppable() droppable}
     *    .setVflex(String | null) // nullable {@link ITreecol#getVflex() vflex}
     *    .setHflex(String | null) // nullable {@link ITreecol#getHflex() hflex}
     *    .setRenderdefer(int) // optional {@link ITreecol#getRenderdefer() renderdefer}
     *    .setClientAction(String | null) // nullable {@link ITreecol#getClientAction() clientAction}
     *    .setTabindex(Integer | null) // nullable {@link ITreecol#getTabindex() tabindex}
     *    .setCtrlKeys(String | null) // nullable {@link ITreecol#getCtrlKeys() ctrlKeys}
     *    .setContext(String | null) // nullable {@link ITreecol#getContext() context}
     *    .setPopup(String | null) // nullable {@link ITreecol#getPopup() popup}
     *    .setTooltip(String | null) // nullable {@link ITreecol#getTooltip() tooltip}
     *    .setLabel(String) // optional {@link ITreecol#getLabel() label}
     *    .setIconSclass(String | null) // nullable {@link ITreecol#getIconSclass() iconSclass}
     *    .setIconSclasses(String[] | null) // nullable {@link ITreecol#getIconSclasses() iconSclasses}
     *    .setIconTooltip(String | null) // nullable {@link ITreecol#getIconTooltip() iconTooltip}
     *    .setIconTooltips(String[] | null) // nullable {@link ITreecol#getIconTooltips() iconTooltips}
     *    .setImage(String | null) // nullable {@link ITreecol#getImage() image}
     *    .setHoverImage(String | null) // nullable {@link ITreecol#getHoverImage() hoverImage}
     *    .setPreloadImage(boolean) // optional {@link ITreecol#isPreloadImage() preloadImage}
     *    .setValign(String | null) // nullable {@link ITreecol#getValign() valign}
     *    .setAlign(String | null) // nullable {@link ITreecol#getAlign() align}
     *    .addChildren|addAllChildren(I) // {@link ITreecol#getChildren() children} elements
     *    .setWidgetClass(String) // optional {@link ITreecol#getWidgetClass() widgetClass}
     *    .setSortDirection(String) // optional {@link ITreecol#getSortDirection() sortDirection}
     *    .setMaxlength(int) // optional {@link ITreecol#getMaxlength() maxlength}
     *    .setSortAscendingString(String) // optional {@link ITreecol#getSortAscendingString() sortAscendingString}
     *    .setSortDescendingString(String) // optional {@link ITreecol#getSortDescendingString() sortDescendingString}
     *    .setSortAscending(Comparator&amp;lt;?&amp;gt; | null) // nullable {@link ITreecol#getSortAscending() sortAscending}
     *    .setSortDescending(Comparator&amp;lt;?&amp;gt; | null) // nullable {@link ITreecol#getSortDescending() sortDescending}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof ITreecol.Builder<?>)) {
        throw new UnsupportedOperationException("Use: new ITreecol.Builder<I>()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code ITreecol} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> from(ITreecol<I> instance) {
      Objects.requireNonNull(instance, "instance");
      setId(instance.getId());
      @Nullable ActionHandler actionValue = instance.getAction();
      if (actionValue != null) {
        setAction(actionValue);
      }
      @Nullable List<ActionHandler> actionsValue = instance.getActions();
      if (actionsValue != null) {
        addAllActions(actionsValue);
      }
      setVisible(instance.isVisible());
      setMold(instance.getMold());
      @Nullable EventListenerMap eventListenerMapValue = instance.getEventListenerMap();
      if (eventListenerMapValue != null) {
        setEventListenerMap(eventListenerMapValue);
      }
      @Nullable Map<String, String> widgetListenersValue = instance.getWidgetListeners();
      if (widgetListenersValue != null) {
        putAllWidgetListeners(widgetListenersValue);
      }
      @Nullable Map<String, String> widgetOverridesValue = instance.getWidgetOverrides();
      if (widgetOverridesValue != null) {
        putAllWidgetOverrides(widgetOverridesValue);
      }
      @Nullable Map<String, String> clientAttributesValue = instance.getClientAttributes();
      if (clientAttributesValue != null) {
        putAllClientAttributes(clientAttributesValue);
      }
      @Nullable String leftValue = instance.getLeft();
      if (leftValue != null) {
        setLeft(leftValue);
      }
      @Nullable String topValue = instance.getTop();
      if (topValue != null) {
        setTop(topValue);
      }
      setZIndex(instance.getZIndex());
      @Nullable String heightValue = instance.getHeight();
      if (heightValue != null) {
        setHeight(heightValue);
      }
      @Nullable String widthValue = instance.getWidth();
      if (widthValue != null) {
        setWidth(widthValue);
      }
      @Nullable String tooltiptextValue = instance.getTooltiptext();
      if (tooltiptextValue != null) {
        setTooltiptext(tooltiptextValue);
      }
      @Nullable String zclassValue = instance.getZclass();
      if (zclassValue != null) {
        setZclass(zclassValue);
      }
      @Nullable String sclassValue = instance.getSclass();
      if (sclassValue != null) {
        setSclass(sclassValue);
      }
      @Nullable String styleValue = instance.getStyle();
      if (styleValue != null) {
        setStyle(styleValue);
      }
      setDraggable(instance.getDraggable());
      setFocus(instance.isFocus());
      setDroppable(instance.getDroppable());
      @Nullable String vflexValue = instance.getVflex();
      if (vflexValue != null) {
        setVflex(vflexValue);
      }
      @Nullable String hflexValue = instance.getHflex();
      if (hflexValue != null) {
        setHflex(hflexValue);
      }
      setRenderdefer(instance.getRenderdefer());
      @Nullable String clientActionValue = instance.getClientAction();
      if (clientActionValue != null) {
        setClientAction(clientActionValue);
      }
      @Nullable Integer tabindexValue = instance.getTabindex();
      if (tabindexValue != null) {
        setTabindex(tabindexValue);
      }
      @Nullable String ctrlKeysValue = instance.getCtrlKeys();
      if (ctrlKeysValue != null) {
        setCtrlKeys(ctrlKeysValue);
      }
      @Nullable String contextValue = instance.getContext();
      if (contextValue != null) {
        setContext(contextValue);
      }
      @Nullable String popupValue = instance.getPopup();
      if (popupValue != null) {
        setPopup(popupValue);
      }
      @Nullable String tooltipValue = instance.getTooltip();
      if (tooltipValue != null) {
        setTooltip(tooltipValue);
      }
      setLabel(instance.getLabel());
      @Nullable String iconSclassValue = instance.getIconSclass();
      if (iconSclassValue != null) {
        setIconSclass(iconSclassValue);
      }
      @Nullable String[] iconSclassesValue = instance.getIconSclasses();
      if (iconSclassesValue != null) {
        setIconSclasses(iconSclassesValue);
      }
      @Nullable String iconTooltipValue = instance.getIconTooltip();
      if (iconTooltipValue != null) {
        setIconTooltip(iconTooltipValue);
      }
      @Nullable String[] iconTooltipsValue = instance.getIconTooltips();
      if (iconTooltipsValue != null) {
        setIconTooltips(iconTooltipsValue);
      }
      @Nullable String imageValue = instance.getImage();
      if (imageValue != null) {
        setImage(imageValue);
      }
      @Nullable String hoverImageValue = instance.getHoverImage();
      if (hoverImageValue != null) {
        setHoverImage(hoverImageValue);
      }
      setPreloadImage(instance.isPreloadImage());
      @Nullable String valignValue = instance.getValign();
      if (valignValue != null) {
        setValign(valignValue);
      }
      @Nullable String alignValue = instance.getAlign();
      if (alignValue != null) {
        setAlign(alignValue);
      }
      addAllChildren(instance.getChildren());
      setWidgetClass(instance.getWidgetClass());
      setSortDirection(instance.getSortDirection());
      setMaxlength(instance.getMaxlength());
      setSortAscendingString(instance.getSortAscendingString());
      setSortDescendingString(instance.getSortDescendingString());
      @Nullable Comparator<?> sortAscendingValue = instance.getSortAscending();
      if (sortAscendingValue != null) {
        setSortAscending(sortAscendingValue);
      }
      @Nullable Comparator<?> sortDescendingValue = instance.getSortDescending();
      if (sortDescendingValue != null) {
        setSortDescending(sortDescendingValue);
      }
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getId() id} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getId() id}.</em>
     * @param id The value for id 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setId(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getAction() action} attribute.
     * @param action The value for action (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setAction(@Nullable ActionHandler action) {
      this.action = action;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Adds one element to {@link ITreecol#getActions() actions} list.
     * @param element A actions element
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> addActions(ActionHandler element) {
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.add(element);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Adds elements to {@link ITreecol#getActions() actions} list.
     * @param elements An array of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> addActions(ActionHandler... elements) {
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.add(elements);
      return (ITreecol.Builder<I>) this;
    }


    /**
     * Sets or replaces all elements for {@link ITreecol#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setActions(@Nullable Iterable<? extends ActionHandler> elements) {
      if (elements == null) {
        this.actions = null;
        return (ITreecol.Builder<I>) this;
      }
      this.actions = ImmutableList.builder();
      return addAllActions(elements);
    }

    /**
     * Adds elements to {@link ITreecol#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> addAllActions(Iterable<? extends ActionHandler> elements) {
      Objects.requireNonNull(elements, "actions element");
      if (this.actions == null) {
        this.actions = ImmutableList.builder();
      }
      this.actions.addAll(elements);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#isVisible() visible} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#isVisible() visible}.</em>
     * @param visible The value for visible 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setVisible(boolean visible) {
      this.visible = visible;
      optBits |= OPT_BIT_VISIBLE;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getMold() mold} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getMold() mold}.</em>
     * @param mold The value for mold 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setMold(String mold) {
      this.mold = Objects.requireNonNull(mold, "mold");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getEventListenerMap() eventListenerMap} attribute.
     * @param eventListenerMap The value for eventListenerMap (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setEventListenerMap(@Nullable EventListenerMap eventListenerMap) {
      this.eventListenerMap = eventListenerMap;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link ITreecol#getWidgetListeners() widgetListeners} map.
     * @param key The key in the widgetListeners map
     * @param value The associated value in the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putWidgetListeners(String key, String value) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.put(key, value);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link ITreecol#getWidgetListeners() widgetListeners} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putWidgetListeners(Map.Entry<String, ? extends String> entry) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.put(entry);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ITreecol#getWidgetListeners() widgetListeners} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setWidgetListeners(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.widgetListeners = null;
        return (ITreecol.Builder<I>) this;
      }
      this.widgetListeners = ImmutableMap.builder();
      return putAllWidgetListeners(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ITreecol#getWidgetListeners() widgetListeners} map. Nulls are not permitted
     * @param entries The entries that will be added to the widgetListeners map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putAllWidgetListeners(Map<String, ? extends String> entries) {
      if (this.widgetListeners == null) {
        this.widgetListeners = ImmutableMap.builder();
      }
      this.widgetListeners.putAll(entries);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link ITreecol#getWidgetOverrides() widgetOverrides} map.
     * @param key The key in the widgetOverrides map
     * @param value The associated value in the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putWidgetOverrides(String key, String value) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.put(key, value);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link ITreecol#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putWidgetOverrides(Map.Entry<String, ? extends String> entry) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.put(entry);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ITreecol#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setWidgetOverrides(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.widgetOverrides = null;
        return (ITreecol.Builder<I>) this;
      }
      this.widgetOverrides = ImmutableMap.builder();
      return putAllWidgetOverrides(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ITreecol#getWidgetOverrides() widgetOverrides} map. Nulls are not permitted
     * @param entries The entries that will be added to the widgetOverrides map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putAllWidgetOverrides(Map<String, ? extends String> entries) {
      if (this.widgetOverrides == null) {
        this.widgetOverrides = ImmutableMap.builder();
      }
      this.widgetOverrides.putAll(entries);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link ITreecol#getClientAttributes() clientAttributes} map.
     * @param key The key in the clientAttributes map
     * @param value The associated value in the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putClientAttributes(String key, String value) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.put(key, value);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Put one entry to the {@link ITreecol#getClientAttributes() clientAttributes} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putClientAttributes(Map.Entry<String, ? extends String> entry) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.put(entry);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ITreecol#getClientAttributes() clientAttributes} map. Nulls are not permitted as keys or values, but parameter itself can be null
     * @param entries The entries that will be added to the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setClientAttributes(@Nullable Map<String, ? extends String> entries) {
      if (entries == null) {
        this.clientAttributes = null;
        return (ITreecol.Builder<I>) this;
      }
      this.clientAttributes = ImmutableMap.builder();
      return putAllClientAttributes(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ITreecol#getClientAttributes() clientAttributes} map. Nulls are not permitted
     * @param entries The entries that will be added to the clientAttributes map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> putAllClientAttributes(Map<String, ? extends String> entries) {
      if (this.clientAttributes == null) {
        this.clientAttributes = ImmutableMap.builder();
      }
      this.clientAttributes.putAll(entries);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getLeft() left} attribute.
     * @param left The value for left (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setLeft(@Nullable String left) {
      this.left = left;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getTop() top} attribute.
     * @param top The value for top (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setTop(@Nullable String top) {
      this.top = top;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getZIndex() zIndex} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getZIndex() zIndex}.</em>
     * @param zIndex The value for zIndex 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setZIndex(int zIndex) {
      this.zIndex = zIndex;
      optBits |= OPT_BIT_Z_INDEX;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getHeight() height} attribute.
     * @param height The value for height (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setHeight(@Nullable String height) {
      this.height = height;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getWidth() width} attribute.
     * @param width The value for width (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setWidth(@Nullable String width) {
      this.width = width;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getTooltiptext() tooltiptext} attribute.
     * @param tooltiptext The value for tooltiptext (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setTooltiptext(@Nullable String tooltiptext) {
      this.tooltiptext = tooltiptext;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getZclass() zclass} attribute.
     * @param zclass The value for zclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setZclass(@Nullable String zclass) {
      this.zclass = zclass;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getSclass() sclass} attribute.
     * @param sclass The value for sclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setSclass(@Nullable String sclass) {
      this.sclass = sclass;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getStyle() style} attribute.
     * @param style The value for style (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setStyle(@Nullable String style) {
      this.style = style;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getDraggable() draggable} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getDraggable() draggable}.</em>
     * @param draggable The value for draggable 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setDraggable(String draggable) {
      this.draggable = Objects.requireNonNull(draggable, "draggable");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#isFocus() focus} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#isFocus() focus}.</em>
     * @param focus The value for focus 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setFocus(boolean focus) {
      this.focus = focus;
      optBits |= OPT_BIT_FOCUS;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getDroppable() droppable} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getDroppable() droppable}.</em>
     * @param droppable The value for droppable 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setDroppable(String droppable) {
      this.droppable = Objects.requireNonNull(droppable, "droppable");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getVflex() vflex} attribute.
     * @param vflex The value for vflex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setVflex(@Nullable String vflex) {
      this.vflex = vflex;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getHflex() hflex} attribute.
     * @param hflex The value for hflex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setHflex(@Nullable String hflex) {
      this.hflex = hflex;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getRenderdefer() renderdefer} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getRenderdefer() renderdefer}.</em>
     * @param renderdefer The value for renderdefer 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setRenderdefer(int renderdefer) {
      this.renderdefer = renderdefer;
      optBits |= OPT_BIT_RENDERDEFER;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getClientAction() clientAction} attribute.
     * @param clientAction The value for clientAction (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setClientAction(@Nullable String clientAction) {
      this.clientAction = clientAction;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getTabindex() tabindex} attribute.
     * @param tabindex The value for tabindex (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setTabindex(@Nullable Integer tabindex) {
      this.tabindex = tabindex;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getCtrlKeys() ctrlKeys} attribute.
     * @param ctrlKeys The value for ctrlKeys (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setCtrlKeys(@Nullable String ctrlKeys) {
      this.ctrlKeys = ctrlKeys;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getContext() context} attribute.
     * @param context The value for context (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setContext(@Nullable String context) {
      this.context = context;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getPopup() popup} attribute.
     * @param popup The value for popup (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setPopup(@Nullable String popup) {
      this.popup = popup;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getTooltip() tooltip} attribute.
     * @param tooltip The value for tooltip (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setTooltip(@Nullable String tooltip) {
      this.tooltip = tooltip;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getLabel() label} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getLabel() label}.</em>
     * @param label The value for label 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setLabel(String label) {
      this.label = Objects.requireNonNull(label, "label");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getIconSclass() iconSclass} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getIconSclass() iconSclass}.</em>
     * @param iconSclass The value for iconSclass (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setIconSclass(@Nullable String iconSclass) {
      this.iconSclass = iconSclass;
      optBits |= OPT_BIT_ICON_SCLASS;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getIconSclasses() iconSclasses} attribute.
     * <p><em>If not set, this attribute will have a default value as defined by {@link ITreecol#getIconSclasses() iconSclasses}.</em>
     * @param iconSclasses The elements for iconSclasses
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setIconSclasses(String... iconSclasses) {
      this.iconSclasses = iconSclasses;
      optBits |= OPT_BIT_ICON_SCLASSES;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getIconTooltip() iconTooltip} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getIconTooltip() iconTooltip}.</em>
     * @param iconTooltip The value for iconTooltip (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setIconTooltip(@Nullable String iconTooltip) {
      this.iconTooltip = iconTooltip;
      optBits |= OPT_BIT_ICON_TOOLTIP;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getIconTooltips() iconTooltips} attribute.
     * <p><em>If not set, this attribute will have a default value as defined by {@link ITreecol#getIconTooltips() iconTooltips}.</em>
     * @param iconTooltips The elements for iconTooltips
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setIconTooltips(String... iconTooltips) {
      this.iconTooltips = iconTooltips;
      optBits |= OPT_BIT_ICON_TOOLTIPS;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getImage() image} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getImage() image}.</em>
     * @param image The value for image (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setImage(@Nullable String image) {
      this.image = image;
      optBits |= OPT_BIT_IMAGE;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getHoverImage() hoverImage} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getHoverImage() hoverImage}.</em>
     * @param hoverImage The value for hoverImage (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setHoverImage(@Nullable String hoverImage) {
      this.hoverImage = hoverImage;
      optBits |= OPT_BIT_HOVER_IMAGE;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#isPreloadImage() preloadImage} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#isPreloadImage() preloadImage}.</em>
     * @param preloadImage The value for preloadImage 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setPreloadImage(boolean preloadImage) {
      this.preloadImage = preloadImage;
      optBits |= OPT_BIT_PRELOAD_IMAGE;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getValign() valign} attribute.
     * @param valign The value for valign (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setValign(@Nullable String valign) {
      this.valign = valign;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getAlign() align} attribute.
     * @param align The value for align (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setAlign(@Nullable String align) {
      this.align = align;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Adds one element to {@link ITreecol#getChildren() children} list.
     * @param element A children element
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> addChildren(I element) {
      this.children.add(element);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Adds elements to {@link ITreecol#getChildren() children} list.
     * @param elements An array of children elements
     * @return {@code this} builder for use in a chained invocation
     */
    @SafeVarargs @SuppressWarnings("varargs")
    public final ITreecol.Builder<I> addChildren(I... elements) {
      this.children.add(elements);
      return (ITreecol.Builder<I>) this;
    }


    /**
     * Sets or replaces all elements for {@link ITreecol#getChildren() children} list.
     * @param elements An iterable of children elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setChildren(Iterable<? extends I> elements) {
      this.children = ImmutableList.builder();
      return addAllChildren(elements);
    }

    /**
     * Adds elements to {@link ITreecol#getChildren() children} list.
     * @param elements An iterable of children elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> addAllChildren(Iterable<? extends I> elements) {
      this.children.addAll(elements);
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getWidgetClass() widgetClass} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getWidgetClass() widgetClass}.</em>
     * @param widgetClass The value for widgetClass 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setWidgetClass(String widgetClass) {
      this.widgetClass = Objects.requireNonNull(widgetClass, "widgetClass");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getSortDirection() sortDirection} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getSortDirection() sortDirection}.</em>
     * @param sortDirection The value for sortDirection 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setSortDirection(String sortDirection) {
      this.sortDirection = Objects.requireNonNull(sortDirection, "sortDirection");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getMaxlength() maxlength} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getMaxlength() maxlength}.</em>
     * @param maxlength The value for maxlength 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setMaxlength(int maxlength) {
      this.maxlength = maxlength;
      optBits |= OPT_BIT_MAXLENGTH;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getSortAscendingString() sortAscendingString} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getSortAscendingString() sortAscendingString}.</em>
     * @param sortAscendingString The value for sortAscendingString 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setSortAscendingString(String sortAscendingString) {
      this.sortAscendingString = Objects.requireNonNull(sortAscendingString, "sortAscendingString");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getSortDescendingString() sortDescendingString} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link ITreecol#getSortDescendingString() sortDescendingString}.</em>
     * @param sortDescendingString The value for sortDescendingString 
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setSortDescendingString(String sortDescendingString) {
      this.sortDescendingString = Objects.requireNonNull(sortDescendingString, "sortDescendingString");
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getSortAscending() sortAscending} attribute.
     * @param sortAscending The value for sortAscending (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setSortAscending(@Nullable Comparator<?> sortAscending) {
      this.sortAscending = sortAscending;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Initializes the value for the {@link ITreecol#getSortDescending() sortDescending} attribute.
     * @param sortDescending The value for sortDescending (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    public final ITreecol.Builder<I> setSortDescending(@Nullable Comparator<?> sortDescending) {
      this.sortDescending = sortDescending;
      return (ITreecol.Builder<I>) this;
    }

    /**
     * Builds a new {@link ITreecol ITreecol}.
     * @return An immutable instance of ITreecol
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ITreecol<I> build() {
      return ImmutableITreecol.validate(new ImmutableITreecol<I>(this));
    }

    private boolean visibleIsSet() {
      return (optBits & OPT_BIT_VISIBLE) != 0;
    }

    private boolean zIndexIsSet() {
      return (optBits & OPT_BIT_Z_INDEX) != 0;
    }

    private boolean focusIsSet() {
      return (optBits & OPT_BIT_FOCUS) != 0;
    }

    private boolean renderdeferIsSet() {
      return (optBits & OPT_BIT_RENDERDEFER) != 0;
    }

    private boolean iconSclassIsSet() {
      return (optBits & OPT_BIT_ICON_SCLASS) != 0;
    }

    private boolean iconSclassesIsSet() {
      return (optBits & OPT_BIT_ICON_SCLASSES) != 0;
    }

    private boolean iconTooltipIsSet() {
      return (optBits & OPT_BIT_ICON_TOOLTIP) != 0;
    }

    private boolean iconTooltipsIsSet() {
      return (optBits & OPT_BIT_ICON_TOOLTIPS) != 0;
    }

    private boolean imageIsSet() {
      return (optBits & OPT_BIT_IMAGE) != 0;
    }

    private boolean hoverImageIsSet() {
      return (optBits & OPT_BIT_HOVER_IMAGE) != 0;
    }

    private boolean preloadImageIsSet() {
      return (optBits & OPT_BIT_PRELOAD_IMAGE) != 0;
    }

    private boolean maxlengthIsSet() {
      return (optBits & OPT_BIT_MAXLENGTH) != 0;
    }
  }
}
